@@ -123,7 +123,11 @@ chipsets = {
     'CNL': gen8_11_chipset_params,
     'ICL': gen8_11_chipset_params,
     'EHL': gen8_11_chipset_params,
-    'TGL': gen8_11_chipset_params,
+    'TGLGT1': gen8_11_chipset_params,
+    'TGLGT2': gen8_11_chipset_params,
+    'RKL': gen8_11_chipset_params,
+    'DG1': gen8_11_chipset_params,
+    'ADL': gen8_11_chipset_params,
 }
 
 register_types = { 'OA', 'NOA', 'FLEX', 'PM' }
@@ -133,7 +137,6 @@ default_set_blacklist = { "RenderDX1x", # TODO: rename to something non 'DX'
                                         # usefull
                           "RenderBalance", # XXX: missing register config
                           "PipelineTimestamps", # Covered by API timestamp queries
-                          "AsyncCompute",
                         }
 
 counter_blacklist = {
@@ -164,8 +167,8 @@ def print_err(*args):
     sys.stderr.write(' '.join(map(str,args)) + '\n')
 
 read_register_offsets = {
-    0x1f0: 'PERFCNT1',
-    0x1f8: 'PERFCNT2',
+    0x1f0: 'PERFCNT 0',
+    0x1f8: 'PERFCNT 1',
 }
 
 def read_value(chipset, offset):
@@ -207,7 +210,7 @@ def read_token_to_rpn_read(chipset, token, raw_offsets):
         elif offset < report_size:
             return "C " + str(int((offset - c_offset) / 4)) + " READ"
         else:
-            return "{0} READ_REG".format(read_value(chipset, offset))
+            return "{0} READ".format(read_value(chipset, offset))
     else:
         idx = int(offset / 8)
         if chipset == "HSW":
@@ -223,7 +226,7 @@ def read_token_to_rpn_read(chipset, token, raw_offsets):
             elif idx < 62:
                 return "C " + str(idx - 54) + " READ"
             else:
-                return "{0} READ_REG".format(read_value(chipset, offset))
+                return "{0} READ".format(read_value(chipset, offset))
         else:
             # For Gen8+ the array of accumulated counters is
             # assumed to start with a GPU_TIME then GPU_CLOCK,
@@ -240,7 +243,7 @@ def read_token_to_rpn_read(chipset, token, raw_offsets):
             elif idx < 54:
                 return "C " + str(idx - 46) + " READ"
             else:
-                return "{0} READ_REG".format(read_value(chipset, offset))
+                return "{0} READ".format(read_value(chipset, offset))
 
     assert 0
 
@@ -484,8 +487,7 @@ def process_mux_configs(mdapi_set):
         mux_config_id_groups = [mux_config_id_groups[0]]
 
     if len(mux_config_id_groups) == 0 or mux_config_id_groups[0]['configs'] == 0:
-        print_err("ERROR: MUX register configs missing: MetricSet=\"" + mdapi_set.get('ShortName'))
-        sys.exit(1)
+        return ()
 
     mux_configs = mux_config_id_groups[0]['configs']
     assert isinstance(mux_configs, list)
