@@ -58,6 +58,7 @@
 #include <glib.h>
 
 #include "drmtest.h"
+#include "intel_allocator.h"
 #include "intel_chipset.h"
 #include "intel_io.h"
 #include "igt_debugfs.h"
@@ -72,6 +73,7 @@
 #include "igt_rc.h"
 #include "igt_list.h"
 #include "igt_device_scan.h"
+#include "igt_thread.h"
 
 #define UNW_LOCAL_ONLY
 #include <libunwind.h>
@@ -249,7 +251,7 @@
  *	FrameDumpPath=/tmp # The path to dump frames that fail comparison checks
  *
  *	&num; Device selection filter
- *	Device=pci:vendor=8086,card=0;vgem:
+ *	Device=pci:vendor=8086,card=0;sys:/sys/devices/platform/vgem
  *
  *	&num; The following section is used for configuring the Device Under Test.
  *	&num; It is not mandatory and allows overriding default values.
@@ -305,6 +307,10 @@ int num_test_children;
 int test_children_sz;
 bool test_child;
 
+/* For allocator purposes */
+pid_t child_pid  = -1;
+__thread pid_t child_tid  = -1;
+
 enum {
 	/*
 	 * Let the first values be used by individual tests so options don't
@@ -320,6 +326,7 @@ enum {
 	OPT_SKIP_CRC,
 	OPT_TRACE_OOPS,
 	OPT_DEVICE,
+	OPT_VERSION,
 	OPT_HELP = 'h'
 };
 
@@ -342,6 +349,21 @@ static bool stderr_needs_sentinel = false;
 
 static int _igt_dynamic_tests_executed = -1;
 
+__attribute__((format(printf, 2, 3)))
+static void internal_assert(bool cond, const char *format, ...)
+{
+	if (!cond) {
+		va_list ap;
+
+		va_start(ap, format);
+		vfprintf(stderr, format, ap);
+		va_end(ap);
+		fprintf(stderr, "please refer to lib/igt_core documentation\n");
+
+		assert(0);
+	}
+}
+
 const char *igt_test_name(void)
 {
 	return command_str;
@@ -539,8 +561,12 @@ uint64_t igt_nsec_elapsed(struct timespec *start)
 
 bool __igt_fixture(void)
 {
-	assert(!in_fixture);
-	assert(test_with_subtests);
+	internal_assert(!in_fixture,
+			"nesting multiple igt_fixtures is invalid\n");
+	internal_assert(!in_subtest,
+			"nesting igt_fixture in igt_subtest is invalid\n");
+	internal_assert(test_with_subtests,
+			"igt_fixture in igt_simple_main is invalid\n");
 
 	if (igt_only_list_subtests())
 		return false;
@@ -590,6 +616,7 @@ static void ftrace_dump_on_oops(bool enable)
 }
 
 bool igt_exit_called;
+bool igt_is_aborting;
 static void common_exit_handler(int sig)
 {
 	if (!igt_only_list_subtests()) {
@@ -598,7 +625,7 @@ static void common_exit_handler(int sig)
 
 	/* When not killed by a signal check that igt_exit() has been properly
 	 * called. */
-	assert(sig != 0 || igt_exit_called);
+	assert(sig != 0 || igt_exit_called || igt_is_aborting);
 }
 
 static void print_line_wrapping(const char *indent, const char *text)
@@ -681,9 +708,11 @@ static void print_usage(const char *help_str, bool output_on_stderr)
 		   "  --debug[=log-domain]\n"
 		   "  --interactive-debug[=domain]\n"
 		   "  --skip-crc-compare\n"
+		   "  --trace-on-oops\n"
 		   "  --help-description\n"
 		   "  --describe\n"
-		   "  --device filter\n"
+		   "  --device filters\n"
+		   "  --version\n"
 		   "  --help|-h\n");
 	if (help_str)
 		fprintf(f, "%s\n", help_str);
@@ -752,21 +781,20 @@ GKeyFile *igt_load_igtrc(void)
 static void common_init_config(void)
 {
 	GError *error = NULL;
-	int ret;
+	int ret = 0;
 
 	igt_key_file = igt_load_igtrc();
-	if (!igt_key_file)
-		return;
 
-	if (!igt_frame_dump_path)
+	if (igt_key_file && !igt_frame_dump_path)
 		igt_frame_dump_path =
 			g_key_file_get_string(igt_key_file, "Common",
 					      "FrameDumpPath", &error);
 
 	g_clear_error(&error);
 
-	ret = g_key_file_get_integer(igt_key_file, "DUT", "SuspendResumeDelay",
-				     &error);
+	if (igt_key_file)
+		ret = g_key_file_get_integer(igt_key_file, "DUT", "SuspendResumeDelay",
+					     &error);
 	assert(!error || error->code != G_KEY_FILE_ERROR_INVALID_VALUE);
 
 	g_clear_error(&error);
@@ -775,29 +803,30 @@ static void common_init_config(void)
 		igt_set_autoresume_delay(ret);
 
 	/* Adding filters, order .igtrc, IGT_DEVICE, --device filter */
-	if (igt_device_is_filter_set())
+	if (igt_device_filter_count() > 0)
 		igt_debug("Notice: using --device filters:\n");
 	else {
 		if (igt_rc_device) {
 			igt_debug("Notice: using IGT_DEVICE env:\n");
 		} else {
-			igt_rc_device =	g_key_file_get_string(igt_key_file,
-							      "Common",
-							      "Device", &error);
+			if (igt_key_file)
+				igt_rc_device =	g_key_file_get_string(igt_key_file,
+								      "Common",
+								      "Device", &error);
 			g_clear_error(&error);
 			if (igt_rc_device)
 				igt_debug("Notice: using .igtrc "
 					  "Common::Device:\n");
 		}
 		if (igt_rc_device) {
-			igt_device_filter_set(igt_rc_device);
+			igt_device_filter_add(igt_rc_device);
 			free(igt_rc_device);
 			igt_rc_device = NULL;
 		}
 	}
 
-	if (igt_device_is_filter_set())
-		igt_debug("[%s]\n", igt_device_filter_get());
+	for (int i = 0; i < igt_device_filter_count(); i++)
+		igt_debug("[%s]\n", igt_device_filter_get(i));
 }
 
 static void common_init_env(void)
@@ -858,6 +887,7 @@ static int common_init(int *argc, char **argv,
 		{"skip-crc-compare",  no_argument,       NULL, OPT_SKIP_CRC},
 		{"trace-on-oops",     no_argument,       NULL, OPT_TRACE_OOPS},
 		{"device",            required_argument, NULL, OPT_DEVICE},
+		{"version",           no_argument,       NULL, OPT_VERSION},
 		{"help",              no_argument,       NULL, OPT_HELP},
 		{0, 0, 0, 0}
 	};
@@ -984,10 +1014,10 @@ static int common_init(int *argc, char **argv,
 			goto out;
 		case OPT_SKIP_CRC:
 			igt_skip_crc_compare = true;
-			goto out;
+			break;
 		case OPT_TRACE_OOPS:
 			show_ftrace = true;
-			goto out;
+			break;
 		case OPT_DEVICE:
 			assert(optarg);
 			/* if set by env IGT_DEVICE we need to free it */
@@ -995,8 +1025,12 @@ static int common_init(int *argc, char **argv,
 				free(igt_rc_device);
 				igt_rc_device = NULL;
 			}
-			igt_device_filter_set(optarg);
+			igt_device_filter_add(optarg);
 			break;
+		case OPT_VERSION:
+			print_version();
+			ret = -1;
+			goto out;
 		case OPT_HELP:
 			print_usage(help_str, false);
 			ret = -1;
@@ -1194,7 +1228,9 @@ static bool valid_name_for_subtest(const char *subtest_name)
  */
 bool __igt_run_subtest(const char *subtest_name, const char *file, const int line)
 {
-	assert(!igt_can_fail());
+	internal_assert(!igt_can_fail(),
+			"igt_subtest can be nested only in igt_main"
+			" or igt_subtest_group\n");
 
 	if (!valid_name_for_subtest(subtest_name)) {
 		igt_critical("Invalid subtest name \"%s\".\n",
@@ -1242,6 +1278,7 @@ bool __igt_run_subtest(const char *subtest_name, const char *file, const int lin
 		fprintf(stderr, "Starting subtest: %s\n", subtest_name);
 
 	_igt_log_buffer_reset();
+	igt_thread_clear_fail_state();
 
 	igt_gettime(&subtest_time);
 	return (in_subtest = subtest_name);
@@ -1249,8 +1286,10 @@ bool __igt_run_subtest(const char *subtest_name, const char *file, const int lin
 
 bool __igt_run_dynamic_subtest(const char *dynamic_subtest_name)
 {
-	assert(in_subtest);
-	assert(_igt_dynamic_tests_executed >= 0);
+	internal_assert(in_subtest && _igt_dynamic_tests_executed >= 0,
+			"igt_dynamic is allowed only inside igt_subtest_with_dynamic\n");
+	internal_assert(!in_dynamic_subtest,
+			"igt_dynamic is not allowed to be nested in another igt_dynamic\n");
 
 	if (!valid_name_for_subtest(dynamic_subtest_name)) {
 			igt_critical("Invalid dynamic subtest name \"%s\".\n",
@@ -1270,6 +1309,7 @@ bool __igt_run_dynamic_subtest(const char *dynamic_subtest_name)
 		fprintf(stderr, "Starting dynamic subtest: %s\n", dynamic_subtest_name);
 
 	_igt_log_buffer_reset();
+	igt_thread_clear_fail_state();
 
 	_igt_dynamic_tests_executed++;
 
@@ -1303,7 +1343,8 @@ bool igt_only_list_subtests(void)
 
 void __igt_subtest_group_save(int *save, int *desc)
 {
-	assert(test_with_subtests);
+	internal_assert(test_with_subtests,
+			"igt_subtest_group is not allowed in igt_simple_main\n");
 
 	if (__current_description[0] != '\0') {
 		struct description_node *new = calloc(1, sizeof(*new));
@@ -1341,8 +1382,7 @@ bool __igt_enter_dynamic_container(void)
 	return true;
 }
 
-static void exit_subtest(const char *) __attribute__((noreturn));
-static void exit_subtest(const char *result)
+__noreturn static void exit_subtest(const char *result)
 {
 	struct timespec now;
 	const char *subtest_text = in_dynamic_subtest ? "Dynamic subtest" : "Subtest";
@@ -1367,11 +1407,26 @@ static void exit_subtest(const char *result)
 
 	/* If the subtest aborted, it may have left children behind */
 	for (int c = 0; c < num_test_children; c++) {
-		kill(test_children[c], SIGKILL);
-		waitpid(test_children[c], NULL, 0); /* don't leave zombies! */
+		if (test_children[c] > 0) {
+			kill(test_children[c], SIGKILL);
+			waitpid(test_children[c], NULL, 0); /* don't leave zombies! */
+		}
 	}
 	num_test_children = 0;
 
+	/*
+	 * When test completes - mostly in fail state it can leave allocated
+	 * objects. An allocator is not an exception as it is global IGT
+	 * entity and when test will allocate some ranges and then it will
+	 * fail no free/close likely will be called (controling potential
+	 * fails and clearing before assertions in IGT is not common).
+	 *
+	 * We call intel_allocator_init() then to prepare the allocator
+	 * infrastructure from scratch for each test. Init also removes
+	 * remnants from previous allocator run (if any).
+	 */
+	intel_allocator_init();
+
 	if (!in_dynamic_subtest)
 		_igt_dynamic_tests_executed = -1;
 
@@ -1406,7 +1461,8 @@ void igt_skip(const char *f, ...)
 	va_list args;
 	skipped_one = true;
 
-	assert(!test_child);
+	internal_assert(!test_child,
+			"skips are not allowed in forks\n");
 
 	if (!igt_only_list_subtests()) {
 		va_start(args, f);
@@ -1419,7 +1475,9 @@ void igt_skip(const char *f, ...)
 		exit_subtest("SKIP");
 	} else if (test_with_subtests) {
 		skip_subtests_henceforth = SKIP;
-		assert(in_fixture);
+		internal_assert(in_fixture,
+			"skipping is allowed only in fixtures, subtests"
+			" or igt_simple_main\n");
 		__igt_fixture_end();
 	} else {
 		igt_exitcode = IGT_EXIT_SKIP;
@@ -1435,6 +1493,9 @@ void __igt_skip_check(const char *file, const int line,
 	int err = errno;
 	char *err_str = NULL;
 
+	if (!igt_thread_is_main())
+		assert(!"igt_require/skip allowed only in the main thread!");
+
 	if (err)
 		igt_assert_neq(asprintf(&err_str, "Last errno: %i, %s\n", err, strerror(err)),
 			       -1);
@@ -1472,6 +1533,8 @@ void __igt_skip_check(const char *file, const int line,
  */
 void igt_success(void)
 {
+	igt_thread_assert_no_failures();
+
 	if (in_subtest && !in_dynamic_subtest && _igt_dynamic_tests_executed >= 0) {
 		/*
 		 * We're exiting a dynamic container, yield a result
@@ -1511,6 +1574,11 @@ void igt_fail(int exitcode)
 {
 	assert(exitcode != IGT_EXIT_SUCCESS && exitcode != IGT_EXIT_SKIP);
 
+	if (!igt_thread_is_main()) {
+		igt_thread_fail();
+		pthread_exit(NULL);
+	}
+
 	igt_debug_wait_for_keypress("failure");
 
 	/* Exit immediately if the test is already exiting and igt_fail is
@@ -1539,7 +1607,8 @@ void igt_fail(int exitcode)
 	if (in_subtest) {
 		exit_subtest("FAIL");
 	} else {
-		assert(igt_can_fail());
+		internal_assert(igt_can_fail(), "failing test is only allowed"
+				" in fixtures, subtests and igt_simple_main");
 
 		if (in_fixture) {
 			skip_subtests_henceforth = FAIL;
@@ -1559,7 +1628,7 @@ void igt_fail(int exitcode)
  * Since out test runner (piglit) does support fatal test exit codes, we
  * implement the default behaviour by waiting endlessly.
  */
-void  __attribute__((noreturn)) igt_fatal_error(void)
+void igt_fatal_error(void)
 {
 	if (igt_check_boolean_env_var("IGT_REBOOT_ON_FATAL_ERROR", false)) {
 		igt_warn("FATAL ERROR - REBOOTING\n");
@@ -1604,8 +1673,9 @@ void igt_describe_f(const char *fmt, ...)
 	int ret;
 	va_list args;
 
-	if (in_subtest && _igt_dynamic_tests_executed >= 0)
-		assert(!"Documenting dynamic subsubtests is impossible. Document the subtest instead.");
+	internal_assert(!in_subtest || _igt_dynamic_tests_executed < 0,
+			"documenting dynamic subsubtests is impossible,"
+			" document the subtest instead.");
 
 	if (!describe_subtests)
 		return;
@@ -1947,6 +2017,50 @@ void __igt_fail_assert(const char *domain, const char *file, const int line,
 	igt_fail(IGT_EXIT_FAILURE);
 }
 
+static void kill_children(void)
+{
+	for (int c = 0; c < num_test_children; c++) {
+		if (test_children[c] > 0)
+			kill(test_children[c], SIGKILL);
+	}
+}
+
+void __igt_abort(const char *domain, const char *file, const int line,
+		 const char *func, const char *expression,
+		 const char *f, ...)
+{
+	va_list args;
+	int err = errno;
+
+	igt_is_aborting = true;
+
+	igt_log(domain, IGT_LOG_CRITICAL,
+		"Test abort in function %s, file %s:%i:\n", func, file,
+		line);
+	igt_log(domain, IGT_LOG_CRITICAL, "abort condition: %s\n", expression);
+	if (err)
+		igt_log(domain, IGT_LOG_CRITICAL, "Last errno: %i, %s\n", err,
+			strerror(err));
+
+	if (f) {
+		va_start(args, f);
+		igt_vlog(domain, IGT_LOG_CRITICAL, f, args);
+		va_end(args);
+	}
+
+	/* just try our best, we are aborting the execution anyway */
+	kill_children();
+
+	print_backtrace();
+
+	if (running_under_gdb())
+		abort();
+
+	_igt_log_buffer_dump();
+
+	exit(IGT_EXIT_ABORT);
+}
+
 /**
  * igt_exit:
  *
@@ -1965,6 +2079,9 @@ void igt_exit(void)
 {
 	int tmp;
 
+	if (!test_with_subtests)
+		igt_thread_assert_no_failures();
+
 	igt_exit_called = true;
 
 	if (igt_key_file)
@@ -1996,8 +2113,7 @@ void igt_exit(void)
 			 command_str, igt_exitcode);
 	igt_debug("Exiting with status code %d\n", igt_exitcode);
 
-	for (int c = 0; c < num_test_children; c++)
-		kill(test_children[c], SIGKILL);
+	kill_children();
 	assert(!num_test_children);
 
 	assert(waitpid(-1, &tmp, WNOHANG) == -1 && errno == ECHILD);
@@ -2173,10 +2289,14 @@ static void children_exit_handler(int sig)
 		;
 }
 
+static pthread_mutex_t print_mutex = PTHREAD_MUTEX_INITIALIZER;
+
 bool __igt_fork(void)
 {
-	assert(!test_with_subtests || in_subtest);
-	assert(!test_child);
+	internal_assert(!test_with_subtests || in_subtest,
+			"forking is only allowed in subtests or igt_simple_main\n");
+	internal_assert(!test_child,
+			"forking is not allowed from already forked children\n");
 
 	igt_install_exit_handler(children_exit_handler);
 
@@ -2196,9 +2316,13 @@ bool __igt_fork(void)
 
 	switch (test_children[num_test_children++] = fork()) {
 	case -1:
+		num_test_children--; /* so we won't kill(-1) during cleanup */
 		igt_assert(0);
 	case 0:
 		test_child = true;
+		pthread_mutex_init(&print_mutex, NULL);
+		child_pid = getpid();
+		child_tid = -1;
 		exit_handler_count = 0;
 		reset_helper_process_list();
 		oom_adjust_for_doom();
@@ -2255,8 +2379,7 @@ int __igt_waitchildren(void)
 				err = 256;
 			}
 
-			for (c = 0; c < num_test_children; c++)
-				kill(test_children[c], SIGKILL);
+			kill_children();
 		}
 
 		count++;
@@ -2290,8 +2413,7 @@ static void igt_alarm_killchildren(int signal)
 {
 	igt_info("Timed out waiting for children\n");
 
-	for (int c = 0; c < num_test_children; c++)
-		kill(test_children[c], SIGKILL);
+	kill_children();
 }
 
 /**
@@ -2604,6 +2726,12 @@ void igt_log(const char *domain, enum igt_log_level level, const char *format, .
 	va_end(args);
 }
 
+static pthread_key_t __vlog_line_continuation;
+
+igt_constructor {
+	pthread_key_create(&__vlog_line_continuation, NULL);
+}
+
 /**
  * igt_vlog:
  * @domain: the log domain, or NULL for no domain
@@ -2622,6 +2750,7 @@ void igt_vlog(const char *domain, enum igt_log_level level, const char *format,
 {
 	FILE *file;
 	char *line, *formatted_line;
+	char *thread_id;
 	const char *program_name;
 	const char *igt_log_level_str[] = {
 		"DEBUG",
@@ -2630,7 +2759,6 @@ void igt_vlog(const char *domain, enum igt_log_level level, const char *format,
 		"CRITICAL",
 		"NONE"
 	};
-	static bool line_continuation = false;
 
 	assert(format);
 
@@ -2640,23 +2768,37 @@ void igt_vlog(const char *domain, enum igt_log_level level, const char *format,
 	program_name = command_str;
 #endif
 
+
+	if (igt_thread_is_main()) {
+		thread_id = strdup("");
+	} else {
+		if (asprintf(&thread_id, "[thread:%d] ", gettid()) == -1)
+			thread_id = NULL;
+	}
+
+	if (!thread_id)
+		return;
+
 	if (list_subtests && level <= IGT_LOG_WARN)
 		return;
 
 	if (vasprintf(&line, format, args) == -1)
 		return;
 
-	if (line_continuation) {
+	if (pthread_getspecific(__vlog_line_continuation)) {
 		formatted_line = strdup(line);
 		if (!formatted_line)
 			goto out;
-	} else if (asprintf(&formatted_line, "(%s:%d) %s%s%s: %s", program_name,
-		     getpid(), (domain) ? domain : "", (domain) ? "-" : "",
+	} else if (asprintf(&formatted_line, "(%s:%d) %s%s%s%s: %s", program_name,
+		     getpid(), thread_id, (domain) ? domain : "", (domain) ? "-" : "",
 		     igt_log_level_str[level], line) == -1) {
 		goto out;
 	}
 
-	line_continuation = line[strlen(line) - 1] != '\n';
+	if (line[strlen(line) - 1] == '\n')
+		pthread_setspecific(__vlog_line_continuation, (void*) false);
+	else
+		pthread_setspecific(__vlog_line_continuation, (void*) true);
 
 	/* append log buffer */
 	_igt_log_buffer_append(formatted_line);
@@ -2675,6 +2817,8 @@ void igt_vlog(const char *domain, enum igt_log_level level, const char *format,
 			goto out;
 	}
 
+	pthread_mutex_lock(&print_mutex);
+
 	/* use stderr for warning messages and above */
 	if (level >= IGT_LOG_WARN) {
 		file = stderr;
@@ -2685,18 +2829,23 @@ void igt_vlog(const char *domain, enum igt_log_level level, const char *format,
 
 	/* prepend all except information messages with process, domain and log
 	 * level information */
-	if (level != IGT_LOG_INFO)
+	if (level != IGT_LOG_INFO) {
 		fwrite(formatted_line, sizeof(char), strlen(formatted_line),
 		       file);
-	else
+	} else {
+		fwrite(thread_id, sizeof(char), strlen(thread_id), file);
 		fwrite(line, sizeof(char), strlen(line), file);
+	}
+
+	pthread_mutex_unlock(&print_mutex);
 
 out:
 	free(line);
+	free(thread_id);
 }
 
 static const char *timeout_op;
-static void __attribute__((noreturn)) igt_alarm_handler(int signal)
+__noreturn static void igt_alarm_handler(int signal)
 {
 	if (timeout_op)
 		igt_info("Timed out: %s\n", timeout_op);
