@@ -23,6 +23,9 @@
  */
 
 #include <sys/ioctl.h>
+#include <cairo.h>
+
+#include "i915/gem.h"
 #include "igt.h"
 #include "igt_x86.h"
 #include "intel_bufops.h"
@@ -52,7 +55,7 @@
  * struct intel_buf ibuf;
  * ...
  * bops = buf_ops_create(fd);
- * intel_buf_init(bops, &ibuf, 512, 512, 32, I915_TILING_X, false);
+ * intel_buf_init(bops, &ibuf, 512, 512, 32, 64, I915_TILING_X, false);
  * ...
  * linear_to_intel_buf(bops, &ibuf, linear);
  * ...
@@ -87,9 +90,9 @@
 #define TILE_Yf     TILE_DEF(I915_TILING_Yf)
 #define TILE_Ys     TILE_DEF(I915_TILING_Ys)
 
-#define CCS_OFFSET(buf) (buf->aux.offset)
+#define CCS_OFFSET(buf) (buf->ccs[0].offset)
 #define CCS_SIZE(gen, buf) \
-	(intel_buf_aux_width(gen, buf) * intel_buf_aux_height(gen, buf))
+	(intel_buf_ccs_width(gen, buf) * intel_buf_ccs_height(gen, buf))
 
 typedef void (*bo_copy)(struct buf_ops *, struct intel_buf *, uint32_t *);
 
@@ -141,6 +144,16 @@ static inline bool is_tiling_supported(struct buf_ops *bops, uint32_t tiling)
 	return bops->supported_tiles & TILE_DEF(tiling);
 }
 
+static uint32_t get_stride(uint32_t devid, uint32_t tiling)
+{
+	uint32_t stride = 128;
+
+	if (IS_915G(devid) || IS_915GM(devid) || tiling == I915_TILING_X)
+		stride = 512;
+
+	return stride;
+}
+
 static int __gem_get_tiling(int fd, struct drm_i915_gem_get_tiling *arg)
 {
 	int err;
@@ -156,7 +169,7 @@ static int __gem_get_tiling(int fd, struct drm_i915_gem_get_tiling *arg)
 }
 
 static bool __get_tiling(int fd, uint32_t handle, uint32_t *tiling,
-			 uint32_t *swizzle)
+			 uint32_t *swizzle, uint32_t *phys_swizzle)
 {
 	struct drm_i915_gem_get_tiling get_tiling = { .handle = handle };
 
@@ -165,19 +178,24 @@ static bool __get_tiling(int fd, uint32_t handle, uint32_t *tiling,
 
 	*tiling = get_tiling.tiling_mode;
 	*swizzle = get_tiling.swizzle_mode;
+	*phys_swizzle = get_tiling.phys_swizzle_mode;
 	igt_debug("buf tiling: %s, swizzle: %x, phys_swizzle: %x\n",
 		  tiling_str(get_tiling.tiling_mode),
 		  get_tiling.swizzle_mode,
 		  get_tiling.phys_swizzle_mode);
 
-	return get_tiling.phys_swizzle_mode == get_tiling.swizzle_mode;
+	return true;
 }
 
 static int __set_tiling(int fd, uint32_t handle, uint32_t tiling,
-			uint32_t stride)
+			uint32_t stride,
+			uint32_t *ret_tiling, uint32_t *ret_swizzle)
 {
+	struct drm_i915_gem_set_tiling st;
+
+	memset(&st, 0, sizeof(st));
 	do {
-		struct drm_i915_gem_set_tiling st;
+
 		int err;
 
 		st.handle = handle;
@@ -188,22 +206,38 @@ static int __set_tiling(int fd, uint32_t handle, uint32_t tiling,
 		if (ioctl(fd, DRM_IOCTL_I915_GEM_SET_TILING, &st))
 			err = -errno;
 		errno = 0;
-		if (err != -EINTR)
+		if (err != -EINTR) {
+			if (ret_tiling)
+				*ret_tiling = st.tiling_mode;
+
+			if (ret_swizzle)
+				*ret_swizzle = st.swizzle_mode;
+
 			return err;
+		}
 	} while (1);
 }
 
 static void set_hw_tiled(struct buf_ops *bops, struct intel_buf *buf)
 {
+	uint32_t ret_tiling, ret_swizzle;
+
 	if (buf->tiling != I915_TILING_X && buf->tiling != I915_TILING_Y)
 		return;
 
-	if (!buf_ops_has_hw_fence(bops, buf->tiling))
+	if (!buf_ops_has_hw_fence(bops, buf->tiling)) {
+		igt_debug("No HW fence for tiling: %d\n", buf->tiling);
 		return;
+	}
 
 	igt_assert_eq(__set_tiling(bops->fd,
-				   buf->handle, buf->tiling, buf->stride),
+				   buf->handle, buf->tiling,
+				   buf->surface[0].stride,
+				   &ret_tiling, &ret_swizzle),
 		      0);
+
+	igt_assert(ret_tiling == buf->tiling);
+	buf->swizzle_mode = ret_swizzle;
 }
 
 static unsigned long swizzle_bit(unsigned int bit, unsigned long offset)
@@ -391,10 +425,10 @@ static void *mmap_write(int fd, struct intel_buf *buf)
 	void *map = NULL;
 
 	if (is_cache_coherent(fd, buf->handle)) {
-		map = __gem_mmap_offset__cpu(fd, buf->handle, 0, buf->size,
+		map = __gem_mmap_offset__cpu(fd, buf->handle, 0, buf->surface[0].size,
 					     PROT_READ | PROT_WRITE);
 		if (!map)
-			map = __gem_mmap__cpu(fd, buf->handle, 0, buf->size,
+			map = __gem_mmap__cpu(fd, buf->handle, 0, buf->surface[0].size,
 					      PROT_READ | PROT_WRITE);
 
 		if (map)
@@ -404,10 +438,10 @@ static void *mmap_write(int fd, struct intel_buf *buf)
 	}
 
 	if (!map) {
-		map = __gem_mmap_offset__wc(fd, buf->handle, 0, buf->size,
+		map = __gem_mmap_offset__wc(fd, buf->handle, 0, buf->surface[0].size,
 					    PROT_READ | PROT_WRITE);
 		if (!map)
-			map = gem_mmap__wc(fd, buf->handle, 0, buf->size,
+			map = gem_mmap__wc(fd, buf->handle, 0, buf->surface[0].size,
 					   PROT_READ | PROT_WRITE);
 
 		gem_set_domain(fd, buf->handle,
@@ -423,9 +457,9 @@ static void *mmap_read(int fd, struct intel_buf *buf)
 
 	if (gem_has_llc(fd) || is_cache_coherent(fd, buf->handle)) {
 		map = __gem_mmap_offset__cpu(fd, buf->handle, 0,
-					     buf->size, PROT_READ);
+					     buf->surface[0].size, PROT_READ);
 		if (!map)
-			map = __gem_mmap__cpu(fd, buf->handle, 0, buf->size,
+			map = __gem_mmap__cpu(fd, buf->handle, 0, buf->surface[0].size,
 					      PROT_READ);
 
 		if (map)
@@ -433,10 +467,10 @@ static void *mmap_read(int fd, struct intel_buf *buf)
 	}
 
 	if (!map) {
-		map = __gem_mmap_offset__wc(fd, buf->handle, 0, buf->size,
+		map = __gem_mmap_offset__wc(fd, buf->handle, 0, buf->surface[0].size,
 					    PROT_READ);
 		if (!map)
-			map = gem_mmap__wc(fd, buf->handle, 0, buf->size,
+			map = gem_mmap__wc(fd, buf->handle, 0, buf->surface[0].size,
 					   PROT_READ);
 
 		gem_set_domain(fd, buf->handle, I915_GEM_DOMAIN_WC, 0);
@@ -456,7 +490,7 @@ static void __copy_linear_to(int fd, struct intel_buf *buf,
 
 	for (int y = 0; y < height; y++) {
 		for (int x = 0; x < width; x++) {
-			uint32_t *ptr = fn(map, x, y, buf->stride, buf->bpp/8);
+			uint32_t *ptr = fn(map, x, y, buf->surface[0].stride, buf->bpp/8);
 
 			if (swizzle)
 				ptr = from_user_pointer(swizzle_addr(ptr,
@@ -465,7 +499,7 @@ static void __copy_linear_to(int fd, struct intel_buf *buf,
 		}
 	}
 
-	munmap(map, buf->size);
+	munmap(map, buf->surface[0].size);
 }
 
 static void copy_linear_to_x(struct buf_ops *bops, struct intel_buf *buf,
@@ -506,7 +540,7 @@ static void __copy_to_linear(int fd, struct intel_buf *buf,
 
 	for (int y = 0; y < height; y++) {
 		for (int x = 0; x < width; x++) {
-			uint32_t *ptr = fn(map, x, y, buf->stride, buf->bpp/8);
+			uint32_t *ptr = fn(map, x, y, buf->surface[0].stride, buf->bpp/8);
 
 			if (swizzle)
 				ptr = from_user_pointer(swizzle_addr(ptr,
@@ -515,7 +549,7 @@ static void __copy_to_linear(int fd, struct intel_buf *buf,
 		}
 	}
 
-	munmap(map, buf->size);
+	munmap(map, buf->surface[0].size);
 }
 
 static void copy_x_to_linear(struct buf_ops *bops, struct intel_buf *buf,
@@ -554,14 +588,14 @@ static void copy_linear_to_gtt(struct buf_ops *bops, struct intel_buf *buf,
 	DEBUGFN();
 
 	map = gem_mmap__gtt(bops->fd, buf->handle,
-			    buf->size, PROT_READ | PROT_WRITE);
+			    buf->surface[0].size, PROT_READ | PROT_WRITE);
 
 	gem_set_domain(bops->fd, buf->handle,
 		       I915_GEM_DOMAIN_GTT, I915_GEM_DOMAIN_GTT);
 
-	memcpy(map, linear, buf->size);
+	memcpy(map, linear, buf->surface[0].size);
 
-	munmap(map, buf->size);
+	munmap(map, buf->surface[0].size);
 }
 
 static void copy_gtt_to_linear(struct buf_ops *bops, struct intel_buf *buf,
@@ -572,14 +606,14 @@ static void copy_gtt_to_linear(struct buf_ops *bops, struct intel_buf *buf,
 	DEBUGFN();
 
 	map = gem_mmap__gtt(bops->fd, buf->handle,
-			    buf->size, PROT_READ);
+			    buf->surface[0].size, PROT_READ);
 
 	gem_set_domain(bops->fd, buf->handle,
 		       I915_GEM_DOMAIN_GTT, 0);
 
-	igt_memcpy_from_wc(linear, map, buf->size);
+	igt_memcpy_from_wc(linear, map, buf->surface[0].size);
 
-	munmap(map, buf->size);
+	munmap(map, buf->surface[0].size);
 }
 
 static void copy_linear_to_wc(struct buf_ops *bops, struct intel_buf *buf,
@@ -590,8 +624,8 @@ static void copy_linear_to_wc(struct buf_ops *bops, struct intel_buf *buf,
 	DEBUGFN();
 
 	map = mmap_write(bops->fd, buf);
-	memcpy(map, linear, buf->size);
-	munmap(map, buf->size);
+	memcpy(map, linear, buf->surface[0].size);
+	munmap(map, buf->surface[0].size);
 }
 
 static void copy_wc_to_linear(struct buf_ops *bops, struct intel_buf *buf,
@@ -602,8 +636,8 @@ static void copy_wc_to_linear(struct buf_ops *bops, struct intel_buf *buf,
 	DEBUGFN();
 
 	map = mmap_read(bops->fd, buf);
-	igt_memcpy_from_wc(linear, map, buf->size);
-	munmap(map, buf->size);
+	igt_memcpy_from_wc(linear, map, buf->surface[0].size);
+	munmap(map, buf->surface[0].size);
 }
 
 void intel_buf_to_linear(struct buf_ops *bops, struct intel_buf *buf,
@@ -673,26 +707,34 @@ void linear_to_intel_buf(struct buf_ops *bops, struct intel_buf *buf,
 static void __intel_buf_init(struct buf_ops *bops,
 			     uint32_t handle,
 			     struct intel_buf *buf,
-			     int width, int height, int bpp,
-			     uint32_t req_tiling, uint32_t compression)
+			     int width, int height, int bpp, int alignment,
+			     uint32_t req_tiling, uint32_t compression,
+			     uint64_t bo_size)
 {
 	uint32_t tiling = req_tiling;
-	uint32_t size;
+	uint64_t size;
+	uint32_t devid;
+	int tile_width;
+	int align_h = 1;
 
 	igt_assert(bops);
 	igt_assert(buf);
 	igt_assert(width > 0 && height > 0);
-	igt_assert(bpp == 8 || bpp == 16 || bpp == 32);
+	igt_assert(bpp == 8 || bpp == 16 || bpp == 32 || bpp == 64);
+	igt_assert(alignment >= 0);
 
 	memset(buf, 0, sizeof(*buf));
 
+	buf->bops = bops;
+	buf->addr.offset = INTEL_BUF_INVALID_ADDRESS;
+	IGT_INIT_LIST_HEAD(&buf->link);
+
 	if (compression) {
 		int aux_width, aux_height;
 
 		igt_require(bops->intel_gen >= 9);
 		igt_assert(req_tiling == I915_TILING_Y ||
 			   req_tiling == I915_TILING_Yf);
-
 		/*
 		 * On GEN12+ we align the main surface to 4 * 4 main surface
 		 * tiles, which is 64kB. These 16 tiles are mapped by 4 AUX
@@ -700,35 +742,50 @@ static void __intel_buf_init(struct buf_ops *bops,
 		 * turn mapped by one L1 AUX page table entry.
 		 */
 		if (bops->intel_gen >= 12)
-			buf->stride = ALIGN(width * (bpp / 8), 128 * 4);
+			buf->surface[0].stride = ALIGN(width * (bpp / 8), 128 * 4);
 		else
-			buf->stride = ALIGN(width * (bpp / 8), 128);
+			buf->surface[0].stride = ALIGN(width * (bpp / 8), 128);
 
 		if (bops->intel_gen >= 12)
 			height = ALIGN(height, 4 * 32);
 
-		buf->size = buf->stride * height;
+		buf->surface[0].size = buf->surface[0].stride * height;
 		buf->tiling = tiling;
 		buf->bpp = bpp;
 		buf->compression = compression;
 
-		aux_width = intel_buf_aux_width(bops->intel_gen, buf);
-		aux_height = intel_buf_aux_height(bops->intel_gen, buf);
+		aux_width = intel_buf_ccs_width(bops->intel_gen, buf);
+		aux_height = intel_buf_ccs_height(bops->intel_gen, buf);
 
-		buf->aux.offset = buf->stride * ALIGN(height, 32);
-		buf->aux.stride = aux_width;
-
-		size = buf->aux.offset + aux_width * aux_height;
+		buf->ccs[0].offset = buf->surface[0].stride * ALIGN(height, 32);
+		buf->ccs[0].stride = aux_width;
 
+		size = buf->ccs[0].offset + aux_width * aux_height;
 	} else {
-		buf->stride = ALIGN(width * (bpp / 8), 128);
-		buf->size = buf->stride * height;
+		if (tiling) {
+			devid =  intel_get_drm_devid(bops->fd);
+			tile_width = get_stride(devid, tiling);
+			buf->surface[0].stride = ALIGN(width * (bpp / 8), tile_width);
+			align_h = tiling == I915_TILING_X ? 8 : 32;
+		} else {
+			buf->surface[0].stride = ALIGN(width * (bpp / 8), alignment ?: 1);
+		}
+
+		buf->surface[0].size = buf->surface[0].stride * height;
 		buf->tiling = tiling;
 		buf->bpp = bpp;
 
-		size = buf->stride * ALIGN(height, 32);
+		size = buf->surface[0].stride * ALIGN(height, align_h);
 	}
 
+	if (bo_size > 0) {
+		igt_assert(bo_size >= size);
+		size = bo_size;
+	}
+
+	/* Store real bo size to avoid mistakes in calculating it again */
+	buf->size = size;
+
 	if (handle)
 		buf->handle = handle;
 	else
@@ -743,22 +800,25 @@ static void __intel_buf_init(struct buf_ops *bops,
  * @buf: pointer to intel_buf structure to be filled
  * @width: surface width
  * @height: surface height
- * @bpp: bits-per-pixel (8 / 16 / 32)
+ * @bpp: bits-per-pixel (8 / 16 / 32 / 64)
+ * @alignment: alignment of the stride for linear surfaces
  * @tiling: surface tiling
  * @compression: surface compression type
  *
  * Function creates new BO within intel_buf structure and fills all
- * structure fields.
+ * structure fields. Takes bo handle ownership.
  *
  * Note. For X / Y if GPU supports fences HW tiling is configured.
  */
 void intel_buf_init(struct buf_ops *bops,
 		    struct intel_buf *buf,
-		    int width, int height, int bpp,
+		    int width, int height, int bpp, int alignment,
 		    uint32_t tiling, uint32_t compression)
 {
-	__intel_buf_init(bops, 0, buf, width, height, bpp, tiling,
-			 compression);
+	__intel_buf_init(bops, 0, buf, width, height, bpp, alignment,
+			 tiling, compression, 0);
+
+	intel_buf_set_ownership(buf, true);
 }
 
 /**
@@ -766,14 +826,27 @@ void intel_buf_init(struct buf_ops *bops,
  * @bops: pointer to buf_ops
  * @buf: pointer to intel_buf structure
  *
- * Function closes gem BO inside intel_buf.
+ * Function closes gem BO inside intel_buf if bo is owned by intel_buf.
+ * For handle passed from the caller intel_buf doesn't take ownership and
+ * doesn't close it in close()/destroy() paths. When intel_buf was previously
+ * added to intel_bb (intel_bb_add_intel_buf() call) it is tracked there and
+ * must be removed from its internal structures.
  */
 void intel_buf_close(struct buf_ops *bops, struct intel_buf *buf)
 {
 	igt_assert(bops);
 	igt_assert(buf);
 
-	gem_close(bops->fd, buf->handle);
+	/* If buf is tracked by some intel_bb ensure it will be removed there */
+	if (buf->ibb) {
+		intel_bb_remove_intel_buf(buf->ibb, buf);
+		buf->addr.offset = INTEL_BUF_INVALID_ADDRESS;
+		buf->ibb = NULL;
+		IGT_INIT_LIST_HEAD(&buf->link);
+	}
+
+	if (buf->is_owner)
+		gem_close(bops->fd, buf->handle);
 }
 
 /**
@@ -783,7 +856,8 @@ void intel_buf_close(struct buf_ops *bops, struct intel_buf *buf)
  * @buf: pointer to intel_buf structure to be filled
  * @width: surface width
  * @height: surface height
- * @bpp: bits-per-pixel (8 / 16 / 32)
+ * @bpp: bits-per-pixel (8 / 16 / 32 / 64)
+ * @alignment: alignment of the stride for linear surfaces
  * @tiling: surface tiling
  * @compression: surface compression type
  *
@@ -791,17 +865,260 @@ void intel_buf_close(struct buf_ops *bops, struct intel_buf *buf)
  * (with all its metadata - width, height, ...). Useful if BO was created
  * outside.
  *
- * Note: intel_buf_close() can be used to close the BO handle, but caller
- * must be aware to not close the BO twice.
+ * Note: intel_buf_close() can be used because intel_buf is aware it is not
+ * buffer owner so it won't close it underneath.
  */
 void intel_buf_init_using_handle(struct buf_ops *bops,
 				 uint32_t handle,
 				 struct intel_buf *buf,
-				 int width, int height, int bpp,
+				 int width, int height, int bpp, int alignment,
 				 uint32_t req_tiling, uint32_t compression)
 {
-	__intel_buf_init(bops, handle, buf, width, height, bpp, req_tiling,
-			 compression);
+	__intel_buf_init(bops, handle, buf, width, height, bpp, alignment,
+			 req_tiling, compression, 0);
+}
+
+/**
+ * intel_buf_create
+ * @bops: pointer to buf_ops
+ * @width: surface width
+ * @height: surface height
+ * @bpp: bits-per-pixel (8 / 16 / 32 / 64)
+ * @alignment: alignment of the stride for linear surfaces
+ * @tiling: surface tiling
+ * @compression: surface compression type
+ *
+ * Function creates intel_buf with created BO handle. Takes ownership of the
+ * buffer.
+ */
+struct intel_buf *intel_buf_create(struct buf_ops *bops,
+				   int width, int height,
+				   int bpp, int alignment,
+				   uint32_t req_tiling, uint32_t compression)
+{
+	struct intel_buf *buf;
+
+	igt_assert(bops);
+
+	buf = calloc(1, sizeof(*buf));
+	igt_assert(buf);
+
+	intel_buf_init(bops, buf, width, height, bpp, alignment,
+		       req_tiling, compression);
+
+	return buf;
+}
+
+/**
+ * intel_buf_create_using_handle
+ * @bops: pointer to buf_ops
+ * @handle: BO handle created by the caller
+ * @width: surface width
+ * @height: surface height
+ * @bpp: bits-per-pixel (8 / 16 / 32 / 64)
+ * @alignment: alignment of the stride for linear surfaces
+ * @tiling: surface tiling
+ * @compression: surface compression type
+ *
+ * Function creates intel_buf with passed BO handle from the caller. Doesn't
+ * take ownership of the buffer. close()/destroy() paths doesn't close
+ * passed handle unless buffer will take ownership using set_ownership().
+ */
+struct intel_buf *intel_buf_create_using_handle(struct buf_ops *bops,
+						uint32_t handle,
+						int width, int height,
+						int bpp, int alignment,
+						uint32_t req_tiling,
+						uint32_t compression)
+{
+	struct intel_buf *buf;
+
+	igt_assert(bops);
+
+	buf = calloc(1, sizeof(*buf));
+	igt_assert(buf);
+
+	intel_buf_init_using_handle(bops, handle, buf, width, height, bpp,
+				    alignment, req_tiling, compression);
+
+	return buf;
+}
+
+struct intel_buf *intel_buf_create_using_handle_and_size(struct buf_ops *bops,
+							 uint32_t handle,
+							 int width, int height,
+							 int bpp, int alignment,
+							 uint32_t req_tiling,
+							 uint32_t compression,
+							 uint64_t size)
+{
+	struct intel_buf *buf;
+
+	igt_assert(bops);
+
+	buf = calloc(1, sizeof(*buf));
+	igt_assert(buf);
+
+	__intel_buf_init(bops, handle, buf, width, height, bpp, alignment,
+			 req_tiling, compression, size);
+
+	return buf;
+}
+
+
+/**
+ * intel_buf_destroy
+ * @buf: intel_buf
+ *
+ * Function frees intel_buf memory. It closes bo handle if intel_buf has
+ * buffer ownership.
+ */
+void intel_buf_destroy(struct intel_buf *buf)
+{
+	igt_assert(buf);
+	igt_assert(buf->ptr == NULL);
+
+	intel_buf_close(buf->bops, buf);
+
+	free(buf);
+}
+
+void *intel_buf_cpu_map(struct intel_buf *buf, bool write)
+{
+	int i915 = buf_ops_get_fd(buf->bops);
+
+	igt_assert(buf);
+	igt_assert(buf->ptr == NULL); /* already mapped */
+
+	buf->cpu_write = write;
+	buf->ptr = gem_mmap__cpu_coherent(i915, buf->handle, 0,
+					  buf->surface[0].size,
+					  write ? PROT_WRITE : PROT_READ);
+
+	gem_set_domain(i915, buf->handle,
+		       I915_GEM_DOMAIN_CPU,
+		       write ? I915_GEM_DOMAIN_CPU : 0);
+
+	return buf->ptr;
+}
+
+void *intel_buf_device_map(struct intel_buf *buf, bool write)
+{
+	int i915 = buf_ops_get_fd(buf->bops);
+
+	igt_assert(buf);
+	igt_assert(buf->ptr == NULL); /* already mapped */
+
+	buf->ptr = gem_mmap__device_coherent(i915, buf->handle, 0,
+					     buf->surface[0].size,
+					     write ? PROT_WRITE : PROT_READ);
+
+	gem_set_domain(i915, buf->handle,
+		       I915_GEM_DOMAIN_WC,
+		       write ? I915_GEM_DOMAIN_WC : 0);
+
+	return buf->ptr;
+}
+
+void intel_buf_unmap(struct intel_buf *buf)
+{
+	igt_assert(buf);
+	igt_assert(buf->ptr);
+
+	munmap(buf->ptr, buf->surface[0].size);
+	buf->ptr = NULL;
+}
+
+void intel_buf_flush_and_unmap(struct intel_buf *buf)
+{
+	igt_assert(buf);
+	igt_assert(buf->ptr);
+
+	if (buf->cpu_write)
+		gem_sw_finish(buf_ops_get_fd(buf->bops), buf->handle);
+
+	intel_buf_unmap(buf);
+}
+
+void intel_buf_print(const struct intel_buf *buf)
+{
+	igt_info("[name: %s]\n", buf->name);
+	igt_info("[%u]: w: %u, h: %u, stride: %u, size: %" PRIx64
+		 ", bo-size: %" PRIx64 ", bpp: %u, tiling: %u, compress: %u\n",
+		 buf->handle, intel_buf_width(buf), intel_buf_height(buf),
+		 buf->surface[0].stride, buf->surface[0].size,
+		 intel_buf_bo_size(buf), buf->bpp,
+		 buf->tiling, buf->compression);
+	igt_info(" ccs <offset: %u, stride: %u, w: %u, h: %u> cc <offset: %u>\n",
+		 buf->ccs[0].offset,
+		 intel_buf_ccs_width(buf->bops->intel_gen, buf),
+		 intel_buf_ccs_height(buf->bops->intel_gen, buf),
+		 buf->ccs[0].stride, buf->cc.offset);
+	igt_info(" addr <offset: %p, ctx: %u>\n",
+		 from_user_pointer(buf->addr.offset), buf->addr.ctx);
+}
+
+void intel_buf_dump(const struct intel_buf *buf, const char *filename)
+{
+	int i915 = buf_ops_get_fd(buf->bops);
+	uint64_t size = intel_buf_bo_size(buf);
+	FILE *out;
+	void *ptr;
+
+	ptr = gem_mmap__device_coherent(i915, buf->handle, 0, size, PROT_READ);
+	out = fopen(filename, "wb");
+	igt_assert(out);
+	fwrite(ptr, size, 1, out);
+	fclose(out);
+	munmap(ptr, size);
+}
+
+const char *intel_buf_set_name(struct intel_buf *buf, const char *name)
+{
+	return strncpy(buf->name, name, INTEL_BUF_NAME_MAXSIZE);
+}
+
+static void __intel_buf_write_to_png(struct buf_ops *bops,
+				     struct intel_buf *buf,
+				     const char *filename,
+				     bool write_ccs)
+{
+	cairo_surface_t *surface;
+	cairo_status_t ret;
+	void *linear;
+	int format, width, height, stride, offset;
+	int gen = bops->intel_gen;
+
+	igt_assert_eq(posix_memalign(&linear, 16, intel_buf_bo_size(buf)), 0);
+
+	format = write_ccs ? CAIRO_FORMAT_A8 : CAIRO_FORMAT_RGB24;
+	width = write_ccs ? intel_buf_ccs_width(gen, buf) : intel_buf_width(buf);
+	height = write_ccs ? intel_buf_ccs_height(gen, buf) : intel_buf_height(buf);
+	stride = write_ccs ? buf->ccs[0].stride : buf->surface[0].stride;
+	offset = write_ccs ? buf->ccs[0].offset : 0;
+
+	intel_buf_to_linear(bops, buf, linear);
+
+	surface = cairo_image_surface_create_for_data((uint8_t *) linear + offset,
+						      format, width, height,
+						      stride);
+	ret = cairo_surface_write_to_png(surface, filename);
+	igt_assert(ret == CAIRO_STATUS_SUCCESS);
+	cairo_surface_destroy(surface);
+
+	free(linear);
+}
+
+void intel_buf_write_to_png(struct intel_buf *buf, const char *filename)
+{
+	__intel_buf_write_to_png(buf->bops, buf, filename, false);
+}
+
+void intel_buf_write_aux_to_png(struct intel_buf *buf, const char *filename)
+{
+	igt_assert(buf->compression);
+
+	__intel_buf_write_to_png(buf->bops, buf, filename, true);
 }
 
 #define DEFAULT_BUFOPS(__gen_start, __gen_end) \
@@ -836,34 +1153,33 @@ struct buf_ops buf_ops_arr[] = {
 	},
 };
 
-static bool probe_hw_tiling(struct buf_ops *bops, uint32_t tiling)
+static bool probe_hw_tiling(struct buf_ops *bops, uint32_t tiling,
+			    bool *swizzling_supported)
 {
 	uint64_t size = 256 * 256;
-	uint32_t handle, buf_tiling, buf_swizzle;
-	uint32_t stride;
+	uint32_t handle, buf_tiling, buf_swizzle, phys_swizzle;
+	uint32_t devid, stride;
 	int ret;
 	bool is_set = false;
 
-	if (tiling == I915_TILING_X)
-		stride = 512;
-	else if (tiling == I915_TILING_Y)
-		stride = 128;
-	else
-		return false;
-
+	devid =  intel_get_drm_devid(bops->fd);
+	stride = get_stride(devid, tiling);
 	handle = gem_create(bops->fd, size);
 
 	/* Single shot, if no fences are available we fail immediately */
-	ret = __set_tiling(bops->fd, handle, tiling, stride);
+	ret = __set_tiling(bops->fd, handle, tiling, stride, NULL, NULL);
 	if (ret)
 		goto end;
 
-	is_set = __get_tiling(bops->fd, handle, &buf_tiling, &buf_swizzle);
+	is_set = __get_tiling(bops->fd, handle, &buf_tiling, &buf_swizzle,
+			      &phys_swizzle);
 	if (is_set) {
 		if (tiling == I915_TILING_X)
 			bops->swizzle_x = buf_swizzle;
 		else if (tiling == I915_TILING_Y)
 			bops->swizzle_y = buf_swizzle;
+
+		*swizzling_supported = buf_swizzle == phys_swizzle;
 	}
 end:
 	gem_close(bops->fd, handle);
@@ -908,15 +1224,15 @@ static void idempotency_selftest(struct buf_ops *bops, uint32_t tiling)
 			  bool_str(software_tiling),
 			  bool_str(!software_tiling),
 			  tiling_str(tiling));
-		intel_buf_init(bops, &buf, width, height, bpp, tiling, false);
+		intel_buf_init(bops, &buf, width, height, bpp, 0, tiling, false);
 		buf_ops_set_software_tiling(bops, tiling, software_tiling);
 
 		linear_to_intel_buf(bops, &buf, (uint32_t *) linear_in);
 		map = __gem_mmap_offset__cpu(bops->fd, buf.handle, 0,
-					   buf.size, PROT_READ);
+					     buf.surface[0].size, PROT_READ);
 		if (!map)
 			map = gem_mmap__cpu(bops->fd, buf.handle, 0,
-					    buf.size, PROT_READ);
+					    buf.surface[0].size, PROT_READ);
 		gem_set_domain(bops->fd, buf.handle, I915_GEM_DOMAIN_CPU, 0);
 		igt_assert(memcmp(linear_in, map, size));
 		munmap(map, size);
@@ -934,16 +1250,12 @@ static void idempotency_selftest(struct buf_ops *bops, uint32_t tiling)
 	buf_ops_set_software_tiling(bops, tiling, false);
 }
 
-/**
- * buf_ops_create
- * @fd: device filedescriptor
- *
- * Create buf_ops structure depending on fd-device capabilities.
- *
- * Returns: opaque pointer to buf_ops.
- *
- */
-struct buf_ops *buf_ops_create(int fd)
+uint64_t intel_buf_bo_size(const struct intel_buf *buf)
+{
+	return buf->size;
+}
+
+static struct buf_ops *__buf_ops_create(int fd, bool check_idempotency)
 {
 	struct buf_ops *bops = calloc(1, sizeof(*bops));
 	uint32_t devid;
@@ -986,7 +1298,14 @@ struct buf_ops *buf_ops_create(int fd)
 
 	/* Let's probe X and Y hw tiling support */
 	if (is_hw_tiling_supported(bops, I915_TILING_X)) {
-		bool supported = probe_hw_tiling(bops, I915_TILING_X);
+		bool swizzling_supported;
+		bool supported = probe_hw_tiling(bops, I915_TILING_X,
+						 &swizzling_supported);
+
+		if (!swizzling_supported) {
+			igt_debug("Swizzling for X is not supported\n");
+			bops->supported_tiles &= ~TILE_X;
+		}
 
 		igt_debug("X fence support: %s\n", bool_str(supported));
 		if (!supported) {
@@ -997,7 +1316,14 @@ struct buf_ops *buf_ops_create(int fd)
 	}
 
 	if (is_hw_tiling_supported(bops, I915_TILING_Y)) {
-		bool supported = probe_hw_tiling(bops, I915_TILING_Y);
+		bool swizzling_supported;
+		bool supported = probe_hw_tiling(bops, I915_TILING_Y,
+						 &swizzling_supported);
+
+		if (!swizzling_supported) {
+			igt_debug("Swizzling for Y is not supported\n");
+			bops->supported_tiles &= ~TILE_Y;
+		}
 
 		igt_debug("Y fence support: %s\n", bool_str(supported));
 		if (!supported) {
@@ -1020,12 +1346,44 @@ struct buf_ops *buf_ops_create(int fd)
 		bops->ys_to_linear = NULL;
 	}
 
-	idempotency_selftest(bops, I915_TILING_X);
-	idempotency_selftest(bops, I915_TILING_Y);
+	if (check_idempotency) {
+		idempotency_selftest(bops, I915_TILING_X);
+		idempotency_selftest(bops, I915_TILING_Y);
+	}
 
 	return bops;
 }
 
+/**
+ * buf_ops_create
+ * @fd: device filedescriptor
+ *
+ * Create buf_ops structure depending on fd-device capabilities.
+ *
+ * Returns: opaque pointer to buf_ops.
+ *
+ */
+struct buf_ops *buf_ops_create(int fd)
+{
+	return __buf_ops_create(fd, false);
+}
+
+/**
+ * buf_ops_create_with_selftest
+ * @fd: device filedescriptor
+ *
+ * Create buf_ops structure depending on fd-device capabilities.
+ * Runs with idempotency selftest to verify software tiling gives same
+ * result like hardware tiling (gens with mappable gtt).
+ *
+ * Returns: opaque pointer to buf_ops.
+ *
+ */
+struct buf_ops *buf_ops_create_with_selftest(int fd)
+{
+	return __buf_ops_create(fd, true);
+}
+
 /**
  * buf_ops_destroy
  * @bops: pointer to buf_ops
@@ -1038,6 +1396,19 @@ void buf_ops_destroy(struct buf_ops *bops)
 	free(bops);
 }
 
+/**
+ * buf_ops_get_fd
+ * @bops: pointer to buf_ops
+ *
+ * Returns: drm fd
+ */
+int buf_ops_get_fd(struct buf_ops *bops)
+{
+	igt_assert(bops);
+
+	return bops->fd;
+}
+
 /**
  * buf_ops_set_software_tiling
  * @bops: pointer to buf_ops
@@ -1069,6 +1440,9 @@ bool buf_ops_set_software_tiling(struct buf_ops *bops,
 	switch (tiling) {
 	case I915_TILING_X:
 		if (use_software_tiling) {
+			bool supported = buf_ops_has_tiling_support(bops, tiling);
+
+			igt_assert_f(supported, "Cannot switch to X software tiling\n");
 			igt_debug("-> change X to SW\n");
 			bops->linear_to_x = copy_linear_to_x;
 			bops->x_to_linear = copy_x_to_linear;
@@ -1086,6 +1460,9 @@ bool buf_ops_set_software_tiling(struct buf_ops *bops,
 
 	case I915_TILING_Y:
 		if (use_software_tiling) {
+			bool supported = buf_ops_has_tiling_support(bops, tiling);
+
+			igt_assert_f(supported, "Cannot switch to Y software tiling\n");
 			igt_debug("-> change Y to SW\n");
 			bops->linear_to_y = copy_linear_to_y;
 			bops->y_to_linear = copy_y_to_linear;
