@@ -315,10 +315,42 @@ static const struct intel_device_info intel_elkhartlake_info = {
 	.codename = "elkhartlake"
 };
 
-static const struct intel_device_info intel_tigerlake_info = {
+static const struct intel_device_info intel_jasperlake_info = {
+	.gen = BIT(10),
+	.is_jasperlake = true,
+	.codename = "jasperlake"
+};
+
+static const struct intel_device_info intel_tigerlake_gt1_info = {
 	.gen = BIT(11),
 	.is_tigerlake = true,
-	.codename = "tigerlake"
+	.codename = "tigerlake",
+	.gt = 1,
+};
+
+static const struct intel_device_info intel_tigerlake_gt2_info = {
+	.gen = BIT(11),
+	.is_tigerlake = true,
+	.codename = "tigerlake",
+	.gt = 2,
+};
+
+static const struct intel_device_info intel_rocketlake_info = {
+	.gen = BIT(11),
+	.is_rocketlake = true,
+	.codename = "rocketlake"
+};
+
+static const struct intel_device_info intel_dg1_info = {
+	.gen = BIT(11),
+	.is_dg1 = true,
+	.codename = "dg1"
+};
+
+static const struct intel_device_info intel_alderlake_s_info = {
+	.gen = BIT(11),
+	.is_alderlake_s = true,
+	.codename = "alderlake_s"
 };
 
 static const struct pci_id_match intel_device_match[] = {
@@ -395,14 +427,23 @@ static const struct pci_id_match intel_device_match[] = {
 
 	INTEL_CML_GT1_IDS(&intel_cometlake_gt1_info),
 	INTEL_CML_GT2_IDS(&intel_cometlake_gt2_info),
+	INTEL_CML_U_GT1_IDS(&intel_cometlake_gt1_info),
+	INTEL_CML_U_GT2_IDS(&intel_cometlake_gt2_info),
 
 	INTEL_CNL_IDS(&intel_cannonlake_info),
 
 	INTEL_ICL_11_IDS(&intel_icelake_info),
 
 	INTEL_EHL_IDS(&intel_elkhartlake_info),
+	INTEL_JSL_IDS(&intel_jasperlake_info),
 
-	INTEL_TGL_12_IDS(&intel_tigerlake_info),
+	INTEL_TGL_12_GT1_IDS(&intel_tigerlake_gt1_info),
+	INTEL_TGL_12_GT2_IDS(&intel_tigerlake_gt2_info),
+	INTEL_RKL_IDS(&intel_rocketlake_info),
+
+	INTEL_DG1_IDS(&intel_dg1_info),
+
+	INTEL_ADLS_IDS(&intel_alderlake_s_info),
 
 	INTEL_VGA_DEVICE(PCI_MATCH_ANY, &intel_generic_info),
 };
@@ -445,32 +486,9 @@ out:
  * Computes the Intel GFX generation for the given device id.
  *
  * Returns:
- * The GFX generation on successful lookup, 0 on failure.
+ * The GFX generation on successful lookup, -1u on failure.
  */
 unsigned intel_gen(uint16_t devid)
 {
-	return ffs(intel_get_device_info(devid)->gen);
-}
-
-/**
- * intel_gt:
- * @devid: pci device id
- *
- * Computes the Intel GFX GT size for the given device id.
- *
- * Returns:
- * The GT size.
- */
-unsigned intel_gt(uint16_t devid)
-{
-	unsigned mask = intel_gen(devid);
-
-	if (mask >= 8)
-		mask = 0xf;
-	else if (mask >= 6)
-		mask = 0x3;
-	else
-		mask = 0;
-
-	return (devid >> 4) & mask;
+	return ffs(intel_get_device_info(devid)->gen) ?: -1u;
 }
