@@ -17,11 +17,13 @@
 #include "executor.h"
 #include "output_strings.h"
 
-#define INCOMPLETE_EXITCODE -1
+#define INCOMPLETE_EXITCODE -1234
+#define GRACEFUL_EXITCODE -SIGHUP
 
 _Static_assert(INCOMPLETE_EXITCODE != IGT_EXIT_SKIP, "exit code clash");
 _Static_assert(INCOMPLETE_EXITCODE != IGT_EXIT_SUCCESS, "exit code clash");
 _Static_assert(INCOMPLETE_EXITCODE != IGT_EXIT_INVALID, "exit code clash");
+_Static_assert(INCOMPLETE_EXITCODE != GRACEFUL_EXITCODE, "exit code clash");
 
 struct subtest
 {
@@ -412,14 +414,21 @@ static struct json_object *new_escaped_json_string(const char *buf, size_t len)
 	size_t strsize = 0;
 	size_t i;
 
+	/*
+	 * Test output may be garbage; strings passed to json-c need to be
+	 * UTF-8 encoded so any non-ASCII characters are converted to their
+	 * UTF-8 representation, which requires 2 bytes per character.
+	 */
+	str = malloc(len * 2);
+	if (!str)
+		return NULL;
+
 	for (i = 0; i < len; i++) {
 		if (buf[i] > 0 && buf[i] < 128) {
-			str = realloc(str, strsize + 1);
 			str[strsize] = buf[i];
 			++strsize;
 		} else {
 			/* Encode > 128 character to UTF-8. */
-			str = realloc(str, strsize + 2);
 			str[strsize] = ((unsigned char)buf[i] >> 6) | 0xC0;
 			str[strsize + 1] = ((unsigned char)buf[i] & 0x3F) | 0x80;
 			strsize += 2;
@@ -645,6 +654,28 @@ static void process_dynamic_subtest_output(const char *piglit_name,
 					     &dynresulttext, &dyntime,
 					     dyn_result_idx < 0 ? NULL : matches.items[dyn_result_idx].where,
 					     dynend);
+
+			/*
+			 * If a dynamic subsubtest is considered
+			 * incomplete we need to check parent's status
+			 * first, to be sure that the binary hasn't
+			 * aborted or stopped gracefully (exit
+			 * code). If it has aborted then we have to
+			 * attribute this status to our subsubtest.
+			 */
+			if (!strcmp(dynresulttext, "incomplete")) {
+				struct json_object *parent_subtest;
+
+				if (json_object_object_get_ex(tests, piglit_name, &parent_subtest) &&
+				    json_object_object_get_ex(parent_subtest, "result", &parent_subtest)) {
+					const char *resulttext = json_object_get_string(parent_subtest);
+
+					if (!strcmp(resulttext, "abort") ||
+					    !strcmp(resulttext, "notrun"))
+						dynresulttext = resulttext;
+				}
+			}
+
 			set_result(current_dynamic_test, dynresulttext);
 			set_runtime(current_dynamic_test, dyntime);
 		}
@@ -783,7 +814,7 @@ static const char igt_dmesg_whitelist[] =
 	 * module load after the testing finishes, which produces this
 	 * message.
 	 */
-	"i915: probe of [0-9:.]+ failed with error -25" _
+	"i915: probe of [0-9a-fA-F:.]+ failed with error -25" _
 	/* swiotbl warns even when asked not to */
 	"mock: DMA: Out of SW-IOMMU space for [0-9]+ bytes" _
 	"usb usb[0-9]+: root hub lost power or was reset"
@@ -791,7 +822,7 @@ static const char igt_dmesg_whitelist[] =
 #undef _
 
 static const char igt_piglit_style_dmesg_blacklist[] =
-	"(\\[drm:|drm_|intel_|i915_)";
+	"(\\[drm:|drm_|intel_|i915_|\\[drm\\])";
 
 static bool init_regex_whitelist(struct settings* settings, GRegex **re)
 {
@@ -1078,8 +1109,12 @@ static const char *result_from_exitcode(int exitcode)
 		return "pass";
 	case IGT_EXIT_INVALID:
 		return "skip";
+	case IGT_EXIT_ABORT:
+		return "abort";
 	case INCOMPLETE_EXITCODE:
 		return "incomplete";
+	case GRACEFUL_EXITCODE:
+		return "notrun";
 	default:
 		return "fail";
 	}
@@ -1154,6 +1189,17 @@ static void fill_from_journal(int fd,
 		}
 	}
 
+	if (subtests->size && (exitcode == IGT_EXIT_ABORT || exitcode == GRACEFUL_EXITCODE)) {
+		char *last_subtest = subtests->subs[subtests->size - 1].name;
+		char subtest_piglit_name[256];
+		struct json_object *subtest_obj;
+
+		generate_piglit_name(entry->binary, last_subtest, subtest_piglit_name, sizeof(subtest_piglit_name));
+		subtest_obj = get_or_create_json_object(tests, subtest_piglit_name);
+
+		set_result(subtest_obj, exitcode == IGT_EXIT_ABORT ? "abort" : "notrun");
+	}
+
 	if (subtests->size == 0) {
 		char *subtestname = NULL;
 		char piglit_name[256];
@@ -1322,6 +1368,7 @@ static struct json_object *get_totals_object(struct json_object *totals,
 	json_object_object_add(obj, "dmesg-warn", json_object_new_int(0));
 	json_object_object_add(obj, "skip", json_object_new_int(0));
 	json_object_object_add(obj, "incomplete", json_object_new_int(0));
+	json_object_object_add(obj, "abort", json_object_new_int(0));
 	json_object_object_add(obj, "timeout", json_object_new_int(0));
 	json_object_object_add(obj, "notrun", json_object_new_int(0));
 	json_object_object_add(obj, "fail", json_object_new_int(0));
@@ -1606,6 +1653,7 @@ struct json_object *generate_results_json(int dirfd)
 		add_to_totals("runner", &abortsub, &results);
 
 		free_subtests(&abortsub);
+		close(fd);
 	}
 
 	free_settings(&settings);
@@ -1630,6 +1678,19 @@ bool generate_results(int dirfd)
 	}
 
 	json_string = json_object_to_json_string_ext(obj, JSON_C_TO_STRING_PRETTY);
+
+	if (json_string == NULL) {
+		fprintf(stderr, "resultgen: Failed to create json representation of the results.\n");
+		fprintf(stderr, "           This usually means that the results are too big\n");
+		fprintf(stderr, "           to fit in the memory as the text representation\n");
+		fprintf(stderr, "           is being created.\n\n");
+		fprintf(stderr, "           Either something was spamming the logs or your\n");
+		fprintf(stderr, "           system is very low on free mem.\n");
+
+		close(resultsfd);
+		return false;
+	}
+
 	write(resultsfd, json_string, strlen(json_string));
 	close(resultsfd);
 	return true;
