@@ -25,9 +25,6 @@
  *
  */
 
-#include "igt.h"
-#include "igt_device.h"
-
 #include <unistd.h>
 #include <stdlib.h>
 #include <stdint.h>
@@ -39,42 +36,13 @@
 #include <sys/stat.h>
 #include <sys/ioctl.h>
 #include <sys/time.h>
+
 #include "drm.h"
-
-
-#define LOCAL_I915_EXEC_VEBOX (4<<0)
-#define LOCAL_I915_EXEC_BSD_MASK (3<<13)
-#define LOCAL_I915_EXEC_BSD_RING1 (1<<13)
-#define LOCAL_I915_EXEC_BSD_RING2 (2<<13)
-#define LOCAL_I915_EXEC_RESOURCE_STREAMER (1<<15)
-#define LOCAL_I915_EXEC_FENCE_IN (1 << 16)
-#define LOCAL_I915_EXEC_FENCE_OUT (1 << 17)
-#define LOCAL_I915_EXEC_BATCH_FIRST (1 << 18)
-#define LOCAL_I915_EXEC_FENCE_ARRAY (1 << 19)
-
-static bool has_ring(int fd, unsigned ring_exec_flags)
-{
-	switch (ring_exec_flags & I915_EXEC_RING_MASK) {
-	case 0:
-	case I915_EXEC_RENDER:
-		return true;
-
-	case I915_EXEC_BSD:
-		if (ring_exec_flags & LOCAL_I915_EXEC_BSD_MASK)
-			return gem_has_bsd2(fd);
-		else
-			return gem_has_bsd(fd);
-
-	case I915_EXEC_BLT:
-		return gem_has_blt(fd);
-
-	case I915_EXEC_VEBOX:
-		return gem_has_vebox(fd);
-	}
-
-	igt_assert_f(0, "invalid exec flag 0x%x\n", ring_exec_flags);
-	return false;
-}
+#include "i915/gem.h"
+#include "i915/gem_ring.h"
+#include "igt.h"
+#include "igt_device.h"
+#include "sw_sync.h"
 
 static bool has_exec_batch_first(int fd)
 {
@@ -100,7 +68,7 @@ static bool has_resource_streamer(int fd)
 
 static void test_batch_first(int fd)
 {
-	const int gen = intel_gen(intel_get_drm_devid(fd));
+	const unsigned int gen = intel_gen(intel_get_drm_devid(fd));
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 obj[3];
 	struct drm_i915_gem_relocation_entry reloc[2];
@@ -183,7 +151,7 @@ static void test_batch_first(int fd)
 	igt_assert_eq_u32(value, 2);
 
 	/* Batch first mode */
-	execbuf.flags |= LOCAL_I915_EXEC_BATCH_FIRST;
+	execbuf.flags |= I915_EXEC_BATCH_FIRST;
 	gem_execbuf(fd, &execbuf);
 	gem_read(fd, obj[1].handle, 0, &value, sizeof(value));
 	igt_assert_eq_u32(value, 1);
@@ -206,6 +174,20 @@ static int has_secure_batches(const int fd)
 	return v > 0;
 }
 
+static bool has_submit_fence(int fd)
+{
+	int v = 0;
+	struct drm_i915_getparam gp = {
+		.param = I915_PARAM_HAS_EXEC_SUBMIT_FENCE,
+		.value = &v,
+	};
+
+	ioctl(fd, DRM_IOCTL_I915_GETPARAM, &gp, sizeof(gp));
+	errno = 0;
+
+	return v > 0;
+}
+
 static uint32_t batch_create(int i915)
 {
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
@@ -268,7 +250,7 @@ static void mmapped(int i915)
 	gem_close(i915, buf);
 }
 
-static uint32_t batch_create_size(int fd, uint32_t size)
+static uint32_t batch_create_size(int fd, uint64_t size)
 {
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
 	uint32_t handle;
@@ -312,6 +294,43 @@ static void test_invalid_batch_start(int fd)
 	gem_close(fd, exec.handle);
 }
 
+static void test_larger_than_life_batch(int fd)
+{
+	const struct intel_execution_engine2 *e;
+	uint64_t size = 1ULL << 32; /* batch_len is __u32 as per the ABI */
+	struct drm_i915_gem_exec_object2 exec = {
+		.handle = batch_create_size(fd, size),
+	};
+	struct drm_i915_gem_execbuffer2 execbuf = {
+		.buffers_ptr = to_user_pointer(&exec),
+		.buffer_count = 1,
+	};
+
+	/*
+	 * batch_len seems like it can have different interaction depending on
+	 * the engine and HW -- but we know that only if the GTT can be larger
+	 * than 4G do we run into u32 issues, so we can safely restrict our
+	 * checking to that subset of machines.
+	 */
+	igt_require(size < gem_aperture_size(fd));
+	intel_require_memory(2, size, CHECK_RAM); /* batch + shadow */
+
+	__for_each_physical_engine(fd, e) {
+		/* Keep the batch_len implicit [0] */
+		execbuf.flags = e->flags;
+
+		/* non-48b objects are limited to the low (4G - 4K) */
+		igt_assert_eq(__gem_execbuf(fd, &execbuf), -ENOSPC);
+
+		exec.flags = EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
+		igt_assert_eq(__gem_execbuf(fd, &execbuf), 0);
+		exec.flags = 0;
+	}
+
+	gem_sync(fd, exec.handle);
+	gem_close(fd, exec.handle);
+}
+
 struct drm_i915_gem_execbuffer2 execbuf;
 struct drm_i915_gem_exec_object2 gem_exec[1];
 uint32_t batch[2] = {MI_BATCH_BUFFER_END};
@@ -320,8 +339,6 @@ int fd;
 
 igt_main
 {
-	const struct intel_execution_engine *e;
-
 	igt_fixture {
 		fd = drm_open_driver(DRIVER_INTEL);
 		igt_require_gem(fd);
@@ -353,15 +370,6 @@ igt_main
 		execbuf.rsvd2 = 0;
 	}
 
-	igt_subtest("control") {
-		for (e = intel_execution_engines; e->name; e++) {
-			if (has_ring(fd, eb_ring(e))) {
-				execbuf.flags = eb_ring(e);
-				gem_execbuf(fd, &execbuf);
-			}
-		}
-	}
-
 	igt_subtest("readonly")
 		readonly(fd);
 
@@ -384,7 +392,7 @@ igt_main
 	}
 	igt_subtest("no-vebox") {
 		igt_require(!gem_has_vebox(fd));
-		execbuf.flags = LOCAL_I915_EXEC_VEBOX;
+		execbuf.flags = I915_EXEC_VEBOX;
 		RUN_FAIL(EINVAL);
 	}
 	igt_subtest("invalid-ring") {
@@ -393,45 +401,50 @@ igt_main
 	}
 
 	igt_subtest("invalid-ring2") {
-		execbuf.flags = LOCAL_I915_EXEC_VEBOX+1;
+		execbuf.flags = I915_EXEC_VEBOX+1;
 		RUN_FAIL(EINVAL);
 	}
 
 	igt_subtest("invalid-bsd-ring") {
-		igt_require(gem_has_bsd2(fd));
-		execbuf.flags = I915_EXEC_BSD | LOCAL_I915_EXEC_BSD_MASK;
-		RUN_FAIL(EINVAL);
+		int expected;
+
+		expected = 0;
+		if (gem_has_bsd2(fd) || !gem_has_bsd(fd))
+			expected = -EINVAL;
+
+		execbuf.flags = I915_EXEC_BSD | I915_EXEC_BSD_MASK;
+		igt_assert_eq(__gem_execbuf(fd, &execbuf), expected);
 	}
 
 	igt_subtest("invalid-bsd1-flag-on-render") {
-		execbuf.flags = I915_EXEC_RENDER | LOCAL_I915_EXEC_BSD_RING1;
+		execbuf.flags = I915_EXEC_RENDER | I915_EXEC_BSD_RING1;
 		RUN_FAIL(EINVAL);
 	}
 
 	igt_subtest("invalid-bsd2-flag-on-render") {
-		execbuf.flags = I915_EXEC_RENDER | LOCAL_I915_EXEC_BSD_RING2;
+		execbuf.flags = I915_EXEC_RENDER | I915_EXEC_BSD_RING2;
 		RUN_FAIL(EINVAL);
 	}
 
 	igt_subtest("invalid-bsd1-flag-on-blt") {
-		execbuf.flags = I915_EXEC_BLT | LOCAL_I915_EXEC_BSD_RING1;
+		execbuf.flags = I915_EXEC_BLT | I915_EXEC_BSD_RING1;
 		RUN_FAIL(EINVAL);
 	}
 
 	igt_subtest("invalid-bsd2-flag-on-blt") {
-		execbuf.flags = I915_EXEC_BLT | LOCAL_I915_EXEC_BSD_RING2;
+		execbuf.flags = I915_EXEC_BLT | I915_EXEC_BSD_RING2;
 		RUN_FAIL(EINVAL);
 	}
 
 	igt_subtest("invalid-bsd1-flag-on-vebox") {
 		igt_require(gem_has_vebox(fd));
-		execbuf.flags = LOCAL_I915_EXEC_VEBOX | LOCAL_I915_EXEC_BSD_RING1;
+		execbuf.flags = I915_EXEC_VEBOX | I915_EXEC_BSD_RING1;
 		RUN_FAIL(EINVAL);
 	}
 
 	igt_subtest("invalid-bsd2-flag-on-vebox") {
 		igt_require(gem_has_vebox(fd));
-		execbuf.flags = LOCAL_I915_EXEC_VEBOX | LOCAL_I915_EXEC_BSD_RING2;
+		execbuf.flags = I915_EXEC_VEBOX | I915_EXEC_BSD_RING2;
 		RUN_FAIL(EINVAL);
 	}
 
@@ -501,7 +514,7 @@ igt_main
 		/* NOTE: This test intentionally exercise the next available
 		 * flag. Don't "fix" this testcase without adding the required
 		 * tests for the new flag first. */
-		execbuf.flags = I915_EXEC_RENDER | (LOCAL_I915_EXEC_FENCE_ARRAY << 1);
+		execbuf.flags = I915_EXEC_RENDER | (I915_EXEC_FENCE_ARRAY << 1);
 		RUN_FAIL(EINVAL);
 	}
 
@@ -518,7 +531,7 @@ igt_main
 	igt_subtest("rs-invalid") {
 		bool has_rs = has_resource_streamer(fd);
 
-		for_each_engine(it, fd) {
+		for_each_ring(it, fd) {
 			int expect = -EINVAL;
 			if (has_rs &&
 			    (eb_ring(it) == 0 ||
@@ -526,20 +539,44 @@ igt_main
 				expect = 0;
 
 			execbuf.flags =
-				eb_ring(it) | LOCAL_I915_EXEC_RESOURCE_STREAMER;
+				eb_ring(it) | I915_EXEC_RESOURCE_STREAMER;
 			igt_assert_eq(__gem_execbuf(fd, &execbuf), expect);
 		}
 	}
 
 	igt_subtest("invalid-fence-in") {
 		igt_require(gem_has_exec_fence(fd));
-		execbuf.flags = LOCAL_I915_EXEC_FENCE_IN;
+		execbuf.flags = I915_EXEC_FENCE_IN;
 		execbuf.rsvd2 = -1;
 		RUN_FAIL(EINVAL);
 		execbuf.rsvd2 = fd;
 		RUN_FAIL(EINVAL);
 	}
 
+	igt_subtest("invalid-fence-in-submit") {
+		int timeline;
+
+		igt_require(gem_has_exec_fence(fd));
+		igt_require(has_submit_fence(fd));
+
+		timeline = sw_sync_timeline_create();
+		execbuf.rsvd2 = sw_sync_timeline_create_fence(timeline, 1);
+
+		execbuf.flags = I915_EXEC_FENCE_IN;
+		gem_execbuf(fd, &execbuf);
+
+		execbuf.flags = I915_EXEC_FENCE_SUBMIT;
+		gem_execbuf(fd, &execbuf);
+
+		execbuf.flags = I915_EXEC_FENCE_IN | I915_EXEC_FENCE_SUBMIT;
+		RUN_FAIL(EINVAL);
+
+		close(execbuf.rsvd2);
+		close(timeline);
+
+		gem_sync(fd, handle);
+	}
+
 	igt_subtest("rsvd2-dirt") {
 		igt_require(!gem_has_exec_fence(fd));
 		execbuf.flags = 0;
@@ -554,6 +591,9 @@ igt_main
 	igt_subtest("invalid-batch-start-offset")
 		test_invalid_batch_start(fd);
 
+	igt_subtest("larger-than-life-batch")
+		test_larger_than_life_batch(fd);
+
 #define DIRT(name) \
 	igt_subtest(#name "-dirt") { \
 		execbuf.flags = 0; \
