@@ -25,8 +25,6 @@
  *
  */
 
-#include "igt.h"
-#include "igt_sysfs.h"
 #include <limits.h>
 #include <stdbool.h>
 #include <unistd.h>
@@ -42,6 +40,10 @@
 #include <time.h>
 #include <signal.h>
 
+#include "i915/gem.h"
+#include "i915/gem_ring.h"
+#include "igt.h"
+#include "igt_sysfs.h"
 
 #define RS_NO_ERROR      0
 #define RS_BATCH_ACTIVE  (1 << 0)
@@ -64,16 +66,14 @@ struct local_drm_i915_reset_stats {
 
 #define GET_RESET_STATS_IOCTL DRM_IOWR(DRM_COMMAND_BASE + 0x32, struct local_drm_i915_reset_stats)
 
-#define LOCAL_I915_EXEC_VEBOX	(4 << 0)
+static int device;
 
 static void sync_gpu(void)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
-	gem_quiescent_gpu(fd);
-	close(fd);
+	gem_quiescent_gpu(device);
 }
 
-static int noop(int fd, uint32_t ctx, const struct intel_execution_engine *e)
+static int noop(int fd, uint32_t ctx, const struct intel_execution_ring *e)
 {
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 eb;
@@ -102,7 +102,7 @@ static int noop(int fd, uint32_t ctx, const struct intel_execution_engine *e)
 
 static int has_engine(int fd,
 		      uint32_t ctx,
-		      const struct intel_execution_engine *e)
+		      const struct intel_execution_ring *e)
 {
 	int handle = noop(fd, ctx, e);
 	if (handle < 0)
@@ -111,14 +111,10 @@ static int has_engine(int fd,
 	return 1;
 }
 
-static void check_context(const struct intel_execution_engine *e)
+static void check_context(const struct intel_execution_ring *e)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
-
-	gem_require_contexts(fd);
-	igt_require(has_engine(fd, gem_context_create(fd), e));
-
-	close(fd);
+	gem_require_contexts(device);
+	igt_require(has_engine(device, gem_context_create(device), e));
 }
 
 static int gem_reset_stats(int fd, int ctx_id,
@@ -157,7 +153,7 @@ static struct timespec ts_injected;
 #define BAN HANG_ALLOW_BAN
 #define ASYNC 2
 static void inject_hang(int fd, uint32_t ctx,
-			const struct intel_execution_engine *e,
+			const struct intel_execution_ring *e,
 			unsigned flags)
 {
 	igt_hang_t hang;
@@ -207,7 +203,7 @@ static int _assert_reset_status(int idx, int fd, int ctx, int status)
 #define assert_reset_status(idx, fd, ctx, status) \
 	igt_assert(_assert_reset_status(idx, fd, ctx, status) == 0)
 
-static void test_rs(const struct intel_execution_engine *e,
+static void test_rs(const struct intel_execution_ring *e,
 		    int num_fds, int hang_index, int rs_assumed_no_hang)
 {
 	int fd[MAX_FD];
@@ -219,7 +215,7 @@ static void test_rs(const struct intel_execution_engine *e,
 	igt_debug("num fds=%d, hang index=%d\n", num_fds, hang_index);
 
 	for (i = 0; i < num_fds; i++) {
-		fd[i] = drm_open_driver(DRIVER_INTEL);
+		fd[i] = gem_reopen_driver(device);
 		assert_reset_status(i, fd[i], 0, RS_NO_ERROR);
 	}
 
@@ -253,7 +249,7 @@ static void test_rs(const struct intel_execution_engine *e,
 }
 
 #define MAX_CTX 100
-static void test_rs_ctx(const struct intel_execution_engine *e,
+static void test_rs_ctx(const struct intel_execution_ring *e,
 			int num_fds, int num_ctx, int hang_index,
 			int hang_context)
 {
@@ -270,7 +266,7 @@ static void test_rs_ctx(const struct intel_execution_engine *e,
 	test_rs(e, num_fds, -1, RS_NO_ERROR);
 
 	for (i = 0; i < num_fds; i++) {
-		fd[i] = drm_open_driver(DRIVER_INTEL);
+		fd[i] = gem_reopen_driver(device);
 		igt_assert(fd[i]);
 		assert_reset_status(i, fd[i], 0, RS_NO_ERROR);
 
@@ -329,15 +325,15 @@ static void test_rs_ctx(const struct intel_execution_engine *e,
 	}
 }
 
-static void test_ban(const struct intel_execution_engine *e)
+static void test_ban(const struct intel_execution_ring *e)
 {
 	struct local_drm_i915_reset_stats rs_bad, rs_good;
 	int fd_bad, fd_good;
 	int ban, retry = 10;
 	int active_count = 0;
 
-	fd_bad = drm_open_driver(DRIVER_INTEL);
-	fd_good = drm_open_driver(DRIVER_INTEL);
+	fd_bad = gem_reopen_driver(device);
+	fd_good = gem_reopen_driver(device);
 
 	assert_reset_status(fd_bad, fd_bad, 0, RS_NO_ERROR);
 	assert_reset_status(fd_good, fd_good, 0, RS_NO_ERROR);
@@ -383,14 +379,14 @@ static void test_ban(const struct intel_execution_engine *e)
 	close(fd_good);
 }
 
-static void test_ban_ctx(const struct intel_execution_engine *e)
+static void test_ban_ctx(const struct intel_execution_ring *e)
 {
 	struct local_drm_i915_reset_stats rs_bad, rs_good;
 	int fd, ban, retry = 10;
 	uint32_t ctx_good, ctx_bad;
 	int active_count = 0;
 
-	fd = drm_open_driver(DRIVER_INTEL);
+	fd = gem_reopen_driver(device);
 
 	assert_reset_status(fd, fd, 0, RS_NO_ERROR);
 
@@ -441,13 +437,13 @@ static void test_ban_ctx(const struct intel_execution_engine *e)
 	close(fd);
 }
 
-static void test_unrelated_ctx(const struct intel_execution_engine *e)
+static void test_unrelated_ctx(const struct intel_execution_ring *e)
 {
 	int fd1,fd2;
 	int ctx_guilty, ctx_unrelated;
 
-	fd1 = drm_open_driver(DRIVER_INTEL);
-	fd2 = drm_open_driver(DRIVER_INTEL);
+	fd1 = gem_reopen_driver(device);
+	fd2 = gem_reopen_driver(device);
 	assert_reset_status(0, fd1, 0, RS_NO_ERROR);
 	assert_reset_status(1, fd2, 0, RS_NO_ERROR);
 	ctx_guilty = gem_context_create(fd1);
@@ -480,9 +476,9 @@ static int get_reset_count(int fd, int ctx)
 	return rs.reset_count;
 }
 
-static void test_close_pending_ctx(const struct intel_execution_engine *e)
+static void test_close_pending_ctx(const struct intel_execution_ring *e)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
+	int fd = gem_reopen_driver(device);
 	uint32_t ctx = gem_context_create(fd);
 
 	assert_reset_status(fd, fd, ctx, RS_NO_ERROR);
@@ -494,9 +490,9 @@ static void test_close_pending_ctx(const struct intel_execution_engine *e)
 	close(fd);
 }
 
-static void test_close_pending(const struct intel_execution_engine *e)
+static void test_close_pending(const struct intel_execution_ring *e)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
+	int fd = gem_reopen_driver(device);
 
 	assert_reset_status(fd, fd, 0, RS_NO_ERROR);
 
@@ -509,7 +505,7 @@ static void noop_on_each_ring(int fd, const bool reverse)
 	const uint32_t bbe = MI_BATCH_BUFFER_END;
 	struct drm_i915_gem_execbuffer2 eb;
 	struct drm_i915_gem_exec_object2 obj;
-	const struct intel_execution_engine *e;
+	const struct intel_execution_ring *e;
 
 	memset(&obj, 0, sizeof(obj));
 	obj.handle = gem_create(fd, 4096);
@@ -520,14 +516,14 @@ static void noop_on_each_ring(int fd, const bool reverse)
 	eb.buffer_count = 1;
 
 	if (reverse) {
-		for (e = intel_execution_engines; e->name; e++)
+		for (e = intel_execution_rings; e->name; e++)
 			;
-		while (--e >= intel_execution_engines) {
+		while (--e >= intel_execution_rings) {
 			eb.flags = eb_ring(e);
 			__gem_execbuf(fd, &eb);
 		}
 	} else {
-		for (e = intel_execution_engines; e->name; e++) {
+		for (e = intel_execution_rings; e->name; e++) {
 			eb.flags = eb_ring(e);
 			__gem_execbuf(fd, &eb);
 		}
@@ -537,10 +533,10 @@ static void noop_on_each_ring(int fd, const bool reverse)
 	gem_close(fd, obj.handle);
 }
 
-static void test_close_pending_fork(const struct intel_execution_engine *e,
+static void test_close_pending_fork(const struct intel_execution_ring *e,
 				    const bool reverse)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
+	int fd = gem_reopen_driver(device);
 	igt_hang_t hang;
 	int pid;
 
@@ -555,7 +551,7 @@ static void test_close_pending_fork(const struct intel_execution_engine *e,
 	 */
 	pid = fork();
 	if (pid == 0) {
-		const int fd2 = drm_open_driver(DRIVER_INTEL);
+		const int fd2 = gem_reopen_driver(device);
 		igt_assert_lte(0, fd2);
 
 		/* The crucial component is that we schedule the same noop batch
@@ -579,10 +575,10 @@ static void test_close_pending_fork(const struct intel_execution_engine *e,
 	close(fd);
 }
 
-static void test_reset_count(const struct intel_execution_engine *e,
+static void test_reset_count(const struct intel_execution_ring *e,
 			     const bool create_ctx)
 {
-	int fd = drm_open_driver(DRIVER_INTEL);
+	int fd = gem_reopen_driver(device);
 	int ctx;
 	long c1, c2;
 
@@ -685,7 +681,7 @@ static void test_params_ctx(void)
 {
 	int fd;
 
-	fd = drm_open_driver(DRIVER_INTEL);
+	fd = gem_reopen_driver(device);
 	_test_param(fd, gem_context_create(fd));
 	close(fd);
 }
@@ -694,18 +690,18 @@ static void test_params(void)
 {
 	int fd;
 
-	fd = drm_open_driver(DRIVER_INTEL);
+	fd = gem_reopen_driver(device);
 	_test_param(fd, 0);
 	close(fd);
 }
 
-static const struct intel_execution_engine *
-next_engine(int fd, const struct intel_execution_engine *e)
+static const struct intel_execution_ring *
+next_engine(int fd, const struct intel_execution_ring *e)
 {
 	do {
 		e++;
 		if (e->name == NULL)
-			e = intel_execution_engines;
+			e = intel_execution_rings;
 		if (e->exec_id == 0)
 			e++;
 	} while (!has_engine(fd, 0, e));
@@ -713,13 +709,13 @@ next_engine(int fd, const struct intel_execution_engine *e)
 	return e;
 }
 
-static void defer_hangcheck(const struct intel_execution_engine *engine)
+static void defer_hangcheck(const struct intel_execution_ring *engine)
 {
-	const struct intel_execution_engine *next;
+	const struct intel_execution_ring *next;
 	int fd, count_start, count_end;
 	int seconds = 30;
 
-	fd = drm_open_driver(DRIVER_INTEL);
+	fd = gem_reopen_driver(device);
 
 	next = next_engine(fd, engine);
 	igt_skip_on(next == engine);
@@ -772,25 +768,21 @@ static bool gem_has_reset_stats(int fd)
 
 igt_main
 {
-	const struct intel_execution_engine *e;
+	const struct intel_execution_ring *e;
 
 	igt_fixture {
-		int fd;
-
 		bool has_reset_stats;
 		bool using_full_reset;
-		fd = drm_open_driver(DRIVER_INTEL);
-		devid = intel_get_drm_devid(fd);
 
-		has_reset_stats = gem_has_reset_stats(fd);
+		device = drm_open_driver(DRIVER_INTEL);
+		devid = intel_get_drm_devid(device);
 
-		igt_assert(igt_sysfs_set_parameter
-			   (fd, "reset", "%d", 1 /* only global reset */));
+		has_reset_stats = gem_has_reset_stats(device);
 
-		using_full_reset = !gem_engine_reset_enabled(fd) &&
-				   gem_gpu_reset_enabled(fd);
+		igt_assert(igt_params_set(device, "reset", "%d", 1 /* only global reset */));
 
-		close(fd);
+		using_full_reset = !gem_engine_reset_enabled(device) &&
+				   gem_gpu_reset_enabled(device);
 
 		igt_require_f(has_reset_stats,
 			      "No reset stats ioctl support. Too old kernel?\n");
@@ -804,7 +796,7 @@ igt_main
 	igt_subtest_f("params-ctx")
 		RUN_TEST(test_params_ctx());
 
-	for (e = intel_execution_engines; e->name; e++) {
+	for (e = intel_execution_rings; e->name; e++) {
 		igt_subtest_f("reset-stats-%s", e->name)
 			RUN_TEST(test_rs(e, 4, 1, 0));
 
@@ -843,11 +835,7 @@ igt_main
 	}
 
 	igt_fixture {
-		int fd;
-
-		fd = drm_open_driver(DRIVER_INTEL);
-		igt_assert(igt_sysfs_set_parameter
-			   (fd, "reset", "%d", INT_MAX /* any reset method */));
-		close(fd);
+		igt_assert(igt_params_set(device, "reset", "%d", INT_MAX /* any reset method */));
+		close(device);
 	}
 }
