@@ -38,7 +38,6 @@
 
 #include <linux/userfaultfd.h>
 
-#include "igt.h"
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
@@ -56,12 +55,13 @@
 #include <pthread.h>
 #include <time.h>
 
-#include <linux/memfd.h>
-
 #include "drm.h"
 #include "i915_drm.h"
 
-#include "intel_bufmgr.h"
+#include "i915/gem.h"
+#include "igt.h"
+#include "igt_sysfs.h"
+#include "sw_sync.h"
 
 #include "eviction_common.c"
 
@@ -69,43 +69,16 @@
 #define PAGE_SIZE 4096
 #endif
 
-#define LOCAL_EXEC_OBJECT_SUPPORTS_48B (1 << 3)
-
-static uint32_t userptr_flags = LOCAL_I915_USERPTR_UNSYNCHRONIZED;
-
-static bool can_gtt_mmap;
+static uint32_t userptr_flags;
 
 #define WIDTH 512
 #define HEIGHT 512
 
 static uint32_t linear[WIDTH*HEIGHT];
 
-static bool has_gtt_mmap(int i915)
-{
-	void *ptr, *map = NULL;
-	uint32_t handle;
-
-	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
-
-	if (__gem_userptr(i915, ptr, 4096, 0,
-			  LOCAL_I915_USERPTR_UNSYNCHRONIZED, &handle))
-		goto out_ptr;
-	igt_assert(handle != 0);
-
-	map = __gem_mmap__gtt(i915, handle, 4096, PROT_WRITE);
-	if (map)
-		munmap(map, 4096);
-
-	gem_close(i915, handle);
-out_ptr:
-	free(ptr);
-
-	return map != NULL;
-}
-
 static void gem_userptr_test_unsynchronized(void)
 {
-	userptr_flags = LOCAL_I915_USERPTR_UNSYNCHRONIZED;
+	userptr_flags = I915_USERPTR_UNSYNCHRONIZED;
 }
 
 static void gem_userptr_test_synchronized(void)
@@ -174,26 +147,36 @@ static int copy(int fd, uint32_t dst, uint32_t src)
 	memset(obj, 0, sizeof(obj));
 
 	obj[exec.buffer_count].handle = dst;
-	obj[exec.buffer_count].flags = LOCAL_EXEC_OBJECT_SUPPORTS_48B;
+	obj[exec.buffer_count].flags = EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
 	exec.buffer_count++;
 
 	if (src != dst) {
 		obj[exec.buffer_count].handle = src;
-		obj[exec.buffer_count].flags = LOCAL_EXEC_OBJECT_SUPPORTS_48B;
+		obj[exec.buffer_count].flags = EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
 		exec.buffer_count++;
 	}
 
 	obj[exec.buffer_count].handle = handle;
 	obj[exec.buffer_count].relocation_count = 2;
 	obj[exec.buffer_count].relocs_ptr = to_user_pointer(reloc);
-	obj[exec.buffer_count].flags = LOCAL_EXEC_OBJECT_SUPPORTS_48B;
+	obj[exec.buffer_count].flags = EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
 	exec.buffer_count++;
 	exec.buffers_ptr = to_user_pointer(obj);
 	exec.flags = HAS_BLT_RING(intel_get_drm_devid(fd)) ? I915_EXEC_BLT : 0;
+	exec.flags |= I915_EXEC_FENCE_OUT;
 
-	ret = __gem_execbuf(fd, &exec);
+	ret = __gem_execbuf_wr(fd, &exec);
 	gem_close(fd, handle);
 
+	if (ret == 0) {
+		int fence = exec.rsvd2 >> 32;
+
+		sync_fence_wait(fence, -1);
+		if (sync_fence_status(fence) < 0)
+			ret = sync_fence_status(fence);
+		close(fence);
+	}
+
 	return ret;
 }
 
@@ -253,10 +236,10 @@ blit(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 	obj = calloc(n_bo + 1, sizeof(*obj));
 	for (n = 0; n < n_bo; n++) {
 		obj[n].handle = all_bo[n];
-		obj[n].flags = LOCAL_EXEC_OBJECT_SUPPORTS_48B;
+		obj[n].flags = EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
 	}
 	obj[n].handle = handle;
-	obj[n].flags = LOCAL_EXEC_OBJECT_SUPPORTS_48B;
+	obj[n].flags = EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
 	obj[n].relocation_count = 2;
 	obj[n].relocs_ptr = to_user_pointer(reloc);
 
@@ -274,7 +257,7 @@ blit(int fd, uint32_t dst, uint32_t src, uint32_t *all_bo, int n_bo)
 static void store_dword(int fd, uint32_t target,
 			uint32_t offset, uint32_t value)
 {
-	const int gen = intel_gen(intel_get_drm_devid(fd));
+	const unsigned int gen = intel_gen(intel_get_drm_devid(fd));
 	struct drm_i915_gem_exec_object2 obj[2];
 	struct drm_i915_gem_relocation_entry reloc;
 	struct drm_i915_gem_execbuffer2 execbuf;
@@ -478,14 +461,11 @@ static int has_userptr(int fd)
 {
 	uint32_t handle = 0;
 	void *ptr;
-	uint32_t oldflags;
 	int ret;
 
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
-	oldflags = userptr_flags;
-	gem_userptr_test_unsynchronized();
 	ret = __gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
-	userptr_flags = oldflags;
+	errno = 0;
 	if (ret != 0) {
 		free(ptr);
 		return 0;
@@ -499,7 +479,7 @@ static int has_userptr(int fd)
 
 static int test_input_checking(int fd)
 {
-	struct local_i915_gem_userptr userptr;
+	struct drm_i915_gem_userptr userptr;
 	int ret;
 
 	/* Invalid flags. */
@@ -507,7 +487,7 @@ static int test_input_checking(int fd)
 	userptr.user_ptr = 0;
 	userptr.user_size = 0;
 	userptr.flags = ~0;
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
+	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_USERPTR, &userptr);
 	igt_assert_neq(ret, 0);
 
 	/* Too big. */
@@ -515,7 +495,7 @@ static int test_input_checking(int fd)
 	userptr.user_ptr = 0;
 	userptr.user_size = ~0;
 	userptr.flags = 0;
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
+	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_USERPTR, &userptr);
 	igt_assert_neq(ret, 0);
 
 	/* Both wrong. */
@@ -523,7 +503,7 @@ static int test_input_checking(int fd)
 	userptr.user_ptr = 0;
 	userptr.user_size = ~0;
 	userptr.flags = ~0;
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
+	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_USERPTR, &userptr);
 	igt_assert_neq(ret, 0);
 
 	/* Zero user_size. */
@@ -531,12 +511,263 @@ static int test_input_checking(int fd)
 	userptr.user_ptr = 0;
 	userptr.user_size = 0;
 	userptr.flags = 0;
-	ret = drmIoctl(fd, LOCAL_IOCTL_I915_GEM_USERPTR, &userptr);
+	ret = drmIoctl(fd, DRM_IOCTL_I915_GEM_USERPTR, &userptr);
 	igt_assert_neq(ret, 0);
 
 	return 0;
 }
 
+static bool __enable_hangcheck(int dir, bool state)
+{
+	return igt_sysfs_set(dir, "enable_hangcheck", state ? "1" : "0");
+}
+
+static int __execbuf(int i915, struct drm_i915_gem_execbuffer2 *execbuf)
+{
+	int err;
+
+	err = 0;
+	if (ioctl(i915, DRM_IOCTL_I915_GEM_EXECBUFFER2_WR, execbuf)) {
+		err = -errno;
+		igt_assume(err);
+	}
+
+	errno = 0;
+	return err;
+}
+
+static void alarm_handler(int sig)
+{
+}
+
+static int fill_ring(int i915, struct drm_i915_gem_execbuffer2 *execbuf)
+{
+	struct sigaction old_sa, sa = { .sa_handler = alarm_handler };
+	int fence = execbuf->rsvd2 >> 32;
+	struct itimerval itv;
+	bool once = false;
+
+	sigaction(SIGALRM, &sa, &old_sa);
+	itv.it_interval.tv_sec = 0;
+	itv.it_interval.tv_usec = 1000;
+	itv.it_value.tv_sec = 0;
+	itv.it_value.tv_usec = 10000;
+	setitimer(ITIMER_REAL, &itv, NULL);
+
+	igt_assert(execbuf->flags & I915_EXEC_FENCE_OUT);
+	do {
+		int err = __execbuf(i915, execbuf);
+
+		if (err == 0) {
+			close(fence);
+			fence = execbuf->rsvd2 >> 32;
+			continue;
+		}
+
+		if (err == -EWOULDBLOCK || once)
+			break;
+
+		/* sleep until the next timer interrupt (woken on signal) */
+		pause();
+		once = true;
+	} while (1);
+
+	memset(&itv, 0, sizeof(itv));
+	setitimer(ITIMER_REAL, &itv, NULL);
+	sigaction(SIGALRM, &old_sa, NULL);
+
+	return fence;
+}
+
+static void test_nohangcheck_hostile(int i915)
+{
+	const struct intel_execution_engine2 *e;
+	igt_hang_t hang;
+	uint32_t ctx;
+	int fence = -1;
+	int err = 0;
+	int dir;
+
+	/*
+	 * Even if the user disables hangcheck, we must still recover.
+	 */
+
+	i915 = gem_reopen_driver(i915);
+	gem_require_contexts(i915);
+
+	dir = igt_params_open(i915);
+	igt_require(dir != -1);
+
+	ctx = gem_context_create(i915);
+	hang = igt_allow_hang(i915, ctx, 0);
+	igt_require(__enable_hangcheck(dir, false));
+
+	____for_each_physical_engine(i915, ctx, e) {
+		igt_spin_t *spin;
+		int new;
+
+		/* Set a fast hang detection to speed up the test */
+		gem_engine_property_printf(i915, e->name,
+					   "preempt_timeout_ms", "%d", 50);
+
+		spin = __igt_spin_new(i915, ctx,
+				      .engine = e->flags,
+				      .flags = (IGT_SPIN_NO_PREEMPTION |
+						IGT_SPIN_USERPTR |
+						IGT_SPIN_FENCE_OUT));
+
+		new = fill_ring(i915, &spin->execbuf);
+		igt_assert(new != -1);
+		spin->out_fence = -1;
+
+		if (fence < 0) {
+			fence = new;
+		} else {
+			int tmp;
+
+			tmp = sync_fence_merge(fence, new);
+			close(fence);
+			close(new);
+
+			fence = tmp;
+		}
+	}
+	gem_context_destroy(i915, ctx);
+	igt_assert(fence != -1);
+
+	if (sync_fence_wait(fence, MSEC_PER_SEC)) { /* 640ms preempt-timeout */
+		igt_debugfs_dump(i915, "i915_engine_info");
+		err = -ETIME;
+	}
+
+	__enable_hangcheck(dir, true);
+	gem_quiescent_gpu(i915);
+	igt_disallow_hang(i915, hang);
+
+	igt_assert_f(err == 0,
+		     "Hostile unpreemptable userptr was not cancelled immediately upon closure\n");
+
+	igt_assert_eq(sync_fence_status(fence), -EIO);
+	close(fence);
+
+	close(dir);
+	close(i915);
+}
+
+static size_t hugepagesize(void)
+{
+#define LINE "Hugepagesize:"
+	size_t sz = 2 << 20;
+	char *line = NULL;
+	size_t len = 0;
+	FILE *file;
+
+	file = fopen("/proc/meminfo", "r");
+	if (!file)
+		return sz;
+
+	while (getline(&line, &len, file) > 0) {
+		if (strncmp(line, LINE, strlen(LINE)))
+			continue;
+
+		if (sscanf(line + strlen(LINE), "%zu", &sz) == 1) {
+			sz <<= 10;
+			igt_debug("Found huge page size: %zu\n", sz);
+		}
+		break;
+	}
+	free(line);
+	fclose(file);
+
+	return sz;
+#undef LINE
+}
+
+static void test_vma_merge(int i915)
+{
+	const size_t sz = 2 * hugepagesize();
+	igt_spin_t *spin;
+	uint32_t handle;
+	void *addr;
+
+	addr = mmap(NULL, sz, PROT_READ | PROT_WRITE,
+		    MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+
+	gem_userptr(i915, addr + sz / 2, 4096, 0, userptr_flags, &handle);
+
+	spin = igt_spin_new(i915, .dependency = handle, .flags = IGT_SPIN_FENCE_OUT);
+	igt_assert(gem_bo_busy(i915, handle));
+
+	for (size_t x = 0; x < sz; x += 4096) {
+		if (x == sz / 2)
+			continue;
+
+		igt_assert(mmap(addr + x, 4096, PROT_READ | PROT_WRITE,
+				MAP_FIXED | MAP_SHARED | MAP_ANON, -1, 0) !=
+			   MAP_FAILED);
+	}
+
+	igt_spin_end(spin);
+	gem_close(i915, handle);
+
+	munmap(addr, sz);
+
+	gem_sync(i915, spin->handle);
+	igt_assert_eq(sync_fence_status(spin->out_fence), 1);
+	igt_spin_free(i915, spin);
+}
+
+static void test_huge_split(int i915)
+{
+	const size_t sz = 2 * hugepagesize();
+	unsigned int flags;
+	igt_spin_t *spin;
+	uint32_t handle;
+	void *addr;
+
+	flags = MFD_HUGETLB;
+#if defined(MFD_HUGE_2MB)
+	flags |= MFD_HUGE_2MB;
+#endif
+
+	do {
+		int memfd;
+
+		memfd = memfd_create("huge", flags);
+		igt_require(memfd != -1);
+		igt_require(ftruncate(memfd, sz) == 0);
+
+		addr = mmap(NULL, sz, PROT_WRITE, MAP_SHARED, memfd, 0);
+		close(memfd);
+		if (addr != MAP_FAILED)
+			break;
+
+		igt_require_f(flags, "memfd not supported\n");
+		flags = 0;
+	} while (1);
+	madvise(addr, sz, MADV_HUGEPAGE);
+
+	gem_userptr(i915, addr + sz / 2 - 4096, 8192, 0, userptr_flags, &handle);
+	spin = igt_spin_new(i915, .dependency = handle, .flags = IGT_SPIN_FENCE_OUT);
+	igt_assert(gem_bo_busy(i915, handle));
+
+	igt_assert(mmap(addr, 4096, PROT_READ,
+			MAP_FIXED | MAP_SHARED | MAP_ANONYMOUS, -1, 0) !=
+		   MAP_FAILED);
+	igt_assert(mmap(addr + sz - 4096, 4096, PROT_READ,
+			MAP_FIXED | MAP_SHARED | MAP_ANONYMOUS, -1, 0) !=
+		   MAP_FAILED);
+
+	igt_spin_end(spin);
+	gem_close(i915, handle);
+
+	munmap(addr, sz);
+
+	gem_sync(i915, spin->handle);
+	igt_assert_eq(sync_fence_status(spin->out_fence), 1);
+	igt_spin_free(i915, spin);
+}
+
 static int test_access_control(int fd)
 {
 	/* CAP_SYS_ADMIN is needed for UNSYNCHRONIZED mappings. */
@@ -642,24 +873,14 @@ static int test_invalid_mapping(int fd, const struct mmap_offset *t)
 	return 0;
 }
 
-#define PE_GTT_MAP 0x1
-#define PE_BUSY 0x2
+#define PE_BUSY 0x1
 static void test_process_exit(int fd, int flags)
 {
-	if (flags & PE_GTT_MAP)
-		igt_require(can_gtt_mmap);
-
 	igt_fork(child, 1) {
 		uint32_t handle;
 
 		handle = create_userptr_bo(fd, sizeof(linear));
 
-		if (flags & PE_GTT_MAP) {
-			uint32_t *ptr = __gem_mmap__gtt(fd, handle, sizeof(linear), PROT_READ | PROT_WRITE);
-			if (ptr)
-				*ptr = 0;
-		}
-
 		if (flags & PE_BUSY)
 			igt_assert_eq(copy(fd, handle, handle), 0);
 	}
@@ -672,10 +893,9 @@ static void test_forked_access(int fd)
 	void *ptr1 = NULL, *ptr2 = NULL;
 	int ret;
 
-	ret = posix_memalign(&ptr1, PAGE_SIZE, sizeof(linear));
-#ifdef MADV_DONTFORK
-	ret |= madvise(ptr1, sizeof(linear), MADV_DONTFORK);
-#endif
+	ptr1 = mmap(NULL, sizeof(linear), PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+	igt_assert(ptr1 != MAP_FAILED);
+
 	gem_userptr(fd, ptr1, sizeof(linear), 0, userptr_flags, &handle1);
 	igt_assert(ptr1);
 	igt_assert(handle1);
@@ -691,8 +911,17 @@ static void test_forked_access(int fd)
 	memset(ptr1, 0x1, sizeof(linear));
 	memset(ptr2, 0x2, sizeof(linear));
 
-	igt_fork(child, 1)
-		igt_assert_eq(copy(fd, handle1, handle2), 0);
+	igt_fork(child, 1) {
+		ret = copy(fd, handle1, handle2);
+		if (ret) {
+			/*
+			 * userptr being exportable is a misfeature,
+			 * and has now been disallowed
+			 */
+			igt_assert_eq(ret, -EFAULT);
+			memset(ptr1, 0x2, sizeof(linear));
+		}
+	}
 	igt_waitchildren();
 
 	gem_userptr_sync(fd, handle1);
@@ -703,11 +932,7 @@ static void test_forked_access(int fd)
 
 	igt_assert(memcmp(ptr1, ptr2, sizeof(linear)) == 0);
 
-#ifdef MADV_DOFORK
-	ret = madvise(ptr1, sizeof(linear), MADV_DOFORK);
-	igt_assert_eq(ret, 0);
-#endif
-	free(ptr1);
+	munmap(ptr1, sizeof(linear));
 
 #ifdef MADV_DOFORK
 	ret = madvise(ptr2, sizeof(linear), MADV_DOFORK);
@@ -718,10 +943,8 @@ static void test_forked_access(int fd)
 
 #define MAP_FIXED_INVALIDATE_OVERLAP	(1<<0)
 #define MAP_FIXED_INVALIDATE_BUSY	(1<<1)
-#define MAP_FIXED_INVALIDATE_GET_PAGES	(1<<2)
 #define ALL_MAP_FIXED_INVALIDATE (MAP_FIXED_INVALIDATE_OVERLAP | \
-				  MAP_FIXED_INVALIDATE_BUSY | \
-				  MAP_FIXED_INVALIDATE_GET_PAGES)
+				  MAP_FIXED_INVALIDATE_BUSY)
 
 static int test_map_fixed_invalidate(int fd, uint32_t flags,
 				     const struct mmap_offset *t)
@@ -760,12 +983,6 @@ static int test_map_fixed_invalidate(int fd, uint32_t flags,
 			      "HW & kernel support for mmap_offset(%s)\n",
 			      t->name);
 
-		if (flags & MAP_FIXED_INVALIDATE_GET_PAGES)
-			igt_assert_eq(__gem_set_domain(fd, handle[0],
-						       I915_GEM_DOMAIN_GTT,
-						       I915_GEM_DOMAIN_GTT),
-				      0);
-
 		if (flags & MAP_FIXED_INVALIDATE_BUSY)
 			igt_assert_eq(copy(fd, handle[0], handle[num_handles-1]), 0);
 
@@ -779,17 +996,6 @@ static int test_map_fixed_invalidate(int fd, uint32_t flags,
 		gem_set_tiling(fd, mmap_offset.handle, I915_TILING_NONE, 0);
 		*map = 0xdead;
 
-		if (flags & MAP_FIXED_INVALIDATE_GET_PAGES) {
-			igt_assert_eq(__gem_set_domain(fd, handle[0],
-						       I915_GEM_DOMAIN_GTT,
-						       I915_GEM_DOMAIN_GTT),
-				      -EFAULT);
-
-			/* Errors are permanent, so we have to recreate */
-			gem_close(fd, handle[0]);
-			handle[0] = create_userptr(fd, 0, ptr + PAGE_SIZE/sizeof(*ptr));
-		}
-
 		gem_set_tiling(fd, mmap_offset.handle, I915_TILING_Y, 512 * 4);
 		*(uint32_t*)map = 0xbeef;
 
@@ -803,53 +1009,30 @@ static int test_map_fixed_invalidate(int fd, uint32_t flags,
 	return 0;
 }
 
-static void test_mmap_offset_invalidate(int fd,
-					const struct mmap_offset *t,
-					unsigned int flags)
-#define MMOI_ACTIVE (1u << 0)
+static void test_mmap_offset_banned(int fd, const struct mmap_offset *t)
 {
-	igt_spin_t *spin = NULL;
-	uint32_t handle;
-	uint32_t *map;
+	struct drm_i915_gem_mmap_offset arg;
 	void *ptr;
 
 	/* check if mmap_offset type is supported by hardware, skip if not */
-	handle = gem_create(fd, PAGE_SIZE);
-	map = __gem_mmap_offset(fd, handle, 0, PAGE_SIZE,
-				PROT_READ | PROT_WRITE, t->type);
-	igt_require_f(map,
-		      "HW & kernel support for mmap_offset(%s)\n", t->name);
-	munmap(map, PAGE_SIZE);
-	gem_close(fd, handle);
+	memset(&arg, 0, sizeof(arg));
+	arg.flags = t->type;
+	arg.handle = gem_create(fd, PAGE_SIZE);
+	igt_skip_on_f(igt_ioctl(fd, DRM_IOCTL_I915_GEM_MMAP_OFFSET, &arg),
+				"HW & kernel support for mmap_offset(%s)\n", t->name);
+	gem_close(fd, arg.handle);
 
 	/* create userptr object */
+	memset(&arg, 0, sizeof(arg));
+	arg.flags = t->type;
 	igt_assert_eq(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE), 0);
-	gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
+	gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &arg.handle);
 
-	/* set up mmap-offset mapping on top of the object, skip if refused */
-	map = __gem_mmap_offset(fd, handle, 0, PAGE_SIZE,
-				PROT_READ | PROT_WRITE, t->type);
-	igt_require_f(map, "mmap-offset banned, lockdep loop prevention\n");
+	/* try to set up mmap-offset mapping on top of the object, fail if not banned */
+	do_ioctl_err(fd, DRM_IOCTL_I915_GEM_MMAP_OFFSET, &arg, ENODEV);
 
-	/* set object pages in order to activate MMU notifier for it */
-	gem_set_domain(fd, handle, t->domain, t->domain);
-	*map = 0;
-
-	if (flags & MMOI_ACTIVE) {
-		gem_quiescent_gpu(fd);
-		spin = igt_spin_new(fd,
-				    .dependency = handle,
-				    .flags = IGT_SPIN_NO_PREEMPTION);
-		igt_spin_set_timeout(spin, NSEC_PER_SEC); /* XXX borked */
-	}
-
-	/* trigger the notifier */
+	gem_close(fd, arg.handle);
 	munmap(ptr, PAGE_SIZE);
-
-	/* cleanup */
-	igt_spin_free(fd, spin);
-	munmap(map, PAGE_SIZE);
-	gem_close(fd, handle);
 }
 
 static int test_forbidden_ops(int fd)
@@ -859,6 +1042,7 @@ static int test_forbidden_ops(int fd)
 	uint32_t handle;
 	void *ptr;
 
+	gem_require_pread_pwrite(fd);
 	igt_assert(posix_memalign(&ptr, PAGE_SIZE, PAGE_SIZE) == 0);
 	gem_userptr(fd, ptr, PAGE_SIZE, 0, userptr_flags, &handle);
 
@@ -936,16 +1120,11 @@ static volatile long sigbus_cnt = -1;
 static void *umap(int fd, uint32_t handle)
 {
 	void *ptr;
+	uint32_t tmp = gem_create(fd, sizeof(linear));
 
-	if (can_gtt_mmap) {
-		ptr = gem_mmap__gtt(fd, handle, sizeof(linear),
-				    PROT_READ | PROT_WRITE);
-	} else {
-		uint32_t tmp = gem_create(fd, sizeof(linear));
-		igt_assert_eq(copy(fd, tmp, handle), 0);
-		ptr = gem_mmap__cpu(fd, tmp, 0, sizeof(linear), PROT_READ);
-		gem_close(fd, tmp);
-	}
+	igt_assert_eq(copy(fd, tmp, handle), 0);
+	ptr = gem_mmap__cpu(fd, tmp, 0, sizeof(linear), PROT_READ);
+	gem_close(fd, tmp);
 
 	return ptr;
 }
@@ -954,7 +1133,6 @@ static void
 check_bo(int fd1, uint32_t handle1, int is_userptr, int fd2, uint32_t handle2)
 {
 	unsigned char *ptr1, *ptr2;
-	unsigned long size = sizeof(linear);
 
 	ptr2 = umap(fd2, handle2);
 	if (is_userptr)
@@ -968,12 +1146,6 @@ check_bo(int fd1, uint32_t handle1, int is_userptr, int fd2, uint32_t handle2)
 	sigbus_start = (unsigned long)ptr2;
 	igt_assert(memcmp(ptr1, ptr2, sizeof(linear)) == 0);
 
-	if (can_gtt_mmap) {
-		counter++;
-		memset(ptr1, counter, size);
-		memset(ptr2, counter, size);
-	}
-
 	if (!is_userptr)
 		munmap(ptr1, sizeof(linear));
 	munmap(ptr2, sizeof(linear));
@@ -996,30 +1168,6 @@ static int export_handle(int fd, uint32_t handle, int *outfd)
 	return ret;
 }
 
-static void sigbus(int sig, siginfo_t *info, void *param)
-{
-	unsigned long ptr = (unsigned long)info->si_addr;
-	void *addr;
-
-	if (ptr >= sigbus_start &&
-	    ptr < sigbus_start + sizeof(linear)) {
-		/* replace mapping to allow progress */
-		munmap((void *)sigbus_start, sizeof(linear));
-		addr = mmap((void *)sigbus_start, sizeof(linear),
-			    PROT_READ | PROT_WRITE,
-			    MAP_ANONYMOUS | MAP_PRIVATE | MAP_FIXED, -1, 0);
-		igt_assert((unsigned long)addr == sigbus_start);
-		memset(addr, counter, sizeof(linear));
-
-		sigbus_cnt++;
-		return;
-	}
-
-	if (orig_sigbus)
-		orig_sigbus(sig, info, param);
-	igt_assert(0);
-}
-
 static int test_dmabuf(void)
 {
 	int fd1, fd2;
@@ -1033,13 +1181,13 @@ static int test_dmabuf(void)
 	memset(get_handle_ptr(handle), counter, sizeof(linear));
 
 	ret = export_handle(fd1, handle, &dma_buf_fd);
-	if (userptr_flags & LOCAL_I915_USERPTR_UNSYNCHRONIZED && ret) {
+	if (userptr_flags & I915_USERPTR_UNSYNCHRONIZED && ret) {
 		igt_assert(ret == EINVAL || ret == ENODEV);
 		free_userptr_bo(fd1, handle);
 		close(fd1);
 		return 0;
 	} else {
-		igt_assert_eq(ret, 0);
+		igt_require(ret == 0);
 		igt_assert_lte(0, dma_buf_fd);
 	}
 
@@ -1055,25 +1203,6 @@ static int test_dmabuf(void)
 	free_userptr_bo(fd1, handle);
 	close(fd1);
 
-	if (can_gtt_mmap) {
-		struct sigaction sigact, orig_sigact;
-
-		memset(&sigact, 0, sizeof(sigact));
-		sigact.sa_sigaction = sigbus;
-		sigact.sa_flags = SA_SIGINFO;
-		ret = sigaction(SIGBUS, &sigact, &orig_sigact);
-		igt_assert_eq(ret, 0);
-
-		orig_sigbus = orig_sigact.sa_sigaction;
-
-		sigbus_cnt = 0;
-		check_bo(fd2, handle_import, -1, fd2, handle_import);
-		igt_assert(sigbus_cnt > 0);
-
-		ret = sigaction(SIGBUS, &orig_sigact, NULL);
-		igt_assert_eq(ret, 0);
-	}
-
 	close(fd2);
 	reset_handle_ptr();
 
@@ -1084,7 +1213,7 @@ static void store_dword_rand(int i915, unsigned int engine,
 			     uint32_t target, uint64_t sz,
 			     int count)
 {
-	const int gen = intel_gen(intel_get_drm_devid(i915));
+	const unsigned int gen = intel_gen(intel_get_drm_devid(i915));
 	struct drm_i915_gem_relocation_entry *reloc;
 	struct drm_i915_gem_exec_object2 obj[2];
 	struct drm_i915_gem_execbuffer2 exec;
@@ -1099,7 +1228,7 @@ static void store_dword_rand(int i915, unsigned int engine,
 
 	memset(obj, 0, sizeof(obj));
 	obj[0].handle = target;
-	obj[0].flags = LOCAL_EXEC_OBJECT_SUPPORTS_48B;
+	obj[0].flags = EXEC_OBJECT_SUPPORTS_48B_ADDRESS;
 	obj[1].handle = gem_create(i915, batchsz);
 	obj[1].relocation_count = count;
 	obj[1].relocs_ptr = to_user_pointer(reloc);
@@ -1155,6 +1284,7 @@ static void test_readonly(int i915)
 {
 	uint64_t aperture_size;
 	uint32_t whandle, rhandle;
+	struct timespec tv;
 	size_t sz, total;
 	void *pages, *space;
 	int memfd;
@@ -1189,6 +1319,7 @@ static void test_readonly(int i915)
 	 * the target address for MI_STORE_DWORD_IMM, so our maximum
 	 * usable object size is only 2GiB. For now.
 	 */
+	igt_nsec_elapsed(memset(&tv, 0, sizeof(tv)));
 	total = 2048ull << 20;
 	aperture_size = gem_aperture_size(i915) / 2;
 	if (aperture_size < total)
@@ -1209,32 +1340,35 @@ static void test_readonly(int i915)
 	igt_assert_eq_u32(*(uint32_t *)pages, (uint32_t)(total - sz));
 	igt_assert(mlock(pages, sz) == 0);
 	close(memfd);
+	igt_info("Arena creation in %.1fms\n", igt_nsec_elapsed(&tv) * 1e-6);
 
 	/* Check we can create a normal userptr bo wrapping the wrapper */
+	igt_nsec_elapsed(memset(&tv, 0, sizeof(tv)));
 	gem_userptr(i915, space, total, false, userptr_flags, &rhandle);
 	gem_set_domain(i915, rhandle, I915_GEM_DOMAIN_CPU, 0);
-	for (size_t offset = 0; offset < total; offset += sz)
-		store_dword(i915, rhandle, offset + 4, offset / sz);
+	store_dword(i915, rhandle, total - sz + 4, total / sz);
 	gem_sync(i915, rhandle);
 	igt_assert_eq_u32(*(uint32_t *)(pages + 0), (uint32_t)(total - sz));
-	igt_assert_eq_u32(*(uint32_t *)(pages + 4), (uint32_t)(total / sz - 1));
+	igt_assert_eq_u32(*(uint32_t *)(pages + 4), (uint32_t)(total / sz));
 	gem_close(i915, rhandle);
+	igt_info("Sanity check took %.1fms\n", igt_nsec_elapsed(&tv) * 1e-6);
 
 	/* Now enforce read-only henceforth */
 	igt_assert(mprotect(space, total, PROT_READ) == 0);
 
 	igt_fork(child, 1) {
+		const struct intel_execution_engine2 *e;
 		char *orig;
 
 		orig = g_compute_checksum_for_data(G_CHECKSUM_SHA1, pages, sz);
 
 		gem_userptr(i915, space, total, true, userptr_flags, &rhandle);
 
-		for_each_engine(e, i915) {
+		__for_each_physical_engine(i915, e) {
 			char *ref, *result;
 
 			/* First tweak the backing store through the write */
-			store_dword_rand(i915, eb_ring(e), whandle, sz, 1024);
+			store_dword_rand(i915, e->flags, whandle, sz, 64);
 			gem_sync(i915, whandle);
 			ref = g_compute_checksum_for_data(G_CHECKSUM_SHA1,
 							  pages, sz);
@@ -1243,7 +1377,7 @@ static void test_readonly(int i915)
 			igt_assert(strcmp(ref, orig));
 
 			/* Now try the same through the read-only handle */
-			store_dword_rand(i915, eb_ring(e), rhandle, total, 1024);
+			store_dword_rand(i915, e->flags, rhandle, total, 64);
 			gem_sync(i915, rhandle);
 			result = g_compute_checksum_for_data(G_CHECKSUM_SHA1,
 							     pages, sz);
@@ -1271,84 +1405,6 @@ static void test_readonly(int i915)
 	munmap(pages, sz);
 }
 
-static jmp_buf sigjmp;
-static void sigjmp_handler(int sig)
-{
-	siglongjmp(sigjmp, sig);
-}
-
-static void test_readonly_mmap(int i915)
-{
-	char *original, *result;
-	uint32_t handle;
-	uint32_t sz;
-	void *pages;
-	void *ptr;
-	int sig;
-
-	/*
-	 * A quick check to ensure that we cannot circumvent the
-	 * read-only nature of our memory by creating a GTT mmap into
-	 * the pages. Imagine receiving a readonly SHM segment from
-	 * another process, or a readonly file mmap, it must remain readonly
-	 * on the GPU as well.
-	 */
-
-	igt_require(igt_setup_clflush());
-
-	sz = 16 << 12;
-	pages = mmap(NULL, sz, PROT_WRITE, MAP_ANON | MAP_PRIVATE, -1, 0);
-	igt_assert(pages != MAP_FAILED);
-
-	igt_require(__gem_userptr(i915, pages, sz, true, userptr_flags, &handle) == 0);
-	gem_set_caching(i915, handle, 0);
-
-	memset(pages, 0xa5, sz);
-	igt_clflush_range(pages, sz);
-	original = g_compute_checksum_for_data(G_CHECKSUM_SHA1, pages, sz);
-
-	ptr = __gem_mmap__gtt(i915, handle, sz, PROT_WRITE);
-	igt_assert(ptr == NULL);
-
-	/* Optional kernel support for GTT mmaps of userptr */
-	ptr = __gem_mmap__gtt(i915, handle, sz, PROT_READ);
-	gem_close(i915, handle);
-
-	if (ptr) { /* Check that a write into the GTT readonly map fails */
-		if (!(sig = sigsetjmp(sigjmp, 1))) {
-			signal(SIGBUS, sigjmp_handler);
-			signal(SIGSEGV, sigjmp_handler);
-			memset(ptr, 0x5a, sz);
-			igt_assert(0);
-		}
-		igt_assert_eq(sig, SIGSEGV);
-
-		/* Check that we disallow removing the readonly protection */
-		igt_assert(mprotect(ptr, sz, PROT_WRITE));
-		if (!(sig = sigsetjmp(sigjmp, 1))) {
-			signal(SIGBUS, sigjmp_handler);
-			signal(SIGSEGV, sigjmp_handler);
-			memset(ptr, 0x5a, sz);
-			igt_assert(0);
-		}
-		igt_assert_eq(sig, SIGSEGV);
-
-		/* A single read from the GTT pointer to prove that works */
-		igt_assert_eq_u32(*(uint8_t *)ptr, 0xa5);
-		munmap(ptr, sz);
-	}
-
-	/* Double check that the kernel did indeed not let any writes through */
-	igt_clflush_range(pages, sz);
-	result = g_compute_checksum_for_data(G_CHECKSUM_SHA1, pages, sz);
-	igt_assert(!strcmp(original, result));
-
-	g_free(original);
-	g_free(result);
-
-	munmap(pages, sz);
-}
-
 static void test_readonly_pwrite(int i915)
 {
 	char *original, *result;
@@ -1363,6 +1419,7 @@ static void test_readonly_pwrite(int i915)
 	 */
 
 	igt_require(igt_setup_clflush());
+	gem_require_pread_pwrite(i915);
 
 	sz = 16 << 12;
 	pages = mmap(NULL, sz, PROT_WRITE, MAP_ANON | MAP_PRIVATE, -1, 0);
@@ -1562,12 +1619,45 @@ static int can_swap(void)
 	return 1;
 }
 
+static bool forked_userptr(int fd)
+{
+	uint32_t handle = 0;
+	int *ptr = NULL;
+	uint32_t ofs = sizeof(linear) / sizeof(*ptr);
+	int ret;
+
+	ptr = mmap(NULL, 2 * sizeof(linear), PROT_READ | PROT_WRITE, MAP_SHARED | MAP_ANONYMOUS, -1, 0);
+	igt_assert(ptr != MAP_FAILED);
+
+	ptr[ofs] = -1;
+
+	gem_userptr(fd, ptr, sizeof(linear), 0, userptr_flags, &handle);
+	igt_assert(handle);
+
+	igt_fork(child, 1)
+		ptr[ofs] = copy(fd, handle, handle);
+
+	igt_waitchildren();
+	ret = ptr[ofs];
+
+	gem_close(fd, handle);
+
+	munmap(ptr, 2 * sizeof(linear));
+
+	if (ret)
+		igt_assert_eq(ret, -EFAULT);
+
+	return !ret;
+}
+
 static void test_forking_evictions(int fd, int size, int count,
 			     unsigned flags)
 {
 	int trash_count;
 	int num_threads;
 
+	igt_require(forked_userptr(fd));
+
 	trash_count = intel_get_total_ram_mb() * 11 / 10;
 	/* Use the fact test will spawn a number of child
 	 * processes meaning swapping will be triggered system
@@ -1731,9 +1821,7 @@ static void test_unmap_after_close(int fd)
 
 static void test_unmap_cycles(int fd, int expected)
 {
-	int i;
-
-	for (i = 0; i < 1000; i++)
+	igt_until_timeout(5)
 		test_unmap(fd, expected);
 }
 
@@ -1886,6 +1974,78 @@ static void test_invalidate_close_race(int fd, bool overlap, int timeout)
 	free(t_data.ptr);
 }
 
+static void test_sd_probe(int i915)
+{
+	const int domains[] = {
+		I915_GEM_DOMAIN_CPU,
+		I915_GEM_DOMAIN_GTT,
+	};
+
+	/*
+	 * Quick and simple test to verify that GEM_SET_DOMAIN can
+	 * be used to probe the existence of the userptr, as used
+	 * by mesa and ddx.
+	 */
+
+	for (int idx = 0; idx < ARRAY_SIZE(domains); idx++) {
+		uint32_t handle;
+		void *page;
+
+		page = mmap(NULL, 4096, PROT_READ | PROT_WRITE,
+			    MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+
+		gem_userptr(i915, page, 4096, 0, 0, &handle);
+		igt_assert_eq(__gem_set_domain(i915, handle, domains[idx], 0),
+			      0);
+		gem_close(i915, handle);
+
+		munmap(page, 4096);
+
+		gem_userptr(i915, page, 4096, 0, 0, &handle);
+		igt_assert_eq(__gem_set_domain(i915, handle, domains[idx], 0),
+			      -EFAULT);
+		gem_close(i915, handle);
+	}
+}
+
+static void test_set_caching(int i915)
+{
+	const int levels[] = {
+		I915_CACHING_NONE,
+		I915_CACHING_CACHED,
+	};
+	uint32_t handle;
+	void *page;
+
+	page = mmap(NULL, 4096, PROT_READ | PROT_WRITE,
+		    MAP_ANONYMOUS | MAP_PRIVATE, -1, 0);
+
+	/*
+	 * A userptr is regular GEM object, mapping system pages from the user
+	 * into the GPU. The GPU knows no difference in the pages, and may use
+	 * the regular PTE cache levels. As does mesa.
+	 *
+	 * We could try and detect the different effects of cache levels, but
+	 * for the moment trust that set-cache-level works and reduces the
+	 * problem to other tests.
+	 */
+
+	for (int idx = 0; idx < ARRAY_SIZE(levels); idx++) {
+		gem_userptr(i915, page, 4096, 0, 0, &handle);
+		igt_assert_eq(__gem_set_caching(i915, handle, levels[idx]), 0);
+		gem_close(i915, handle);
+	}
+
+	gem_userptr(i915, page, 4096, 0, 0, &handle);
+	for (int idx = 0; idx < ARRAY_SIZE(levels); idx++)
+		igt_assert_eq(__gem_set_caching(i915, handle, levels[idx]), 0);
+	for (int idx = 0; idx < ARRAY_SIZE(levels); idx++)
+		igt_assert_eq(__gem_set_caching(i915, handle, levels[idx]), 0);
+	gem_close(i915, handle);
+
+	munmap(page, 4096);
+}
+
 struct ufd_thread {
 	uint32_t *page;
 	int i915;
@@ -1973,7 +2133,6 @@ static void test_userfault(int i915)
 	reg.range.start = to_user_pointer(t.page);
 	reg.range.len = 4096;
 	do_ioctl(ufd, UFFDIO_REGISTER, &reg);
-	igt_assert(reg.ioctls == UFFD_API_RANGE_IOCTLS);
 
 	igt_assert(pthread_create(&thread, NULL, ufd_thread, &t) == 0);
 
@@ -2022,12 +2181,17 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 	int size = sizeof(linear);
 
 	igt_fixture {
+		unsigned int mmo_max = 0;
+
 		fd = drm_open_driver(DRIVER_INTEL);
 		igt_assert(fd >= 0);
 		igt_require_gem(fd);
 		gem_require_blitter(fd);
 
-		can_gtt_mmap = has_gtt_mmap(fd) && gem_has_llc(fd);
+		for_each_mmap_offset_type(fd, t)
+			if (t->type >= mmo_max)
+				mmo_max = t->type + 1;
+		igt_assert(mmo_max);
 
 		size = sizeof(linear);
 
@@ -2048,6 +2212,10 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 
 	igt_subtest_group {
 		igt_fixture {
+			/* Either mode will do for parameter checking */
+			gem_userptr_test_synchronized();
+			if (!has_userptr(fd))
+				gem_userptr_test_unsynchronized();
 			igt_require(has_userptr(fd));
 		}
 
@@ -2066,6 +2234,12 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		igt_subtest("forbidden-operations")
 			test_forbidden_ops(fd);
 
+		igt_subtest("sd-probe")
+			test_sd_probe(fd);
+
+		igt_subtest("set-cache-level")
+			test_set_caching(fd);
+
 		igt_subtest("userfault")
 			test_userfault(fd);
 
@@ -2110,9 +2284,6 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		igt_subtest("readonly-unsync")
 			test_readonly(fd);
 
-		igt_subtest("readonly-mmap-unsync")
-			test_readonly_mmap(fd);
-
 		igt_subtest("readonly-pwrite-unsync")
 			test_readonly_pwrite(fd);
 
@@ -2184,15 +2355,9 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		igt_subtest("process-exit")
 			test_process_exit(fd, 0);
 
-		igt_subtest("process-exit-gtt")
-			test_process_exit(fd, PE_GTT_MAP);
-
 		igt_subtest("process-exit-busy")
 			test_process_exit(fd, PE_BUSY);
 
-		igt_subtest("process-exit-gtt-busy")
-			test_process_exit(fd, PE_GTT_MAP | PE_BUSY);
-
 		igt_subtest("create-destroy-sync")
 			test_create_destroy(fd, 5);
 
@@ -2221,13 +2386,11 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 
 		for (unsigned flags = 0; flags < ALL_MAP_FIXED_INVALIDATE + 1; flags++) {
 			igt_describe("Try to anger lockdep with MMU notifier still active after MAP_FIXED remap");
-			igt_subtest_with_dynamic_f("map-fixed-invalidate%s%s%s",
+			igt_subtest_with_dynamic_f("map-fixed-invalidate%s%s",
 					flags & MAP_FIXED_INVALIDATE_OVERLAP ?
 							"-overlap" : "",
 					flags & MAP_FIXED_INVALIDATE_BUSY ?
-							"-busy" : "",
-					flags & MAP_FIXED_INVALIDATE_GET_PAGES ?
-							"-gup" : "") {
+							"-busy" : "") {
 				igt_require_f(gem_available_fences(fd),
 					      "HW & kernel support for tiling\n");
 
@@ -2238,18 +2401,11 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 			}
 		}
 
-		igt_describe("Invalidate pages of idle userptr with mmap-offset on top");
-		igt_subtest_with_dynamic("mmap-offset-invalidate-idle")
+		igt_describe("Verify mmap_offset to userptr is banned");
+		igt_subtest_with_dynamic("mmap-offset-banned")
 			for_each_mmap_offset_type(fd, t)
 				igt_dynamic_f("%s", t->name)
-					test_mmap_offset_invalidate(fd, t, 0);
-
-		igt_describe("Invalidate pages of active userptr with mmap-offset on top");
-		igt_subtest_with_dynamic("mmap-offset-invalidate-active")
-			for_each_mmap_offset_type(fd, t)
-				igt_dynamic_f("%s", t->name)
-					test_mmap_offset_invalidate(fd, t,
-								   MMOI_ACTIVE);
+					test_mmap_offset_banned(fd, t);
 
 		igt_subtest("coherency-sync")
 			test_coherency(fd, count);
@@ -2311,6 +2467,23 @@ igt_main_args("c:", NULL, help_str, opt_handler, NULL)
 		igt_stop_signal_helper();
 	}
 
+	igt_subtest_group {
+		igt_fixture {
+			gem_userptr_test_synchronized();
+			if (!has_userptr(fd))
+				gem_userptr_test_unsynchronized();
+			igt_require(has_userptr(fd));
+		}
+
+		igt_subtest("nohangcheck")
+			test_nohangcheck_hostile(fd);
+
+		igt_subtest("vma-merge")
+			test_vma_merge(fd);
+
+		igt_subtest("huge-split")
+			test_huge_split(fd);
+	}
 
 	igt_subtest("access-control")
 		test_access_control(fd);
