@@ -519,7 +519,7 @@ run_tests_for_pipe(data_t *data, enum pipe p)
 
 	igt_fixture {
 
-		igt_require(p < data->display.n_pipes);
+		igt_require_pipe(&data->display, p);
 
 		pipe = &data->display.pipes[p];
 		igt_require(pipe->n_planes >= 0);
@@ -610,9 +610,11 @@ run_tests_for_pipe(data_t *data, enum pipe p)
 				expected_colors[1].g =
 				expected_colors[2].b =
 				0.25 + delta * (i - 2);
-			success |= test_pipe_ctm(data, primary,
-						 red_green_blue,
-						 expected_colors, ctm);
+			if(test_pipe_ctm(data, primary, red_green_blue,
+					 expected_colors, ctm)) {
+				success = true;
+				break;
+			}
 		}
 		igt_assert(success);
 	}
@@ -632,9 +634,11 @@ run_tests_for_pipe(data_t *data, enum pipe p)
 				expected_colors[1].g =
 				expected_colors[2].b =
 				0.5 + delta * (i - 2);
-			success |= test_pipe_ctm(data, primary,
-						 red_green_blue,
-						 expected_colors, ctm);
+			if(test_pipe_ctm(data, primary, red_green_blue,
+					 expected_colors, ctm)) {
+				success = true;
+				break;
+			}
 		}
 		igt_assert(success);
 	}
@@ -654,9 +658,11 @@ run_tests_for_pipe(data_t *data, enum pipe p)
 				expected_colors[1].g =
 				expected_colors[2].b =
 				0.75 + delta * (i - 3);
-			success |= test_pipe_ctm(data, primary,
-						 red_green_blue,
-						 expected_colors, ctm);
+			if(test_pipe_ctm(data, primary, red_green_blue,
+					 expected_colors, ctm)) {
+				success = true;
+				break;
+			}
 		}
 		igt_assert(success);
 	}
@@ -724,6 +730,13 @@ igt_main
 		data.drm_fd = drm_open_driver_master(DRIVER_ANY);
 		if (is_i915_device(data.drm_fd))
 			data.devid = intel_get_drm_devid(data.drm_fd);
+
+		igt_display_require(&data.display, data.drm_fd);
+		igt_require(data.display.is_atomic);
+
+		igt_chamelium_allow_fsm_handling = false;
+
+		/* we need to initalize chamelium after igt_display_require */
 		data.chamelium = chamelium_init(data.drm_fd);
 		igt_require(data.chamelium);
 
@@ -734,8 +747,6 @@ igt_main
 			igt_skip("No ports connected\n");
 
 		kmstest_set_vt_graphics_mode();
-		igt_display_require(&data.display, data.drm_fd);
-		igt_require(data.display.is_atomic);
 	}
 
 	for_each_pipe_static(pipe)
