@@ -25,6 +25,7 @@
 #include <sched.h>
 #include <sys/poll.h>
 
+#include "i915/gem.h"
 #include "igt.h"
 #include "igt_rand.h"
 #include "igt_stats.h"
@@ -57,7 +58,7 @@ static void stress(igt_display_t *display,
 	uint64_t *results;
 	bool torture;
 	int n;
-	unsigned crtc_id[IGT_MAX_PIPES], num_crtcs;
+	unsigned crtc_id[IGT_MAX_PIPES] = {0}, num_crtcs;
 
 	torture = false;
 	if (num_children < 0) {
@@ -83,8 +84,10 @@ static void stress(igt_display_t *display,
 		}
 	} else {
 		num_crtcs = 1;
-		arg.crtc_id = crtc_id[0] = display->pipes[pipe].crtc_id;
-		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
+		if(display->pipes[pipe].enabled) {
+			arg.crtc_id = crtc_id[0] = display->pipes[pipe].crtc_id;
+			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
+		}
 	}
 
 	arg.flags = mode;
@@ -102,7 +105,8 @@ static void stress(igt_display_t *display,
 		hars_petruska_f54_1_random_perturb(child);
 		igt_until_timeout(timeout) {
 			arg.crtc_id = crtc_id[hars_petruska_f54_1_random_unsafe() % num_crtcs];
-			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
+			if (arg.crtc_id)
+				do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg);
 			count++;
 		}
 
@@ -325,6 +329,7 @@ static void transition_nonblocking(igt_display_t *display, enum pipe pipe_id,
 
 		igt_assert(ret == -EINVAL);
 
+		igt_plane_set_fb(primary, prim_fb);
 		igt_plane_set_fb(sprite, prim_fb);
 	}
 	igt_display_commit_atomic(display, DRM_MODE_ATOMIC_NONBLOCK | DRM_MODE_PAGE_FLIP_EVENT, display);
@@ -465,28 +470,6 @@ static void flip(igt_display_t *display,
 		igt_remove_fb(display->drm_fd, &cursor_fb2);
 }
 
-static inline uint32_t pipe_select(enum pipe pipe)
-{
-	if (pipe > 1)
-		return pipe << DRM_VBLANK_HIGH_CRTC_SHIFT;
-	else if (pipe > 0)
-		return DRM_VBLANK_SECONDARY;
-	else
-		return 0;
-}
-
-static unsigned get_vblank(int fd, enum pipe pipe, unsigned flags)
-{
-	union drm_wait_vblank vbl;
-
-	memset(&vbl, 0, sizeof(vbl));
-	vbl.request.type = DRM_VBLANK_RELATIVE | pipe_select(pipe) | flags;
-	if (drmIoctl(fd, DRM_IOCTL_WAIT_VBLANK, &vbl))
-		return 0;
-
-	return vbl.reply.sequence;
-}
-
 enum basic_flip_cursor {
 	FLIP_BEFORE_CURSOR,
 	FLIP_AFTER_CURSOR
@@ -521,10 +504,10 @@ static void basic_flip_cursor(igt_display_t *display,
 	igt_display_commit2(display, display->is_atomic ? COMMIT_ATOMIC : COMMIT_LEGACY);
 
 	/* Quick sanity check that we can update a cursor in a single vblank */
-	vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
-	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+	igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 	do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
 	for (i = 0; i < 25; i++) {
 		bool miss;
@@ -538,7 +521,7 @@ static void basic_flip_cursor(igt_display_t *display,
 					    .dependency = fb_info.gem_handle);
 
 		/* Start with a synchronous query to align with the vblank */
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+		vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 
 		switch (order) {
 		case FLIP_BEFORE_CURSOR:
@@ -552,7 +535,7 @@ static void basic_flip_cursor(igt_display_t *display,
 				break;
 			}
 
-			delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+			delta = kmstest_get_vblank(display->drm_fd, pipe, 0) - vblank_start;
 			miss = delta != 0;
 
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
@@ -561,7 +544,7 @@ static void basic_flip_cursor(igt_display_t *display,
 		case FLIP_AFTER_CURSOR:
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
 
-			delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+			delta = kmstest_get_vblank(display->drm_fd, pipe, 0) - vblank_start;
 			miss = delta != 0;
 
 			switch (mode) {
@@ -575,7 +558,7 @@ static void basic_flip_cursor(igt_display_t *display,
 			}
 		}
 
-		delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+		delta = kmstest_get_vblank(display->drm_fd, pipe, 0) - vblank_start;
 
 		if (spin) {
 			struct pollfd pfd = { display->drm_fd, POLLIN };
@@ -599,7 +582,7 @@ static void basic_flip_cursor(igt_display_t *display,
 		if (miss1)
 			continue;
 
-		delta = get_vblank(display->drm_fd, pipe, 0) - vblank_start;
+		delta = kmstest_get_vblank(display->drm_fd, pipe, 0) - vblank_start;
 
 		if (!mode_requires_extra_vblank(mode))
 			miss2 += delta != 1;
@@ -627,13 +610,13 @@ get_cursor_updates_per_vblank(igt_display_t *display, enum pipe pipe,
 	int target;
 
 	for (target = 65536; target; target /= 2) {
-		unsigned vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+		unsigned vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
 		for (int n = 0; n < target; n++)
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, arg);
-		if (get_vblank(display->drm_fd, pipe, 0) == vblank_start)
+		if (kmstest_get_vblank(display->drm_fd, pipe, 0) == vblank_start)
 			break;
 	}
 
@@ -678,11 +661,11 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 	else
 		target = 1;
 
-	vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
-	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+	igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 	for (int n = 0; n < target; n++)
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[0]);
-	igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+	igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
 	/*
 	 * There are variations caused by using cpu frequency changing. To
@@ -720,7 +703,7 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
 
 		/* Start with a synchronous query to align with the vblank */
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+		vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 		switch (mode) {
 		default:
 			flip_nonblocking(display, pipe, mode >= flip_test_atomic, &fb_info, NULL);
@@ -732,23 +715,23 @@ static void flip_vs_cursor(igt_display_t *display, enum flip_test mode, int nloo
 		}
 
 		/* The nonblocking flip should not have delayed us */
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 		for (int n = 0; n < target; n++)
 			do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[nloops & 1]);
 
 		/* Nor should it have delayed the following cursor update */
 		if (!cursor_slowpath(mode))
-			igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+			igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 		else
-			igt_assert_lte(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+			igt_assert_lte(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
 
 		igt_set_timeout(1, "Stuck page flip");
 		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
 
 		if (!mode_requires_extra_vblank(mode))
-			igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+			igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
 		else
-			igt_assert_lte(get_vblank(display->drm_fd, pipe, 0), vblank_start + 2);
+			igt_assert_lte(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start + 2);
 
 		igt_reset_timeout();
 	} while (nloops--);
@@ -852,6 +835,19 @@ static void nonblocking_modeset_vs_cursor(igt_display_t *display, int loops)
 	igt_remove_fb(display->drm_fd, &cursor_fb);
 }
 
+static void wait_for_modeset(igt_display_t *display, unsigned flags, int timeout,
+			     const char *info)
+{
+	int ret;
+
+	igt_set_timeout(timeout, info);
+	do {
+		ret = igt_display_try_commit_atomic(display, flags, NULL);
+	} while (ret == -EBUSY);
+	igt_assert(!ret);
+	igt_reset_timeout();
+}
+
 static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool modeset, bool atomic)
 {
 	struct drm_mode_cursor arg1[2], arg2[2];
@@ -920,14 +916,14 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 		 * Try a page flip on crtc 1, if we succeed pump page flips and
 		 * modesets interleaved, else do a single atomic commit with both.
 		 */
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+		vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 		igt_plane_set_fb(plane, &fb_info);
 		ret = igt_display_try_commit_atomic(display, flags, (void*)(ptrdiff_t)vblank_start);
 		igt_assert(!ret || ret == -EBUSY);
 
 		if (ret == -EBUSY) {
 			/* Force completion on both pipes, and generate event. */
-			igt_display_commit_atomic(display, flags, NULL);
+			wait_for_modeset(display, flags, 5, "Stuck with -EBUSY");
 
 			while (nloops--) {
 				shared[1] = nloops & 1;
@@ -945,21 +941,16 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 				igt_output_set_pipe(output2, enabled ? PIPE_NONE : pipe2);
 				enabled = !enabled;
 
-				igt_set_timeout(5, "Scheduling modeset\n");
-				do {
-					ret = igt_display_try_commit_atomic(display, flags, NULL);
-				} while (ret == -EBUSY);
-				igt_assert(!ret);
-				igt_reset_timeout();
+				wait_for_modeset(display, flags, 5, "Scheduling modeset");
 			}
 
 			goto done;
 		}
 	} else {
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+		vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 		flip_nonblocking(display, pipe, atomic, &fb_info, (void*)(ptrdiff_t)vblank_start);
 
-		vblank_start = get_vblank(display->drm_fd, pipe2, DRM_VBLANK_NEXTONMISS);
+		vblank_start = kmstest_get_vblank(display->drm_fd, pipe2, DRM_VBLANK_NEXTONMISS);
 		flip_nonblocking(display, pipe2, atomic, &fb2_info, (void*)(ptrdiff_t)vblank_start);
 	}
 
@@ -984,7 +975,7 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 		}
 
 		if (vbl.crtc_id == display->pipes[pipe].crtc_id) {
-			vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+			vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 			flip_nonblocking(display, pipe, atomic, &fb_info, (void*)(ptrdiff_t)vblank_start);
 		} else {
 			igt_assert(vbl.crtc_id == display->pipes[pipe2].crtc_id);
@@ -992,7 +983,7 @@ static void two_screens_flip_vs_cursor(igt_display_t *display, int nloops, bool
 			nloops--;
 
 			if (!modeset) {
-				vblank_start = get_vblank(display->drm_fd, pipe2, DRM_VBLANK_NEXTONMISS);
+				vblank_start = kmstest_get_vblank(display->drm_fd, pipe2, DRM_VBLANK_NEXTONMISS);
 				flip_nonblocking(display, pipe2, atomic, &fb2_info, (void*)(ptrdiff_t)vblank_start);
 			} else {
 				igt_output_set_pipe(output2, enabled ? PIPE_NONE : pipe2);
@@ -1252,18 +1243,18 @@ static void flip_vs_cursor_crc(igt_display_t *display, bool atomic)
 
 	/* Disable cursor, and immediately queue a flip. Check if resulting crc is correct. */
 	for (int i = 1; i >= 0; i--) {
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+		vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 
 		flip_nonblocking(display, pipe, atomic, &fb_info, NULL);
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[i]);
 
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
 		igt_set_timeout(1, "Stuck page flip");
 		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
 		igt_reset_timeout();
 
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
+		igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start + 1);
 
 		igt_pipe_crc_collect_crc(pipe_crc, &crcs[2]);
 
@@ -1333,12 +1324,12 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 		spin = igt_spin_new(display->drm_fd,
 				    .dependency = fb_info[1].gem_handle);
 
-		vblank_start = get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
+		vblank_start = kmstest_get_vblank(display->drm_fd, pipe, DRM_VBLANK_NEXTONMISS);
 
 		flip_nonblocking(display, pipe, atomic, &fb_info[1], NULL);
 		do_ioctl(display->drm_fd, DRM_IOCTL_MODE_CURSOR, &arg[i]);
 
-		igt_assert_eq(get_vblank(display->drm_fd, pipe, 0), vblank_start);
+		igt_assert_eq(kmstest_get_vblank(display->drm_fd, pipe, 0), vblank_start);
 
 		igt_pipe_crc_get_current(display->drm_fd, pipe_crc, &test_crc);
 
@@ -1348,7 +1339,7 @@ static void flip_vs_cursor_busy_crc(igt_display_t *display, bool atomic)
 		igt_ignore_warn(read(display->drm_fd, &vbl, sizeof(vbl)));
 		igt_reset_timeout();
 
-		igt_assert_lte(vblank_start + 1, get_vblank(display->drm_fd, pipe, 0));
+		igt_assert_lte(vblank_start + 1, kmstest_get_vblank(display->drm_fd, pipe, 0));
 
 		igt_plane_set_fb(plane_primary, &fb_info[0]);
 		igt_display_commit2(display, COMMIT_UNIVERSAL);
@@ -1374,13 +1365,16 @@ igt_main
 		igt_display_require(&display, display.drm_fd);
 	}
 
+	/*Test description for pipe-* and all-pipe-* tests*/
+	igt_describe("Test checks how many cursor updates we can fit between vblanks "
+		     "on single pipe with different modes, priority and number of processes");
 	igt_subtest_group {
 		enum pipe n;
 		for_each_pipe_static(n) {
 			errno = 0;
 
 			igt_fixture {
-				igt_skip_on(n >= display.n_pipes);
+				igt_require_pipe(&display, n);
 			}
 
 			igt_subtest_f("pipe-%s-single-bo", kmstest_pipe_name(n))
@@ -1400,68 +1394,90 @@ igt_main
 		}
 	}
 
-	igt_subtest("all-pipes-single-bo")
-		stress(&display, -1, 1, DRM_MODE_CURSOR_BO, 20);
-	igt_subtest("all-pipes-single-move")
-		stress(&display, -1, 1, DRM_MODE_CURSOR_MOVE, 20);
+	/*Test description for pipe-* and all-pipe-* tests*/
+	igt_describe("Test checks how many cursor updates we can fit between vblanks "
+		    "on all pipes with different modes, priority and number of processes");
+	igt_subtest_group {
 
-	igt_subtest("all-pipes-forked-bo")
-		stress(&display, -1, ncpus, DRM_MODE_CURSOR_BO, 20);
-	igt_subtest("all-pipes-forked-move")
-		stress(&display, -1, ncpus, DRM_MODE_CURSOR_MOVE, 20);
+		igt_subtest("all-pipes-single-bo")
+			stress(&display, -1, 1, DRM_MODE_CURSOR_BO, 20);
+		igt_subtest("all-pipes-single-move")
+			stress(&display, -1, 1, DRM_MODE_CURSOR_MOVE, 20);
 
-	igt_subtest("all-pipes-torture-bo")
-		stress(&display, -1, -ncpus, DRM_MODE_CURSOR_BO, 20);
-	igt_subtest("all-pipes-torture-move")
-		stress(&display, -1, -ncpus, DRM_MODE_CURSOR_MOVE, 20);
+		igt_subtest("all-pipes-forked-bo")
+			stress(&display, -1, ncpus, DRM_MODE_CURSOR_BO, 20);
+		igt_subtest("all-pipes-forked-move")
+			stress(&display, -1, ncpus, DRM_MODE_CURSOR_MOVE, 20);
 
-	igt_subtest("nonblocking-modeset-vs-cursor-atomic")
-		nonblocking_modeset_vs_cursor(&display, 1);
+		igt_subtest("all-pipes-torture-bo")
+			stress(&display, -1, -ncpus, DRM_MODE_CURSOR_BO, 20);
+		igt_subtest("all-pipes-torture-move")
+			stress(&display, -1, -ncpus, DRM_MODE_CURSOR_MOVE, 20);
 
-	igt_subtest("long-nonblocking-modeset-vs-cursor-atomic")
-		nonblocking_modeset_vs_cursor(&display, 16);
+		igt_subtest("nonblocking-modeset-vs-cursor-atomic")
+			nonblocking_modeset_vs_cursor(&display, 1);
 
-	igt_subtest("2x-flip-vs-cursor-legacy")
-		two_screens_flip_vs_cursor(&display, 8, false, false);
+		igt_subtest("long-nonblocking-modeset-vs-cursor-atomic")
+			nonblocking_modeset_vs_cursor(&display, 16);
+	}
 
-	igt_subtest("2x-flip-vs-cursor-atomic")
-		two_screens_flip_vs_cursor(&display, 8, false, true);
+	/*Test Description for 2x-* tests*/
+	igt_describe("This test executes flips on both CRTCs "
+		     "while running cursor updates in parallel");
+	igt_subtest_group {
 
-	igt_subtest("2x-cursor-vs-flip-legacy")
-		two_screens_cursor_vs_flip(&display, 8, false);
+		igt_subtest("2x-flip-vs-cursor-legacy")
+			two_screens_flip_vs_cursor(&display, 8, false, false);
 
-	igt_subtest("2x-long-flip-vs-cursor-legacy")
-		two_screens_flip_vs_cursor(&display, 150, false, false);
+		igt_subtest("2x-flip-vs-cursor-atomic")
+			two_screens_flip_vs_cursor(&display, 8, false, true);
 
-	igt_subtest("2x-long-flip-vs-cursor-atomic")
-		two_screens_flip_vs_cursor(&display, 150, false, true);
+		igt_subtest("2x-cursor-vs-flip-legacy")
+			two_screens_cursor_vs_flip(&display, 8, false);
 
-	igt_subtest("2x-long-cursor-vs-flip-legacy")
-		two_screens_cursor_vs_flip(&display, 50, false);
+		igt_subtest("2x-long-flip-vs-cursor-legacy")
+			two_screens_flip_vs_cursor(&display, 150, false, false);
 
-	igt_subtest("2x-nonblocking-modeset-vs-cursor-atomic")
-		two_screens_flip_vs_cursor(&display, 4, true, true);
+		igt_subtest("2x-long-flip-vs-cursor-atomic")
+			two_screens_flip_vs_cursor(&display, 150, false, true);
 
-	igt_subtest("2x-cursor-vs-flip-atomic")
-		two_screens_cursor_vs_flip(&display, 8, true);
+		igt_subtest("2x-long-cursor-vs-flip-legacy")
+			two_screens_cursor_vs_flip(&display, 50, false);
 
-	igt_subtest("2x-long-nonblocking-modeset-vs-cursor-atomic")
-		two_screens_flip_vs_cursor(&display, 15, true, true);
+		igt_subtest("2x-nonblocking-modeset-vs-cursor-atomic")
+			two_screens_flip_vs_cursor(&display, 4, true, true);
 
-	igt_subtest("2x-long-cursor-vs-flip-atomic")
-		two_screens_cursor_vs_flip(&display, 50, true);
+		igt_subtest("2x-cursor-vs-flip-atomic")
+			two_screens_cursor_vs_flip(&display, 8, true);
 
-	igt_subtest("flip-vs-cursor-crc-legacy")
-		flip_vs_cursor_crc(&display, false);
+		igt_subtest("2x-long-nonblocking-modeset-vs-cursor-atomic")
+			two_screens_flip_vs_cursor(&display, 15, true, true);
 
-	igt_subtest("flip-vs-cursor-crc-atomic")
-		flip_vs_cursor_crc(&display, true);
+		igt_subtest("2x-long-cursor-vs-flip-atomic")
+			two_screens_cursor_vs_flip(&display, 50, true);
+	}
 
-	igt_subtest("flip-vs-cursor-busy-crc-legacy")
-		flip_vs_cursor_busy_crc(&display, false);
+	/*Test description for cursor-crc tests*/
+	igt_describe("Test will first does a page flip and then cursor update");
+	igt_subtest_group {
 
-	igt_subtest("flip-vs-cursor-busy-crc-atomic")
-		flip_vs_cursor_busy_crc(&display, true);
+		igt_subtest("flip-vs-cursor-crc-legacy")
+			flip_vs_cursor_crc(&display, false);
+
+		igt_subtest("flip-vs-cursor-crc-atomic")
+			flip_vs_cursor_crc(&display, true);
+	}
+
+	/*Test description for busy-crc tests*/
+	igt_describe("this test perform a busy bo update followed by a cursor update");
+	igt_subtest_group {
+
+		igt_subtest("flip-vs-cursor-busy-crc-legacy")
+			flip_vs_cursor_busy_crc(&display, false);
+
+		igt_subtest("flip-vs-cursor-busy-crc-atomic")
+			flip_vs_cursor_busy_crc(&display, true);
+	}
 
 	for (i = 0; i <= flip_test_last; i++) {
 		const char *modes[flip_test_last+1] = {
@@ -1483,6 +1499,12 @@ igt_main
 		default: break;
 		}
 
+	igt_describe("Adds variety of tests: "
+		     "varying-size: change the size of cursor b/w 64*64 to maxw x maxh. "
+		     "atomic-transition: alternates between a full screen sprite plane. "
+						"and full screen primary plane. "
+		     "toggle: which toggles cursor visibility and make sure cursor moves between updates.");
+	igt_subtest_group {
 		igt_subtest_f("%sflip-before-cursor-%s", prefix, modes[i])
 			basic_flip_cursor(&display, i, FLIP_BEFORE_CURSOR, 0);
 
@@ -1493,28 +1515,35 @@ igt_main
 						  BASIC_BUSY);
 			}
 		}
-
-		igt_subtest_f("%sflip-after-cursor-%s", prefix, modes[i])
-			basic_flip_cursor(&display, i, FLIP_AFTER_CURSOR, 0);
-
-		igt_subtest_f("flip-vs-cursor-%s", modes[i])
-			flip_vs_cursor(&display, i, 150);
-		igt_subtest_f("cursor-vs-flip-%s", modes[i])
-			cursor_vs_flip(&display, i, 50);
-
-		igt_subtest_f("cursorA-vs-flipA-%s", modes[i])
-			flip(&display, 0, 0, 10, i);
-
-		igt_subtest_f("cursorA-vs-flipB-%s", modes[i])
-			flip(&display, 0, 1, 10, i);
-
-		igt_subtest_f("cursorB-vs-flipA-%s", modes[i])
-			flip(&display, 1, 0, 10, i);
-
-		igt_subtest_f("cursorB-vs-flipB-%s", modes[i])
-			flip(&display, 1, 1, 10, i);
 	}
 
+		igt_describe("The essence of the basic test is that neither the cursor nor the "
+			     "nonblocking flip stall the application of the next");
+		igt_subtest_group {
+
+			igt_subtest_f("%sflip-after-cursor-%s", prefix, modes[i])
+				basic_flip_cursor(&display, i, FLIP_AFTER_CURSOR, 0);
+
+			igt_subtest_f("flip-vs-cursor-%s", modes[i])
+				flip_vs_cursor(&display, i, 150);
+
+			igt_subtest_f("cursor-vs-flip-%s", modes[i])
+				cursor_vs_flip(&display, i, 50);
+
+			igt_subtest_f("cursorA-vs-flipA-%s", modes[i])
+				flip(&display, 0, 0, 10, i);
+
+			igt_subtest_f("cursorA-vs-flipB-%s", modes[i])
+				flip(&display, 0, 1, 10, i);
+
+			igt_subtest_f("cursorB-vs-flipA-%s", modes[i])
+				flip(&display, 1, 0, 10, i);
+
+			igt_subtest_f("cursorB-vs-flipB-%s", modes[i])
+				flip(&display, 1, 1, 10, i);
+			}
+		}
+
 	igt_fixture {
 		igt_display_fini(&display);
 	}
