@@ -42,6 +42,7 @@
 #include <time.h>
 #include <pthread.h>
 
+#include "i915/gem.h"
 #include "igt_stats.h"
 
 #define TEST_DPMS		(1 << 0)
@@ -86,8 +87,7 @@
 
 drmModeRes *resources;
 int drm_fd;
-static drm_intel_bufmgr *bufmgr;
-struct intel_batchbuffer *batch;
+static struct buf_ops *bops;
 uint32_t devid;
 int test_time = 3;
 static bool monotonic_timestamp;
@@ -211,19 +211,19 @@ static void emit_fence_stress(struct test_output *o)
 	struct drm_i915_gem_execbuffer2 execbuf;
 	struct drm_i915_gem_exec_object2 *exec;
 	uint32_t buf[2] = { MI_BATCH_BUFFER_END, 0 };
-	drm_intel_bo **bo;
+	struct intel_buf **bo;
 	int i;
 
-	igt_require(bufmgr);
+	igt_require(bops);
 
+	igt_assert(num_fences);
 	bo = calloc(sizeof(*bo), num_fences);
 	exec = calloc(sizeof(*exec), num_fences+1);
 	for (i = 0; i < num_fences - 1; i++) {
 		uint32_t tiling = I915_TILING_X;
-		unsigned long pitch = 0;
-		bo[i] = drm_intel_bo_alloc_tiled(bufmgr,
-						 "X tiled bo", 1024, 1024, 4,
-						 &tiling, &pitch, 0);
+		bo[i] = intel_buf_create(bops, 1024, 1024, 32, 0, tiling,
+					 I915_COMPRESSION_NONE);
+
 		exec[i].handle = bo[i]->handle;
 		exec[i].flags = EXEC_OBJECT_NEEDS_FENCE;
 	}
@@ -243,7 +243,7 @@ static void emit_fence_stress(struct test_output *o)
 
 	gem_close(drm_fd, exec[i].handle);
 	for (i = 0; i < num_fences - 1; i++)
-		drm_intel_bo_unreference(bo[i]);
+		intel_buf_destroy(bo[i]);
 	free(bo);
 	free(exec);
 }
@@ -481,7 +481,7 @@ static void vblank_handler(int fd, unsigned int frame, unsigned int sec,
 	fixup_premature_vblank_ts(o, &o->vblank_state);
 }
 
-static void check_state(const struct test_output *o, const struct event_state *es)
+static bool check_state(const struct test_output *o, const struct event_state *es)
 {
 	struct timeval diff;
 
@@ -495,7 +495,7 @@ static void check_state(const struct test_output *o, const struct event_state *e
 	}
 
 	if (es->count == 0)
-		return;
+		return true;
 
 	timersub(&es->current_ts, &es->last_received_ts, &diff);
 	igt_assert_f(timercmp(&es->last_received_ts, &es->current_ts, <),
@@ -506,10 +506,12 @@ static void check_state(const struct test_output *o, const struct event_state *e
 	/* check only valid if no modeset happens in between, that increments by
 	 * (1 << 23) on each step. This bounding matches the one in
 	 * DRM_IOCTL_WAIT_VBLANK. */
-	if (!(o->flags & (TEST_DPMS | TEST_MODESET | TEST_NO_VBLANK)))
-		igt_assert_f(es->current_seq - (es->last_seq + o->seq_step) <= 1UL << 23,
-			     "unexpected %s seq %u, should be >= %u\n",
-			     es->name, es->current_seq, es->last_seq + o->seq_step);
+	if (!(o->flags & (TEST_DPMS | TEST_MODESET | TEST_NO_VBLANK)) &&
+	    es->current_seq - (es->last_seq + o->seq_step) > 1UL << 23) {
+		igt_debug("unexpected %s seq %u, should be >= %u\n",
+			  es->name, es->current_seq, es->last_seq + o->seq_step);
+		return false;
+	}
 
 	if (o->flags & TEST_CHECK_TS) {
 		double elapsed, expected;
@@ -524,17 +526,25 @@ static void check_state(const struct test_output *o, const struct event_state *e
 			  elapsed, expected, expected * 0.005,
 			  fabs((elapsed - expected) / expected) * 100);
 
-		igt_assert_f(fabs((elapsed - expected) / expected) <= 0.005,
-			     "inconsistent %s ts/seq: last %.06f/%u, current %.06f/%u: elapsed=%.1fus expected=%.1fus\n",
-			     es->name, timeval_float(&es->last_ts), es->last_seq,
-			     timeval_float(&es->current_ts), es->current_seq,
-			     elapsed, expected);
+		if (fabs((elapsed - expected) / expected) > 0.005) {
+			igt_debug("inconsistent %s ts/seq: last %.06f/%u, current %.06f/%u: elapsed=%.1fus expected=%.1fus\n",
+				  es->name, timeval_float(&es->last_ts), es->last_seq,
+				  timeval_float(&es->current_ts), es->current_seq,
+				  elapsed, expected);
 
-		igt_assert_f(es->current_seq == es->last_seq + o->seq_step,
-			     "unexpected %s seq %u, expected %u\n",
-			     es->name, es->current_seq,
-			     es->last_seq + o->seq_step);
+			return false;
+		}
+
+		if (es->current_seq != es->last_seq + o->seq_step) {
+			igt_debug("unexpected %s seq %u, expected %u\n",
+				  es->name, es->current_seq,
+				  es->last_seq + o->seq_step);
+
+			return false;
+		}
 	}
+
+	return true;
 }
 
 static void check_state_correlation(struct test_output *o,
@@ -561,7 +571,7 @@ static void check_state_correlation(struct test_output *o,
 		     es1->name, es2->name, usec_diff / USEC_PER_SEC);
 }
 
-static void check_all_state(struct test_output *o,
+static bool check_all_state(struct test_output *o,
 			    unsigned int completed_events)
 {
 	bool flip, vblank;
@@ -569,14 +579,17 @@ static void check_all_state(struct test_output *o,
 	flip = completed_events & EVENT_FLIP;
 	vblank = completed_events & EVENT_VBLANK;
 
-	if (flip)
-		check_state(o, &o->flip_state);
-	if (vblank)
-		check_state(o, &o->vblank_state);
+	if (flip && !check_state(o, &o->flip_state))
+		return false;
+
+	if (vblank && !check_state(o, &o->vblank_state))
+		return false;
 
 	/* FIXME: Correlation check is broken. */
 	if (flip && vblank && 0)
 		check_state_correlation(o, &o->flip_state, &o->vblank_state);
+
+	return true;
 }
 
 static void recreate_fb(struct test_output *o)
@@ -644,8 +657,12 @@ static int set_mode(struct test_output *o, uint32_t fb, int x, int y)
 	return 0;
 }
 
-/* Return mask of completed events. */
-static unsigned int run_test_step(struct test_output *o)
+/*
+ * Return true if the test steps were run successfully, false in case of a
+ * failure that requires rerunning the test steps. On success events will
+ * contain the mask of completed events.
+ */
+static bool run_test_step(struct test_output *o, unsigned int *events)
 {
 	unsigned int new_fb_id;
 	/* for funny reasons page_flip returns -EBUSY on disabled crtcs ... */
@@ -695,7 +712,11 @@ static unsigned int run_test_step(struct test_output *o)
 		end = gettime_us();
 		igt_debug("Vblank took %luus\n", end - start);
 		igt_assert(end - start < 500);
-		igt_assert_eq(reply.sequence, exp_seq);
+		if (reply.sequence != exp_seq) {
+			igt_debug("unexpected vblank seq %u, should be %u\n",
+				  reply.sequence, exp_seq);
+			return false;
+		}
 		igt_assert(timercmp(&reply.ts, &o->flip_state.last_ts, ==));
 	}
 
@@ -825,7 +846,9 @@ static unsigned int run_test_step(struct test_output *o)
 
 	unhang_gpu(drm_fd, hang);
 
-	return completed_events;
+	*events = completed_events;
+
+	return true;
 }
 
 static void update_state(struct event_state *es)
@@ -959,7 +982,7 @@ static void paint_flip_mode(struct igt_fb *fb, bool odd_frame)
 	cairo_set_source_rgb(cr, 1, 1, 1);
 	cairo_fill(cr);
 
-	igt_put_cairo_ctx(drm_fd, fb, cr);
+	igt_put_cairo_ctx(cr);
 }
 
 static bool fb_is_bound(struct test_output *o, int fb)
@@ -981,7 +1004,7 @@ static bool fb_is_bound(struct test_output *o, int fb)
 	return true;
 }
 
-static void check_final_state(const struct test_output *o,
+static bool check_final_state(const struct test_output *o,
 			      const struct event_state *es,
 			      unsigned int elapsed)
 {
@@ -998,11 +1021,16 @@ static void check_final_state(const struct test_output *o,
 		igt_debug("expected %d, counted %d, encoder type %d\n",
 			  (int)(elapsed / actual_frame_time(o)), count,
 			  o->kencoder[0]->encoder_type);
-		igt_assert_f(elapsed >= min && elapsed <= max,
-			     "dropped frames, expected %d, counted %d, encoder type %d\n",
-			     (int)(elapsed / actual_frame_time(o)), count,
-			     o->kencoder[0]->encoder_type);
+		if (elapsed < min || elapsed > max) {
+			igt_debug("dropped frames, expected %d, counted %d, encoder type %d\n",
+				  (int)(elapsed / actual_frame_time(o)), count,
+				  o->kencoder[0]->encoder_type);
+
+			return false;
+		}
 	}
+
+	return true;
 }
 
 /*
@@ -1049,7 +1077,8 @@ static unsigned int wait_for_events(struct test_output *o)
 }
 
 /* Returned the elapsed time in us */
-static unsigned event_loop(struct test_output *o, unsigned duration_ms)
+static bool event_loop(struct test_output *o, unsigned duration_ms,
+		       unsigned *elapsed)
 {
 	unsigned long start, end;
 	int count = 0;
@@ -1059,10 +1088,15 @@ static unsigned event_loop(struct test_output *o, unsigned duration_ms)
 	while (1) {
 		unsigned int completed_events;
 
-		completed_events = run_test_step(o);
+		if (!run_test_step(o, &completed_events))
+			return false;
+
 		if (o->pending_events)
 			completed_events |= wait_for_events(o);
-		check_all_state(o, completed_events);
+
+		if (!check_all_state(o, completed_events))
+			return false;
+
 		update_all_state(o, completed_events);
 
 		if (count && (gettime_us() - start) / 1000 >= duration_ms)
@@ -1077,7 +1111,9 @@ static unsigned event_loop(struct test_output *o, unsigned duration_ms)
 	if (o->pending_events)
 		wait_for_events(o);
 
-	return end - start;
+	*elapsed = end - start;
+
+	return true;
 }
 
 static void free_test_output(struct test_output *o)
@@ -1090,7 +1126,7 @@ static void free_test_output(struct test_output *o)
 	}
 }
 
-static void calibrate_ts(struct test_output *o, int crtc_idx)
+static bool calibrate_ts(struct test_output *o, int crtc_idx)
 {
 #define CALIBRATE_TS_STEPS 16
 	drmVBlank wait;
@@ -1100,6 +1136,7 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 	double expected;
 	double mean;
 	double stddev;
+	bool failed = false;
 	int n;
 
 	memset(&wait, 0, sizeof(wait));
@@ -1151,7 +1188,18 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 			igt_assert_eq(errno, EINTR);
 		}
 		igt_assert(read(drm_fd, &ev, sizeof(ev)) == sizeof(ev));
-		igt_assert_eq(ev.sequence, last_seq + 1);
+
+		if (failed)
+			continue;
+
+		if (ev.sequence != last_seq + 1) {
+			igt_debug("Unexpected frame sequence %d vs. expected %d\n",
+				  ev.sequence, last_seq + 1);
+			failed = true;
+
+			/* Continue to flush all the events queued up */
+			continue;
+		}
 
 		now = ev.tv_sec;
 		now *= 1000000;
@@ -1163,6 +1211,9 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 		last_seq = ev.sequence;
 	}
 
+	if (failed)
+		return false;
+
 	expected = mode_frame_time(o);
 
 	mean = igt_stats_get_mean(&stats);
@@ -1182,54 +1233,46 @@ static void calibrate_ts(struct test_output *o, int crtc_idx)
 	}
 
 	o->vblank_interval = mean;
+
+	return true;
 }
 
-static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
-				 int crtc_count, int duration_ms)
+/*
+ * Some monitors with odd behavior signal a bad link after waking from a power
+ * saving state and the subsequent (successful) modeset. This will result in a
+ * link-retraining (DP) or async modeset (HDMI), which in turn makes the test
+ * miss vblank/flip events and fail.  Work around this by retrying the test
+ * once in case of such a link reset event, which the driver signals with a
+ * hotplug event.
+ */
+static bool needs_retry_after_link_reset(struct udev_monitor *mon)
 {
-	char test_name[128];
-	unsigned elapsed;
+	bool hotplug_detected;
+
+	igt_suspend_signal_helper();
+	hotplug_detected = igt_hotplug_detected(mon, 3);
+	igt_resume_signal_helper();
+
+	if (hotplug_detected)
+		igt_debug("Retrying after a hotplug event\n");
+
+	return hotplug_detected;
+}
+
+static void __run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
+				   int crtc_count, int duration_ms)
+{
+	struct udev_monitor *mon = igt_watch_uevents();
 	unsigned bo_size = 0;
+	bool vblank = true;
+	bool retried = false;
+	bool state_ok;
+	unsigned elapsed;
 	uint64_t tiling;
 	int i;
-	bool vblank = true;
-
-	switch (crtc_count) {
-	case RUN_TEST:
-		connector_find_preferred_mode(o->_connector[0], crtc_idxs[0], o);
-		if (!o->mode_valid)
-			return;
-		snprintf(test_name, sizeof(test_name),
-			 "%s on pipe %s, connector %s-%d",
-			 igt_subtest_name(),
-			 kmstest_pipe_name(o->_pipe[0]),
-			 kmstest_connector_type_str(o->kconnector[0]->connector_type),
-			 o->kconnector[0]->connector_type_id);
-		break;
-	case RUN_PAIR:
-		connector_find_compatible_mode(crtc_idxs[0], crtc_idxs[1], o);
-		if (!o->mode_valid)
-			return;
-		snprintf(test_name, sizeof(test_name),
-			 "%s on pipe %s:%s, connector %s-%d:%s-%d",
-			 igt_subtest_name(),
-			 kmstest_pipe_name(o->_pipe[0]),
-			 kmstest_pipe_name(o->_pipe[1]),
-			 kmstest_connector_type_str(o->kconnector[0]->connector_type),
-			 o->kconnector[0]->connector_type_id,
-			 kmstest_connector_type_str(o->kconnector[1]->connector_type),
-			 o->kconnector[1]->connector_type_id);
-		break;
-	default:
-		igt_assert(0);
-	}
-
-	igt_assert_eq(o->count, crtc_count);
 
 	last_connector = o->kconnector[0];
 
-	igt_info("Beginning %s\n", test_name);
-
 	if (o->flags & TEST_PAN)
 		o->fb_width *= 2;
 
@@ -1240,7 +1283,7 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	/* 256 MB is usually the maximum mappable aperture,
 	 * (make it 4x times that to ensure failure) */
 	if (o->flags & TEST_BO_TOOBIG) {
-		bo_size = 4*gem_mappable_aperture_size();
+		bo_size = 4*gem_mappable_aperture_size(drm_fd);
 		igt_require(bo_size < gem_global_aperture_size(drm_fd));
 	}
 
@@ -1265,15 +1308,22 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	for (i = 0; i < o->count; i++)
 		kmstest_dump_mode(&o->kmode[i]);
 
+retry:
+	memset(&o->vblank_state, 0, sizeof(o->vblank_state));
+	memset(&o->flip_state, 0, sizeof(o->flip_state));
+	o->flip_state.name = "flip";
+	o->vblank_state.name = "vblank";
+
 	kmstest_unset_all_crtcs(drm_fd, resources);
 
+	igt_flush_uevents(mon);
+
 	if (set_mode(o, o->fb_ids[0], 0, 0)) {
 		/* We may fail to apply the mode if there are hidden
 		 * constraints, such as bandwidth on the third pipe.
 		 */
 		igt_assert_f(crtc_count > 1 || crtc_idxs[0] < 2,
 			     "set_mode may only fail on the 3rd pipe or in multiple crtc tests\n");
-		igt_info("\n%s: SKIPPED\n\n", test_name);
 		goto out;
 	}
 	igt_assert(fb_is_bound(o, o->fb_ids[0]));
@@ -1287,8 +1337,13 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	}
 
 	/* quiescent the hw a bit so ensure we don't miss a single frame */
-	if (o->flags & TEST_CHECK_TS)
-		calibrate_ts(o, crtc_idxs[0]);
+	if (o->flags & TEST_CHECK_TS && !calibrate_ts(o, crtc_idxs[0])) {
+		igt_assert(!retried && needs_retry_after_link_reset(mon));
+
+		retried = true;
+
+		goto retry;
+	}
 
 	if (o->flags & TEST_BO_TOOBIG) {
 		int err = do_page_flip(o, o->fb_ids[1], true);
@@ -1314,14 +1369,20 @@ static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
 	/* We run the vblank and flip actions in parallel by default. */
 	o->seq_step = max(o->vblank_state.seq_step, o->flip_state.seq_step);
 
-	elapsed = event_loop(o, duration_ms);
+	state_ok = event_loop(o, duration_ms, &elapsed);
 
 	if (o->flags & TEST_FLIP && !(o->flags & TEST_NOEVENT))
-		check_final_state(o, &o->flip_state, elapsed);
+		state_ok &= check_final_state(o, &o->flip_state, elapsed);
 	if (o->flags & TEST_VBLANK)
-		check_final_state(o, &o->vblank_state, elapsed);
+		state_ok &= check_final_state(o, &o->vblank_state, elapsed);
 
-	igt_info("\n%s: PASSED\n\n", test_name);
+	if (!state_ok) {
+		igt_assert(!retried && needs_retry_after_link_reset(mon));
+
+		retried = true;
+
+		goto retry;
+	}
 
 out:
 	igt_remove_fb(drm_fd, &o->fb_info[2]);
@@ -1331,6 +1392,47 @@ out:
 	last_connector = NULL;
 
 	free_test_output(o);
+
+	igt_cleanup_uevents(mon);
+}
+
+static void run_test_on_crtc_set(struct test_output *o, int *crtc_idxs,
+				 int crtc_count, int duration_ms)
+{
+	char test_name[128];
+
+	switch (crtc_count) {
+	case RUN_TEST:
+		connector_find_preferred_mode(o->_connector[0], crtc_idxs[0], o);
+		if (!o->mode_valid)
+			return;
+		snprintf(test_name, sizeof(test_name),
+			 "%s-%s%d",
+			 kmstest_pipe_name(o->_pipe[0]),
+			 kmstest_connector_type_str(o->kconnector[0]->connector_type),
+			 o->kconnector[0]->connector_type_id);
+		break;
+	case RUN_PAIR:
+		connector_find_compatible_mode(crtc_idxs[0], crtc_idxs[1], o);
+		if (!o->mode_valid)
+			return;
+		snprintf(test_name, sizeof(test_name),
+			 "%s%s-%s%d-%s%d",
+			 kmstest_pipe_name(o->_pipe[0]),
+			 kmstest_pipe_name(o->_pipe[1]),
+			 kmstest_connector_type_str(o->kconnector[0]->connector_type),
+			 o->kconnector[0]->connector_type_id,
+			 kmstest_connector_type_str(o->kconnector[1]->connector_type),
+			 o->kconnector[1]->connector_type_id);
+		break;
+	default:
+		igt_assert(0);
+	}
+
+	igt_assert_eq(o->count, crtc_count);
+
+	igt_dynamic_f("%s", test_name)
+		__run_test_on_crtc_set(o, crtc_idxs, crtc_count, duration_ms);
 }
 
 static int run_test(int duration, int flags)
@@ -1339,6 +1441,8 @@ static int run_test(int duration, int flags)
 	int i, n, modes = 0;
 
 	igt_require((flags & TEST_HANG) == 0 || !is_wedged(drm_fd));
+	igt_require(!(flags & TEST_FENCE_STRESS) ||
+		    gem_available_fences(drm_fd));
 
 	resources = drmModeGetResources(drm_fd);
 	igt_require(resources);
@@ -1350,8 +1454,6 @@ static int run_test(int duration, int flags)
 			o.count = 1;
 			o._connector[0] = resources->connectors[i];
 			o.flags = flags;
-			o.flip_state.name = "flip";
-			o.vblank_state.name = "vblank";
 			o.bpp = 32;
 			o.depth = 24;
 
@@ -1376,8 +1478,6 @@ static int run_test(int duration, int flags)
 			o.count = 1;
 			o._connector[0] = resources->connectors[i];
 			o.flags = flags;
-			o.flip_state.name = "flip";
-			o.vblank_state.name = "vblank";
 			o.bpp = 32;
 			o.depth = 24;
 
@@ -1396,6 +1496,8 @@ static int run_pair(int duration, int flags)
 	int i, j, m, n, modes = 0;
 
 	igt_require((flags & TEST_HANG) == 0 || !is_wedged(drm_fd));
+	igt_require(!(flags & TEST_FENCE_STRESS) ||
+		    gem_available_fences(drm_fd));
 
 	resources = drmModeGetResources(drm_fd);
 	igt_require(resources);
@@ -1410,8 +1512,6 @@ static int run_pair(int duration, int flags)
 					o._connector[0] = resources->connectors[i];
 					o._connector[1] = resources->connectors[j];
 					o.flags = flags;
-					o.flip_state.name = "flip";
-					o.vblank_state.name = "vblank";
 					o.bpp = 32;
 					o.depth = 24;
 
@@ -1443,8 +1543,6 @@ static int run_pair(int duration, int flags)
 					o._connector[0] = resources->connectors[i];
 					o._connector[1] = resources->connectors[j];
 					o.flags = flags;
-					o.flip_state.name = "flip";
-					o.vblank_state.name = "vblank";
 					o.bpp = 32;
 					o.depth = 24;
 
@@ -1477,10 +1575,8 @@ static void get_timestamp_format(void)
 
 static void kms_flip_exit_handler(int sig)
 {
-	igt_fixture {
-		if (last_connector)
-			kmstest_set_connector_dpms(drm_fd, last_connector, DRM_MODE_DPMS_ON);
-	}
+	if (last_connector)
+		kmstest_set_connector_dpms(drm_fd, last_connector, DRM_MODE_DPMS_ON);
 }
 
 static void test_nonblocking_read(int in)
@@ -1522,27 +1618,27 @@ igt_main
 					"absolute-wf_vblank" },
 		{ 30,  TEST_VBLANK | TEST_VBLANK_BLOCK | TEST_VBLANK_ABSOLUTE,
 					"blocking-absolute-wf_vblank" },
-		{ 10, TEST_FLIP | TEST_BASIC, "plain-flip" },
+		{ 2, TEST_FLIP | TEST_BASIC, "plain-flip" },
 		{ 1, TEST_FLIP | TEST_EBUSY, "busy-flip" },
 		{ 30, TEST_FLIP | TEST_FENCE_STRESS , "flip-vs-fences" },
 		{ 30, TEST_FLIP | TEST_CHECK_TS, "plain-flip-ts-check" },
 		{ 30, TEST_FLIP | TEST_CHECK_TS | TEST_FB_RECREATE,
 			"plain-flip-fb-recreate" },
 		{ 30, TEST_FLIP | TEST_RMFB | TEST_MODESET , "flip-vs-rmfb" },
-		{ 20, TEST_FLIP | TEST_DPMS | TEST_EINVAL | TEST_BASIC, "flip-vs-dpms" },
+		{ 2, TEST_FLIP | TEST_DPMS | TEST_EINVAL | TEST_BASIC, "flip-vs-dpms" },
 		{ 30,  TEST_FLIP | TEST_PAN, "flip-vs-panning" },
-		{ 20, TEST_FLIP | TEST_MODESET | TEST_EINVAL | TEST_BASIC, "flip-vs-modeset" },
+		{ 2, TEST_FLIP | TEST_MODESET | TEST_EINVAL | TEST_BASIC, "flip-vs-modeset" },
 		{ 30,  TEST_FLIP | TEST_VBLANK_EXPIRED_SEQ,
 					"flip-vs-expired-vblank" },
 
 		{ 30, TEST_FLIP | TEST_VBLANK | TEST_VBLANK_ABSOLUTE |
 		      TEST_CHECK_TS, "flip-vs-absolute-wf_vblank" },
-		{ 10, TEST_FLIP | TEST_VBLANK | TEST_CHECK_TS | TEST_BASIC,
+		{ 2, TEST_FLIP | TEST_VBLANK | TEST_CHECK_TS | TEST_BASIC,
 					"flip-vs-wf_vblank" },
 		{ 30, TEST_FLIP | TEST_VBLANK | TEST_VBLANK_BLOCK |
 			TEST_CHECK_TS, "flip-vs-blocking-wf-vblank" },
-		{ 30, TEST_FLIP | TEST_MODESET | TEST_HANG | TEST_NOEVENT, "flip-vs-modeset-vs-hang" },
-		{ 30, TEST_FLIP | TEST_PAN | TEST_HANG, "flip-vs-panning-vs-hang" },
+		{ 1, TEST_FLIP | TEST_MODESET | TEST_HANG | TEST_NOEVENT, "flip-vs-modeset-vs-hang" },
+		{ 1, TEST_FLIP | TEST_PAN | TEST_HANG, "flip-vs-panning-vs-hang" },
 
 		{ 1, TEST_DPMS_OFF | TEST_MODESET | TEST_FLIP,
 					"flip-vs-dpms-off-vs-modeset" },
@@ -1567,11 +1663,7 @@ igt_main
 		get_timestamp_format();
 
 		if (is_i915_device(drm_fd)) {
-			bufmgr = drm_intel_bufmgr_gem_init(drm_fd, 4096);
-			if (bufmgr) {
-				devid = intel_get_drm_devid(drm_fd);
-				batch = intel_batchbuffer_alloc(bufmgr, devid);
-			}
+			bops = buf_ops_create(drm_fd);
 		}
 	}
 
@@ -1579,7 +1671,7 @@ igt_main
 		test_nonblocking_read(drm_fd);
 
 	for (i = 0; i < sizeof(tests) / sizeof (tests[0]); i++) {
-		igt_subtest_f("%s%s",
+		igt_subtest_with_dynamic_f("%s%s",
 			      tests[i].flags & TEST_BASIC ? "basic-" : "",
 			      tests[i].name)
 			run_test(tests[i].duration, tests[i].flags);
@@ -1587,7 +1679,7 @@ igt_main
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
-		igt_subtest_f( "2x-%s", tests[i].name)
+		igt_subtest_with_dynamic_f("2x-%s", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
 
@@ -1600,19 +1692,26 @@ igt_main
 			continue;
 
 		/*
+		 * -EINVAL are negative API tests, they are rejected before
+		 *  any waits and so not subject to interruptiblity.
+		 *
 		 * -EBUSY needs to complete in a single vblank, skip them for
 		 * interruptible tests
+		 *
+		 * HANGs are slow enough and interruptible hang testing is
+		 * an oxymoron (can't force the wait-for-hang if being
+		 * interrupted all the time).
 		 */
-		if (tests[i].flags & TEST_EBUSY)
+		if (tests[i].flags & (TEST_EINVAL | TEST_EBUSY | TEST_HANG))
 			continue;
 
-		igt_subtest_f( "%s-interruptible", tests[i].name)
+		igt_subtest_with_dynamic_f("%s-interruptible", tests[i].name)
 			run_test(tests[i].duration, tests[i].flags);
 
 		if (tests[i].flags & TEST_NO_2X_OUTPUT)
 			continue;
 
-		igt_subtest_f( "2x-%s-interruptible", tests[i].name)
+		igt_subtest_with_dynamic_f("2x-%s-interruptible", tests[i].name)
 			run_pair(tests[i].duration, tests[i].flags);
 	}
 	igt_stop_signal_helper();
