@@ -35,7 +35,7 @@ IGT_TEST_DESCRIPTION("Test HDR metadata interfaces and bpc switch");
 
 /* DRM HDR definitions. Not in the UAPI header, unfortunately. */
 enum hdmi_metadata_type {
-	HDMI_STATIC_METADATA_TYPE1 = 1,
+	HDMI_STATIC_METADATA_TYPE1 = 0,
 };
 
 enum hdmi_eotf {
@@ -141,7 +141,7 @@ static void draw_hdr_pattern(igt_fb_t *fb)
 	igt_paint_color(cr, 0, 0, fb->width, fb->height, 1.0, 1.0, 1.0);
 	igt_paint_test_pattern(cr, fb->width, fb->height);
 
-	igt_put_cairo_ctx(fb->fd, fb, cr);
+	igt_put_cairo_ctx(cr);
 }
 
 /* Prepare test data. */
@@ -223,7 +223,14 @@ static void test_bpc_switch_on_output(data_t *data, igt_output_t *output,
 			data->h = afb.height;
 		}
 
-		igt_plane_set_fb(data->primary, NULL);
+		/*
+		 * amdgpu requires a primary plane when the CRTC is enabled.
+		 * However, some older Intel hardware (hsw) have scaling
+		 * requirements that are not met by the plane, so remove it
+		 * for non-AMD devices.
+		 */
+		if (!is_amdgpu_device(data->fd))
+			igt_plane_set_fb(data->primary, NULL);
 
 		/*
 		 * i915 driver doesn't expose max bpc as debugfs entry,
@@ -633,7 +640,7 @@ static void test_hdr(data_t *data, const char *test_name, uint32_t flags)
 
 igt_main
 {
-	data_t data = { 0 };
+	data_t data = {};
 
 	igt_fixture {
 		data.fd = drm_open_driver_master(DRIVER_AMDGPU | DRIVER_INTEL);
