@@ -136,9 +136,10 @@ static enum pipe crtc_id_to_pipe(igt_display_t *display, uint32_t crtc_id)
 {
 	enum pipe pipe;
 
-	for (pipe = 0; pipe < display->n_pipes; pipe++)
-		if (display->pipes[pipe].crtc_id == crtc_id)
+	for_each_pipe(display, pipe) {
+		if(display->pipes[pipe].crtc_id == crtc_id)
 			return pipe;
+	}
 	return -1;
 }
 
@@ -181,7 +182,7 @@ static int prepare_crtc(lease_t *lease, uint32_t connector_id, uint32_t crtc_id)
 	if (ret)
 		return ret;
 
-	igt_wait_for_vblank(lease->fd, pipe);
+	igt_wait_for_vblank(lease->fd, display->pipes[pipe].crtc_offset);
 
 	lease->output = output;
 	lease->mode = mode;
@@ -319,6 +320,8 @@ static void page_flip_implicit_plane(data_t *data)
 	drmModePlaneRes *plane_resources;
 	uint32_t wrong_plane_id = 0;
 	int i;
+	enum pipe pipe;
+	igt_display_t *display;
 
 	/* find a plane which isn't the primary one for us */
 	plane_resources = drmModeGetPlaneResources(data->master.fd);
@@ -349,9 +352,13 @@ static void page_flip_implicit_plane(data_t *data)
 	do_or_die(drmModePageFlip(data->master.fd, data->crtc_id,
 			      data->master.primary_fb.fb_id,
 			      0, NULL));
-	igt_wait_for_vblank_count(data->master.fd,
-				  crtc_id_to_pipe(&data->master.display, data->crtc_id),
-				  1);
+
+	display = &data->master.display;
+	pipe = crtc_id_to_pipe(display, data->crtc_id);
+
+	igt_wait_for_vblank(data->master.fd,
+			display->pipes[pipe].crtc_offset);
+
 	do_or_die(drmModePageFlip(mcl.fd, data->crtc_id,
 			      data->master.primary_fb.fb_id,
 			      0, NULL));
@@ -360,9 +367,11 @@ static void page_flip_implicit_plane(data_t *data)
 	object_ids[mcl.object_count++] = wrong_plane_id;
 	do_or_die(create_lease(data->master.fd, &mcl));
 
-	igt_wait_for_vblank_count(data->master.fd,
-				  crtc_id_to_pipe(&data->master.display, data->crtc_id),
-				  1);
+	pipe = crtc_id_to_pipe(display, data->crtc_id);
+
+	igt_wait_for_vblank(data->master.fd,
+			display->pipes[pipe].crtc_offset);
+
 	igt_assert_eq(drmModePageFlip(mcl.fd, data->crtc_id,
 				      data->master.primary_fb.fb_id,
 				      0, NULL),
@@ -683,7 +692,10 @@ static void lease_unleased_crtc(data_t *data)
 
 	/* Find another CRTC that we don't control */
 	bad_crtc_id = 0;
-	for (p = 0; bad_crtc_id == 0 && p < data->master.display.n_pipes; p++) {
+
+	for_each_pipe(&data->master.display, p) {
+		if (bad_crtc_id != 0)
+			break;
 		if (pipe_to_crtc_id(&data->master.display, p) != data->crtc_id)
 			bad_crtc_id = pipe_to_crtc_id(&data->master.display, p);
 	}
@@ -1238,9 +1250,9 @@ static void lease_uevent(data_t *data)
 	struct local_drm_mode_list_lessees mll;
 	struct udev_monitor *uevent_monitor;
 
-	uevent_monitor = igt_watch_hotplug();
+	uevent_monitor = igt_watch_uevents();
 
-	igt_flush_hotplugs(uevent_monitor);
+	igt_flush_uevents(uevent_monitor);
 
 	lease_fd = create_simple_lease(data->master.fd, data);
 
@@ -1260,7 +1272,7 @@ static void lease_uevent(data_t *data)
 	igt_assert_eq(list_lessees(data->master.fd, &mll), 0);
 	igt_assert_eq(mll.count_lessees, 0);
 
-	igt_cleanup_hotplug(uevent_monitor);
+	igt_cleanup_uevents(uevent_monitor);
 }
 
 igt_main
@@ -1300,21 +1312,31 @@ igt_main
 		}
 	}
 
+	igt_describe("Tests error handling while creating invalid corner-cases for "
+		     "create-lease ioctl");
 	igt_subtest("invalid-create-leases")
 		invalid_create_leases(&data);
 
+	igt_describe("Tests that  possible_crtcs logically match between master and "
+		     "lease, and that the values are correctly renumbered on the lease side.");
 	igt_subtest("possible-crtcs-filtering")
 		possible_crtcs_filtering(&data);
 
+	igt_describe("Tests the drop/set_master interactions.");
 	igt_subtest("master-vs-lease")
 		master_vs_lease(&data);
 
+	igt_describe("Tests that the 2nd master can only create leases while being active "
+		     "master, and that leases on the first master don't prevent lease creation "
+		     "for the 2nd master.");
 	igt_subtest("multimaster-lease")
 		multimaster_lease(&data);
 
+	igt_describe("Tests the implicitly added planes.");
 	igt_subtest("implicit-plane-lease")
 		implicit_plane_lease(&data);
 
+	igt_describe("Tests all the uevent cases");
 	igt_subtest("lease-uevent")
 		lease_uevent(&data);
 }
