@@ -25,6 +25,7 @@
  */
 
 #include "igt.h"
+#include "igt_vec.h"
 #include <errno.h>
 #include <stdbool.h>
 #include <stdio.h>
@@ -44,6 +45,11 @@ typedef struct {
 	float blue;
 } color_t;
 
+typedef struct {
+	int x, y;
+	color_t color;
+} rectangle_t;
+
 typedef struct {
 	int drm_fd;
 	igt_display_t display;
@@ -71,9 +77,52 @@ static void test_fini(data_t *data)
 	igt_pipe_crc_free(data->pipe_crc);
 }
 
+enum {
+	TEST_POSITION_PARTIALLY_COVERED = 1 << 0,
+	TEST_DPMS                       = 1 << 1,
+	TEST_PANNING_TOP_LEFT           = 1 << 2,
+	TEST_PANNING_BOTTOM_RIGHT       = 1 << 3,
+	TEST_SUSPEND_RESUME             = 1 << 4,
+};
+
+/*
+ * create a colored fb, possibly with a series of 64x64 colored rectangles (used
+ * for position tests)
+ */
+static void
+create_fb_for_mode(data_t *data, drmModeModeInfo *mode,
+		   color_t *fb_color,
+		   const rectangle_t *rects, int rect_cnt,
+		   struct igt_fb *fb /* out */)
+{
+	unsigned int fb_id;
+	cairo_t *cr;
+
+	fb_id = igt_create_fb(data->drm_fd,
+			      mode->hdisplay, mode->vdisplay,
+			      DRM_FORMAT_XRGB8888,
+			      LOCAL_DRM_FORMAT_MOD_NONE,
+			      fb);
+	igt_assert_fd(fb_id);
+
+	cr = igt_get_cairo_ctx(data->drm_fd, fb);
+	igt_paint_color(cr, 0, 0, mode->hdisplay, mode->vdisplay,
+			fb_color->red, fb_color->green, fb_color->blue);
+	for (int i = 0; i < rect_cnt; i++) {
+		const rectangle_t *rect = &rects[i];
+		igt_paint_color(cr,
+				rect->x, rect->y, 64, 64,
+				rect->color.red,
+				rect->color.green,
+				rect->color.blue);
+	}
+
+	igt_put_cairo_ctx(cr);
+}
+
 static void
 test_grab_crc(data_t *data, igt_output_t *output, enum pipe pipe,
-	      color_t *fb_color, igt_crc_t *crc /* out */)
+	      color_t *fb_color, unsigned int flags, igt_crc_t *crc /* out */)
 {
 	struct igt_fb fb;
 	drmModeModeInfo *mode;
@@ -86,13 +135,24 @@ test_grab_crc(data_t *data, igt_output_t *output, enum pipe pipe,
 	primary = igt_output_get_plane(output, 0);
 
 	mode = igt_output_get_mode(output);
-	igt_create_color_fb(data->drm_fd, mode->hdisplay, mode->vdisplay,
-			    DRM_FORMAT_XRGB8888,
-			    LOCAL_DRM_FORMAT_MOD_NONE,
-			    fb_color->red, fb_color->green, fb_color->blue,
-			    &fb);
-	igt_plane_set_fb(primary, &fb);
+	if (flags & TEST_POSITION_PARTIALLY_COVERED) {
+		static const rectangle_t rects[] = {
+			{ .x = 100, .y = 100, .color = { 0.0, 0.0, 0.0 }},
+			{ .x = 132, .y = 132, .color = { 0.0, 1.0, 0.0 }},
+		};
 
+		create_fb_for_mode(data, mode, fb_color,
+				   rects, ARRAY_SIZE(rects), &fb);
+	} else {
+		igt_assert_fd(igt_create_color_fb(data->drm_fd,
+						  mode->hdisplay, mode->vdisplay,
+						  DRM_FORMAT_XRGB8888,
+						  LOCAL_DRM_FORMAT_MOD_NONE,
+						  fb_color->red, fb_color->green, fb_color->blue,
+						  &fb));
+	}
+
+	igt_plane_set_fb(primary, &fb);
 	ret = igt_display_try_commit2(&data->display, COMMIT_LEGACY);
 	igt_skip_on(ret != 0);
 
@@ -104,19 +164,24 @@ test_grab_crc(data_t *data, igt_output_t *output, enum pipe pipe,
 	igt_remove_fb(data->drm_fd, &fb);
 
 	crc_str = igt_crc_to_string(crc);
-	igt_debug("CRC for a (%.02f,%.02f,%.02f) fb: %s\n", fb_color->red,
-		  fb_color->green, fb_color->blue, crc_str);
+	igt_debug("CRC for a %s covered (%.02f,%.02f,%.02f) fb: %s\n",
+		  flags & TEST_POSITION_PARTIALLY_COVERED ? "partially" : "fully",
+		  fb_color->red, fb_color->green, fb_color->blue,
+		  crc_str);
 	free(crc_str);
 }
 
 /*
  * Plane position test.
- *   - We start by grabbing a reference CRC of a full green fb being scanned
- *     out on the primary plane
+ *   - For testing positions that fully cover our hole, we start by grabbing a
+ *     reference CRC of a full green fb being scanned out on the primary plane.
+ *     For testing positions that only partially cover our hole, we instead use
+ *     a full green fb with a partially covered black rectangle.
  *   - Then we scannout 2 planes:
  *      - the primary plane uses a green fb with a black rectangle
  *      - a plane, on top of the primary plane, with a green fb that is set-up
- *        to cover the black rectangle of the primary plane fb
+ *        to fully or partially cover the black rectangle of the primary plane
+ *        fb
  *     The resulting CRC should be identical to the reference CRC
  */
 
@@ -125,38 +190,6 @@ typedef struct {
 	igt_crc_t reference_crc;
 } test_position_t;
 
-/*
- * create a green fb with a black rectangle at (rect_x,rect_y) and of size
- * (rect_w,rect_h)
- */
-static void
-create_fb_for_mode__position(data_t *data, drmModeModeInfo *mode,
-			     double rect_x, double rect_y,
-			     double rect_w, double rect_h,
-			     struct igt_fb *fb /* out */)
-{
-	unsigned int fb_id;
-	cairo_t *cr;
-
-	fb_id = igt_create_fb(data->drm_fd,
-				  mode->hdisplay, mode->vdisplay,
-				  DRM_FORMAT_XRGB8888,
-				  LOCAL_DRM_FORMAT_MOD_NONE,
-				  fb);
-	igt_assert(fb_id);
-
-	cr = igt_get_cairo_ctx(data->drm_fd, fb);
-	igt_paint_color(cr, 0, 0, mode->hdisplay, mode->vdisplay,
-			    0.0, 1.0, 0.0);
-	igt_paint_color(cr, rect_x, rect_y, rect_w, rect_h, 0.0, 0.0, 0.0);
-	igt_put_cairo_ctx(data->drm_fd, fb, cr);
-}
-
-enum {
-	TEST_POSITION_FULLY_COVERED = 1 << 0,
-	TEST_DPMS = 1 << 1,
-};
-
 static void
 test_plane_position_with_output(data_t *data,
 				enum pipe pipe,
@@ -165,6 +198,7 @@ test_plane_position_with_output(data_t *data,
 				igt_crc_t *reference_crc,
 				unsigned int flags)
 {
+	rectangle_t rect = { .x = 100, .y = 100, .color = { 0.0, 0.0, 0.0 }};
 	igt_plane_t *primary, *sprite;
 	struct igt_fb primary_fb, sprite_fb;
 	drmModeModeInfo *mode;
@@ -179,26 +213,26 @@ test_plane_position_with_output(data_t *data,
 	primary = igt_output_get_plane_type(output, DRM_PLANE_TYPE_PRIMARY);
 	sprite = igt_output_get_plane(output, plane);
 
-	create_fb_for_mode__position(data, mode, 100, 100, 64, 64,
-				     &primary_fb);
+	create_fb_for_mode(data, mode, &green, &rect, 1, &primary_fb);
 	igt_plane_set_fb(primary, &primary_fb);
 
 	igt_create_color_fb(data->drm_fd,
-				64, 64, /* width, height */
-				DRM_FORMAT_XRGB8888,
-				LOCAL_DRM_FORMAT_MOD_NONE,
-				0.0, 1.0, 0.0,
-				&sprite_fb);
+			    64, 64, /* width, height */
+			    DRM_FORMAT_XRGB8888,
+			    LOCAL_DRM_FORMAT_MOD_NONE,
+			    0.0, 1.0, 0.0,
+			    &sprite_fb);
 	igt_plane_set_fb(sprite, &sprite_fb);
 
-	if (flags & TEST_POSITION_FULLY_COVERED)
-		igt_plane_set_position(sprite, 100, 100);
-	else
+	if (flags & TEST_POSITION_PARTIALLY_COVERED)
 		igt_plane_set_position(sprite, 132, 132);
+	else
+		igt_plane_set_position(sprite, 100, 100);
 
 	igt_display_commit(&data->display);
 
 	igt_pipe_crc_collect_crc(data->pipe_crc, &crc);
+	igt_assert_crc_equal(reference_crc, &crc);
 
 	if (flags & TEST_DPMS) {
 		kmstest_set_connector_dpms(data->drm_fd,
@@ -211,12 +245,6 @@ test_plane_position_with_output(data_t *data,
 
 	igt_pipe_crc_collect_crc(data->pipe_crc, &crc2);
 
-	if (flags & TEST_POSITION_FULLY_COVERED)
-		igt_assert_crc_equal(reference_crc, &crc);
-	else {
-		;/* FIXME: missing reference CRCs */
-	}
-
 	igt_assert_crc_equal(&crc, &crc2);
 
 	igt_plane_set_fb(primary, NULL);
@@ -237,8 +265,7 @@ test_plane_position(data_t *data, enum pipe pipe, unsigned int flags)
 	igt_require(output);
 
 	test_init(data, pipe);
-
-	test_grab_crc(data, output, pipe, &green, &reference_crc);
+	test_grab_crc(data, output, pipe, &green, flags, &reference_crc);
 
 	for (int plane = 1; plane < n_planes; plane++)
 		test_plane_position_with_output(data, pipe, plane,
@@ -284,21 +311,14 @@ create_fb_for_mode__panning(data_t *data, drmModeModeInfo *mode,
 			mode->hdisplay, mode->vdisplay,
 			0.0, 0.0, 1.0);
 
-	igt_put_cairo_ctx(data->drm_fd, fb, cr);
+	igt_put_cairo_ctx(cr);
 }
 
-enum {
-	TEST_PANNING_TOP_LEFT	  = 1 << 0,
-	TEST_PANNING_BOTTOM_RIGHT = 1 << 1,
-	TEST_SUSPEND_RESUME	  = 1 << 2,
-};
-
 static void
 test_plane_panning_with_output(data_t *data,
 			       enum pipe pipe,
-			       int plane,
 			       igt_output_t *output,
-			       igt_crc_t *red_crc, igt_crc_t *blue_crc,
+			       igt_crc_t *ref_crc,
 			       unsigned int flags)
 {
 	igt_plane_t *primary;
@@ -306,8 +326,8 @@ test_plane_panning_with_output(data_t *data,
 	drmModeModeInfo *mode;
 	igt_crc_t crc;
 
-	igt_info("Testing connector %s using pipe %s plane %d\n",
-		 igt_output_name(output), kmstest_pipe_name(pipe), plane);
+	igt_info("Testing connector %s using pipe %s\n",
+		 igt_output_name(output), kmstest_pipe_name(pipe));
 
 	igt_output_set_pipe(output, pipe);
 
@@ -330,10 +350,7 @@ test_plane_panning_with_output(data_t *data,
 
 	igt_pipe_crc_collect_crc(data->pipe_crc, &crc);
 
-	if (flags & TEST_PANNING_TOP_LEFT)
-		igt_assert_crc_equal(red_crc, &crc);
-	else
-		igt_assert_crc_equal(blue_crc, &crc);
+	igt_assert_crc_equal(ref_crc, &crc);
 
 	igt_plane_set_fb(primary, NULL);
 
@@ -345,22 +362,20 @@ test_plane_panning_with_output(data_t *data,
 static void
 test_plane_panning(data_t *data, enum pipe pipe, unsigned int flags)
 {
-	int n_planes = data->display.pipes[pipe].n_planes;
 	igt_output_t *output;
-	igt_crc_t red_crc;
-	igt_crc_t blue_crc;
+	igt_crc_t ref_crc;
 
 	output = igt_get_single_output_for_pipe(&data->display, pipe);
 	igt_require(output);
 
 	test_init(data, pipe);
 
-	test_grab_crc(data, output, pipe, &red, &red_crc);
-	test_grab_crc(data, output, pipe, &blue, &blue_crc);
+	if (flags & TEST_PANNING_TOP_LEFT)
+		test_grab_crc(data, output, pipe, &red, flags, &ref_crc);
+	else
+		test_grab_crc(data, output, pipe, &blue, flags, &ref_crc);
 
-	for (int plane = 1; plane < n_planes; plane++)
-		test_plane_panning_with_output(data, pipe, plane, output,
-					       &red_crc, &blue_crc, flags);
+	test_plane_panning_with_output(data, pipe, output, &ref_crc, flags);
 
 	test_fini(data);
 }
@@ -440,61 +455,72 @@ static bool set_c8_legacy_lut(data_t *data, enum pipe pipe,
 	return true;
 }
 
+static void draw_entire_color_array(data_t *data, cairo_t *cr, uint32_t format,
+				    struct igt_fb *fb)
+{
+	const int color_amount = ARRAY_SIZE(colors_extended);
+	const int x = format == DRM_FORMAT_XRGB8888 ? 0 : data->crop;
+
+	for (int n = 0; n < color_amount; n++) {
+		int y = (fb->height - x * 2) * n / color_amount + x;
+
+		igt_paint_color(cr, x, y,
+				fb->width - x * 2,
+				(fb->height - x * 2) / color_amount,
+				colors_extended[n].red,
+				colors_extended[n].green,
+				colors_extended[n].blue);
+	}
+}
+
 static void prepare_format_color(data_t *data, enum pipe pipe,
 				 igt_plane_t *plane,
 				 uint32_t format, uint64_t modifier,
 				 int width, int height,
 				 enum igt_color_encoding color_encoding,
 				 enum igt_color_range color_range,
-				 const color_t *c, struct igt_fb *fb)
+				 const color_t *c, struct igt_fb *fb,
+				 bool packed)
 {
 	cairo_t *cr;
+	const int localcrop = format == DRM_FORMAT_XRGB8888 ? 0 : data->crop;
 
-	if (data->crop == 0 || format == DRM_FORMAT_XRGB8888) {
-		igt_create_fb_with_bo_size(data->drm_fd, width, height,
-					   format, modifier, color_encoding,
-					   color_range, fb, 0, 0);
+	igt_create_fb_with_bo_size(data->drm_fd,
+				   width + localcrop * 2,
+				   height + localcrop * 2,
+				   format, modifier, color_encoding,
+				   color_range, fb, 0, 0);
 
-		cr = igt_get_cairo_ctx(data->drm_fd, fb);
-
-		igt_paint_color(cr, 0, 0, width, height,
-				c->red, c->green, c->blue);
-
-		igt_put_cairo_ctx(data->drm_fd, fb, cr);
-	} else {
-		igt_create_fb_with_bo_size(data->drm_fd,
-					   width + data->crop * 2,
-					   height + data->crop * 2,
-					   format, modifier, color_encoding,
-					   color_range, fb, 0, 0);
-
-		/*
-		 * paint border in inverted color, then visible area in middle
-		 * with correct color for clamping test
-		 */
-		cr = igt_get_cairo_ctx(data->drm_fd, fb);
+	cr = igt_get_cairo_ctx(data->drm_fd, fb);
 
+	/*
+	 * paint border in inverted color, then visible area in middle
+	 * with correct color for clamping test
+	 */
+	if (localcrop)
 		igt_paint_color(cr, 0, 0,
-				width + data->crop * 2,
-				height + data->crop * 2,
+				width + localcrop * 2,
+				height + localcrop * 2,
 				1.0f - c->red,
 				1.0f - c->green,
 				1.0f - c->blue);
 
-		igt_paint_color(cr, data->crop, data->crop,
+
+	if (packed)
+		draw_entire_color_array(data, cr, format, fb);
+	else
+		igt_paint_color(cr, localcrop, localcrop,
 				width, height,
 				c->red, c->green, c->blue);
 
-		igt_put_cairo_ctx(data->drm_fd, fb, cr);
-	}
-
+	igt_put_cairo_ctx(cr);
 	igt_plane_set_fb(plane, fb);
 
 	/*
-	 * if clamping test. DRM_FORMAT_XRGB8888 is used for reference color.
+	 * if clamping test.
 	 */
-	if (data->crop != 0 && format != DRM_FORMAT_XRGB8888) {
-		igt_fb_set_position(fb, plane, data->crop, data->crop);
+	if (localcrop) {
+		igt_fb_set_position(fb, plane, localcrop, localcrop);
 		igt_fb_set_size(fb, plane, width, height);
 		igt_plane_set_size(plane, width, height);
 	}
@@ -523,29 +549,55 @@ static void capture_crc(data_t *data, unsigned int vblank, igt_crc_t *crc)
 {
 	igt_pipe_crc_get_for_frame(data->drm_fd, data->pipe_crc, vblank, crc);
 
-	igt_fail_on_f(crc->has_valid_frame && crc->frame != vblank,
+	igt_fail_on_f(!igt_skip_crc_compare &&
+		      crc->has_valid_frame && crc->frame != vblank,
 		      "Got CRC for the wrong frame (got %u, expected %u). CRC buffer overflow?\n",
 		      crc->frame, vblank);
 }
 
-static void capture_format_crcs(data_t *data, enum pipe pipe,
-				igt_plane_t *plane,
-				uint32_t format, uint64_t modifier,
-				int width, int height,
-				enum igt_color_encoding encoding,
-				enum igt_color_range range,
-				igt_crc_t crc[], struct igt_fb *fb)
+static void capture_format_crcs_packed(data_t *data, enum pipe pipe,
+				       igt_plane_t *plane,
+				       uint32_t format, uint64_t modifier,
+				       int width, int height,
+				       enum igt_color_encoding encoding,
+				       enum igt_color_range range,
+				       igt_crc_t crc[], struct igt_fb *fb)
+{
+	struct igt_fb old_fb = *fb;
+	const color_t black = { 0.0f, 0.0f, 0.0f };
+
+	prepare_format_color(data, pipe, plane, format, modifier,
+			     width, height, encoding, range, &black, fb, true);
+
+	igt_display_commit2(&data->display, data->display.is_atomic ?
+			    COMMIT_ATOMIC : COMMIT_UNIVERSAL);
+
+	igt_remove_fb(data->drm_fd, &old_fb);
+
+	igt_pipe_crc_get_current(data->drm_fd, data->pipe_crc, &crc[0]);
+}
+
+static void capture_format_crcs_planar(data_t *data, enum pipe pipe,
+				       igt_plane_t *plane,
+				       uint32_t format, uint64_t modifier,
+				       int width, int height,
+				       enum igt_color_encoding encoding,
+				       enum igt_color_range range,
+				       igt_crc_t crc[], struct igt_fb *fb)
 {
 	unsigned int vblank[ARRAY_SIZE(colors_extended)];
 	struct drm_event_vblank ev;
 	int i;
 
+restart_round:
 	for (i = 0; i < data->num_colors; i++) {
 		const color_t *c = &data->colors[i];
 		struct igt_fb old_fb = *fb;
+		int ret;
 
 		prepare_format_color(data, pipe, plane, format, modifier,
-				     width, height, encoding, range, c, fb);
+				     width, height, encoding, range, c, fb,
+				     false);
 
 		if (data->display.is_atomic && i >= 1) {
 			igt_assert(read(data->drm_fd, &ev, sizeof(ev)) == sizeof(ev));
@@ -573,9 +625,22 @@ static void capture_format_crcs(data_t *data, enum pipe pipe,
 			 * to be prepared in parallel while the current fb
 			 * awaits to be latched.
 			 */
-			igt_display_commit_atomic(&data->display,
-						  DRM_MODE_ATOMIC_NONBLOCK |
-						  DRM_MODE_PAGE_FLIP_EVENT, NULL);
+			ret = igt_display_try_commit_atomic(&data->display,
+							    DRM_MODE_ATOMIC_NONBLOCK |
+							    DRM_MODE_PAGE_FLIP_EVENT, NULL);
+			if (ret) {
+				/*
+				 * there was needed modeset for pixel format.
+				 * modeset here happen only on first color of
+				 * given set so restart round as modeset will
+				 * mess up crc frame sequence.
+				 */
+				igt_display_commit_atomic(&data->display,
+							  DRM_MODE_ATOMIC_ALLOW_MODESET,
+							  NULL);
+				igt_remove_fb(data->drm_fd, &old_fb);
+				goto restart_round;
+			}
 		} else {
 			/*
 			 * Last moment to grab the previous crc
@@ -592,8 +657,12 @@ static void capture_format_crcs(data_t *data, enum pipe pipe,
 			igt_display_commit2(&data->display, COMMIT_UNIVERSAL);
 
 			/* setplane for the cursor does not block */
-			if (plane->type == DRM_PLANE_TYPE_CURSOR)
-				igt_wait_for_vblank(data->drm_fd, pipe);
+			if (plane->type == DRM_PLANE_TYPE_CURSOR) {
+				igt_display_t *display = &data->display;
+
+				igt_wait_for_vblank(data->drm_fd,
+						display->pipes[pipe].crtc_offset);
+			}
 		}
 
 		igt_remove_fb(data->drm_fd, &old_fb);
@@ -644,12 +713,20 @@ static bool test_format_plane_colors(data_t *data, enum pipe pipe,
 	unsigned int crc_mismatch_mask = 0;
 	int crc_mismatch_count = 0;
 	bool result = true;
-	int i;
+	int i, total_crcs = 1;
+	bool planar = igt_format_is_yuv_semiplanar(format);
 
-	capture_format_crcs(data, pipe, plane, format, modifier,
-			    width, height, encoding, range, crc, fb);
+	if (planar) {
+		capture_format_crcs_planar(data, pipe, plane, format, modifier,
+					   width, height, encoding, range, crc,
+					   fb);
+		total_crcs = data->num_colors;
+	} else
+		capture_format_crcs_packed(data, pipe, plane, format, modifier,
+					   width, height, encoding, range, crc,
+					   fb);
 
-	for (i = 0; i < data->num_colors; i++) {
+	for (i = 0; i < total_crcs; i++) {
 		if (!igt_check_crc_equal(&crc[i], &ref_crc[i])) {
 			crc_mismatch_count++;
 			crc_mismatch_mask |= (1 << i);
@@ -737,15 +814,26 @@ static bool test_format_plane_yuv(data_t *data, enum pipe pipe,
 	return result;
 }
 
+enum crc_set { PACKED_CRC_SET,
+	       PLANAR_CRC_SET,
+	       MAX_CRC_SET };
+
+struct format_mod {
+	uint64_t modifier;
+	uint32_t format;
+};
+
 static bool test_format_plane(data_t *data, enum pipe pipe,
-			      igt_output_t *output, igt_plane_t *plane)
+			      igt_output_t *output, igt_plane_t *plane, igt_fb_t *primary_fb)
 {
 	struct igt_fb fb = {};
+	struct igt_fb *clear_fb = plane->type == DRM_PLANE_TYPE_PRIMARY ? primary_fb : NULL;
 	drmModeModeInfo *mode;
-	uint32_t format, ref_format;
-	uint64_t modifier, ref_modifier;
 	uint64_t width, height;
-	igt_crc_t ref_crc[ARRAY_SIZE(colors_extended)];
+	igt_crc_t ref_crc[MAX_CRC_SET][ARRAY_SIZE(colors_extended)];
+	struct igt_vec tested_formats;
+	struct format_mod ref = {};
+	igt_crc_t* crcset;
 	bool result = true;
 
 	/*
@@ -754,12 +842,14 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 	if (data->crop != 0 && plane->type == DRM_PLANE_TYPE_CURSOR)
 		return true;
 
+	igt_vec_init(&tested_formats, sizeof(struct format_mod));
+
 	mode = igt_output_get_mode(output);
 	if (plane->type != DRM_PLANE_TYPE_CURSOR) {
 		width = mode->hdisplay;
 		height = mode->vdisplay;
-		ref_format = format = DRM_FORMAT_XRGB8888;
-		ref_modifier = modifier = DRM_FORMAT_MOD_NONE;
+		ref.format = DRM_FORMAT_XRGB8888;
+		ref.modifier = DRM_FORMAT_MOD_NONE;
 	} else {
 		if (!plane->drm_plane) {
 			igt_debug("Only legacy cursor ioctl supported, skipping cursor plane\n");
@@ -767,8 +857,8 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 		}
 		do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_WIDTH, &width));
 		do_or_die(drmGetCap(data->drm_fd, DRM_CAP_CURSOR_HEIGHT, &height));
-		ref_format = format = DRM_FORMAT_ARGB8888;
-		ref_modifier = modifier = DRM_FORMAT_MOD_NONE;
+		ref.format = DRM_FORMAT_ARGB8888;
+		ref.modifier = DRM_FORMAT_MOD_NONE;
 	}
 
 	igt_debug("Testing connector %s on %s plane %s.%u\n",
@@ -778,77 +868,109 @@ static bool test_format_plane(data_t *data, enum pipe pipe,
 	igt_pipe_crc_start(data->pipe_crc);
 
 	igt_info("Testing format " IGT_FORMAT_FMT " / modifier 0x%" PRIx64 " on %s.%u\n",
-		 IGT_FORMAT_ARGS(format), modifier,
+		 IGT_FORMAT_ARGS(ref.format), ref.modifier,
 		 kmstest_pipe_name(pipe), plane->index);
 
 	if (data->display.is_atomic) {
 		struct igt_fb test_fb;
 		int ret;
 
-		igt_create_fb(data->drm_fd, 64, 64, format,
-			      LOCAL_DRM_FORMAT_MOD_NONE, &test_fb);
+		igt_create_fb(data->drm_fd, 64, 64, ref.format,
+			      DRM_FORMAT_MOD_LINEAR, &test_fb);
 
 		igt_plane_set_fb(plane, &test_fb);
 
-		ret = igt_display_try_commit_atomic(&data->display, DRM_MODE_ATOMIC_TEST_ONLY, NULL);
-
+		ret = igt_display_try_commit_atomic(&data->display,
+						    DRM_MODE_ATOMIC_TEST_ONLY |
+						    DRM_MODE_ATOMIC_ALLOW_MODESET,
+						    NULL);
 		if (!ret) {
 			width = test_fb.width;
 			height = test_fb.height;
 		}
 
-		igt_plane_set_fb(plane, NULL);
+		igt_plane_set_fb(plane, clear_fb);
 
 		igt_remove_fb(data->drm_fd, &test_fb);
 	}
 
-	capture_format_crcs(data, pipe, plane, format, modifier,
-			    width, height, IGT_COLOR_YCBCR_BT709,
-			    IGT_COLOR_YCBCR_LIMITED_RANGE, ref_crc, &fb);
+	capture_format_crcs_packed(data, pipe, plane, ref.format, ref.modifier,
+				   width, height, IGT_COLOR_YCBCR_BT709,
+				   IGT_COLOR_YCBCR_LIMITED_RANGE,
+				   ref_crc[PACKED_CRC_SET], &fb);
+
+	capture_format_crcs_planar(data, pipe, plane, ref.format, ref.modifier,
+				   width, height, IGT_COLOR_YCBCR_BT709,
+				   IGT_COLOR_YCBCR_LIMITED_RANGE,
+				   ref_crc[PLANAR_CRC_SET], &fb);
 
 	/*
 	 * Make sure we have some difference between the colors. This
 	 * at least avoids claiming success when everything is just
 	 * black all the time (eg. if the plane is never even on).
 	 */
-	igt_require(num_unique_crcs(ref_crc, data->num_colors) > 1);
+	igt_require(num_unique_crcs(ref_crc[PLANAR_CRC_SET], data->num_colors) > 1);
 
 	for (int i = 0; i < plane->format_mod_count; i++) {
-		format = plane->formats[i];
-		modifier = plane->modifiers[i];
+		struct format_mod f = {
+			.format = plane->formats[i],
+			.modifier = plane->modifiers[i],
+		};
 
-		if (format == ref_format &&
-		    modifier == ref_modifier)
+		if (f.format == ref.format &&
+		    f.modifier == ref.modifier)
 			continue;
 
-		if (format == DRM_FORMAT_C8) {
+		/* test each format "class" only once in non-extended tests */
+		if (!data->extended && f.modifier != DRM_FORMAT_MOD_LINEAR) {
+			struct format_mod rf = {
+				.format = igt_reduce_format(f.format),
+				.modifier = f.modifier,
+			};
+
+			if (igt_vec_index(&tested_formats, &rf) >= 0) {
+				igt_info("Skipping format " IGT_FORMAT_FMT " / modifier 0x%" PRIx64 " on %s.%u\n",
+					 IGT_FORMAT_ARGS(f.format), f.modifier,
+					 kmstest_pipe_name(pipe), plane->index);
+				continue;
+			}
+
+			igt_vec_push(&tested_formats, &rf);
+		}
+
+		if (f.format == DRM_FORMAT_C8) {
 			if (!set_c8_legacy_lut(data, pipe, LUT_MASK))
 				continue;
 		} else {
-			if (!igt_fb_supported_format(format))
+			if (!igt_fb_supported_format(f.format))
 				continue;
 		}
 
-		if (igt_format_is_yuv(format))
+		crcset = ref_crc[(igt_format_is_yuv_semiplanar(f.format)
+				 ? PLANAR_CRC_SET : PACKED_CRC_SET)];
+
+		if (igt_format_is_yuv(f.format))
 			result &= test_format_plane_yuv(data, pipe, plane,
-							format, modifier,
+							f.format, f.modifier,
 							width, height,
-							ref_crc, &fb);
+							crcset, &fb);
 		else
 			result &= test_format_plane_rgb(data, pipe, plane,
-							format, modifier,
+							f.format, f.modifier,
 							width, height,
-							ref_crc, &fb);
+							crcset, &fb);
 
-		if (format == DRM_FORMAT_C8)
+		if (f.format == DRM_FORMAT_C8)
 			set_legacy_lut(data, pipe, LUT_MASK);
 	}
 
 	igt_pipe_crc_stop(data->pipe_crc);
 
-	igt_plane_set_fb(plane, NULL);
+	igt_plane_set_fb(plane, clear_fb);
 	igt_remove_fb(data->drm_fd, &fb);
 
+	igt_vec_fini(&tested_formats);
+
 	return result;
 }
 
@@ -919,7 +1041,7 @@ test_pixel_formats(data_t *data, enum pipe pipe)
 	for_each_plane_on_pipe(&data->display, pipe, plane) {
 		if (skip_plane(data, plane))
 			continue;
-		result &= test_format_plane(data, pipe, output, plane);
+		result &= test_format_plane(data, pipe, output, plane, &primary_fb);
 	}
 
 	test_fini(data);
@@ -939,14 +1061,16 @@ static void
 run_tests_for_pipe_plane(data_t *data, enum pipe pipe)
 {
 	igt_fixture {
-		igt_skip_on(pipe >= data->display.n_pipes);
+		igt_require_pipe(&data->display, pipe);
 		igt_require(data->display.pipes[pipe].n_planes > 0);
 	}
 
+	igt_describe("verify the pixel formats for given plane and pipe");
 	igt_subtest_f("pixel-format-pipe-%s-planes",
 		      kmstest_pipe_name(pipe))
 		test_pixel_formats(data, pipe);
 
+	igt_describe("verify the pixel formats for given plane and pipe with source clamping");
 	igt_subtest_f("pixel-format-pipe-%s-planes-source-clamping",
 		      kmstest_pipe_name(pipe)) {
 		data->crop = 4;
@@ -954,26 +1078,37 @@ run_tests_for_pipe_plane(data_t *data, enum pipe pipe)
 	}
 
 	data->crop = 0;
+	igt_describe("verify plane position using two planes to create a fully covered screen");
 	igt_subtest_f("plane-position-covered-pipe-%s-planes",
 		      kmstest_pipe_name(pipe))
-		test_plane_position(data, pipe, TEST_POSITION_FULLY_COVERED);
-
-	igt_subtest_f("plane-position-hole-pipe-%s-planes",
-		      kmstest_pipe_name(pipe))
 		test_plane_position(data, pipe, 0);
 
+	igt_describe("verify plane position using two planes to create a partially covered screen");
+	igt_subtest_f("plane-position-hole-pipe-%s-planes",
+		      kmstest_pipe_name(pipe))
+		test_plane_position(data, pipe,
+				    TEST_POSITION_PARTIALLY_COVERED);
+
+	igt_describe("verify plane position using two planes to create a partially covered screen and"
+		       "check for DPMS");
 	igt_subtest_f("plane-position-hole-dpms-pipe-%s-planes",
 		      kmstest_pipe_name(pipe))
-		test_plane_position(data, pipe, TEST_DPMS);
+		test_plane_position(data, pipe,
+				    TEST_POSITION_PARTIALLY_COVERED |
+				    TEST_DPMS);
 
+	igt_describe("verify plane panning at top-left position using primary plane");
 	igt_subtest_f("plane-panning-top-left-pipe-%s-planes",
 		      kmstest_pipe_name(pipe))
 		test_plane_panning(data, pipe, TEST_PANNING_TOP_LEFT);
 
+	igt_describe("verify plane panning at bottom-right position using primary plane");
 	igt_subtest_f("plane-panning-bottom-right-pipe-%s-planes",
 		      kmstest_pipe_name(pipe))
 		test_plane_panning(data, pipe, TEST_PANNING_BOTTOM_RIGHT);
 
+	igt_describe("verify plane panning at bottom-right position using primary plane and executes system"
+		       "suspend cycles");
 	igt_subtest_f("plane-panning-bottom-right-suspend-pipe-%s-planes",
 		      kmstest_pipe_name(pipe))
 		test_plane_panning(data, pipe, TEST_PANNING_BOTTOM_RIGHT |
