@@ -102,7 +102,7 @@ static void draw_color(igt_fb_t *fb, double r, double g, double b)
 
 	cairo_set_operator(cr, CAIRO_OPERATOR_SOURCE);
 	igt_paint_color(cr, 0, 0, fb->width, fb->height, r, g, b);
-	igt_put_cairo_ctx(fb->fd, fb, cr);
+	igt_put_cairo_ctx(cr);
 }
 
 /*
@@ -129,7 +129,7 @@ static void test_cursor_pos(data_t *data, igt_fb_t *pfb, igt_fb_t *ofb,
 		igt_paint_color(cr, or->x, or->y, or->w, or->h, 0.5, 0.5, 0.5);
 
 	igt_paint_color(cr, x, y, cw, ch, 1.0, 0.0, 1.0);
-	igt_put_cairo_ctx(pfb->fd, pfb, cr);
+	igt_put_cairo_ctx(cr);
 
 	igt_plane_set_fb(data->overlay, NULL);
 	igt_plane_set_fb(data->cursor, NULL);
@@ -302,7 +302,7 @@ static void test_cursor_viewport(data_t *data, int size, enum pipe pipe_id)
 igt_main
 {
 	static const int cursor_sizes[] = { 64, 128, 256 };
-	data_t data = { 0 };
+	data_t data = {};
 	enum pipe pipe;
 	int i;
 
@@ -320,14 +320,18 @@ igt_main
 		for (i = 0; i < ARRAY_SIZE(cursor_sizes); ++i) {
 			int size = cursor_sizes[i];
 
+			igt_describe("Tests atomic cursor positioning on primary plane and overlay plane");
 			igt_subtest_f("pipe-%s-overlay-size-%d",
 				      kmstest_pipe_name(pipe), size)
 				test_cursor_overlay(&data, size, pipe);
 
+			igt_describe("Tests atomic cursor positioning on primary plane");
 			igt_subtest_f("pipe-%s-primary-size-%d",
 				      kmstest_pipe_name(pipe), size)
 				test_cursor_primary(&data, size, pipe);
 
+			igt_describe("Tests atomic cursor positioning on primary plane and overlay plane"
+				"with buffer larger than viewport used for display");
 			igt_subtest_f("pipe-%s-viewport-size-%d",
 				      kmstest_pipe_name(pipe), size)
 				test_cursor_viewport(&data, size, pipe);
