@@ -158,7 +158,7 @@ static void create_ref_fb(data_t *data, uint64_t modifier,
 	cr = igt_get_cairo_ctx(data->drm_fd, fb);
 	blit(data, cr, &data->fb_primary, 0, 0);
 	blit(data, cr, &data->fb_plane[0], data->x, data->y);
-	igt_put_cairo_ctx(data->drm_fd, fb, cr);
+	igt_put_cairo_ctx(cr);
 }
 
 static unsigned
@@ -259,7 +259,7 @@ test_planes_on_pipe(data_t *data, uint64_t modifier)
 	igt_plane_t *plane;
 	unsigned tested = 0;
 
-	igt_skip_on(data->pipe >= data->display.n_pipes);
+	igt_require_pipe(&data->display, data->pipe);
 	igt_display_require_output_on_pipe(&data->display, data->pipe);
 	igt_skip_on(!igt_display_has_format_mod(&data->display,
 						DRM_FORMAT_XRGB8888, modifier));
@@ -301,15 +301,23 @@ igt_main
 
 	for_each_pipe_static(pipe) {
 		data.pipe = pipe;
+		igt_describe("Tests the visibility of the planes when switching between "
+			     "high and low resolution with tiling as none.");
 		igt_subtest_f("pipe-%s-tiling-none", kmstest_pipe_name(pipe))
 			test_planes_on_pipe(&data, LOCAL_DRM_FORMAT_MOD_NONE);
 
+		igt_describe("Tests the visibility of the planes when switching between "
+			     "high and low resolution with x-tiling.");
 		igt_subtest_f("pipe-%s-tiling-x", kmstest_pipe_name(pipe))
 			test_planes_on_pipe(&data, LOCAL_I915_FORMAT_MOD_X_TILED);
 
+		igt_describe("Tests the visibility of the planes when switching between "
+			     "high and low resolution with y-tiling.");
 		igt_subtest_f("pipe-%s-tiling-y", kmstest_pipe_name(pipe))
 			test_planes_on_pipe(&data, LOCAL_I915_FORMAT_MOD_Y_TILED);
 
+		igt_describe("Tests the visibility of the planes when switching between "
+			     "high and low resolution with yf-tiling.");
 		igt_subtest_f("pipe-%s-tiling-yf", kmstest_pipe_name(pipe))
 			test_planes_on_pipe(&data, LOCAL_I915_FORMAT_MOD_Yf_TILED);
 	}
