@@ -62,19 +62,6 @@ static int get_num_scalers(data_t* d, enum pipe pipe)
 		return 1;
 }
 
-static bool is_planar_yuv_format(uint32_t pixelformat)
-{
-	switch (pixelformat) {
-	case DRM_FORMAT_NV12:
-	case DRM_FORMAT_P010:
-	case DRM_FORMAT_P012:
-	case DRM_FORMAT_P016:
-		return true;
-	default:
-		return false;
-	}
-}
-
 static void cleanup_fbs(data_t *data)
 {
 	int i;
@@ -147,9 +134,7 @@ static void check_scaling_pipe_plane_rot(data_t *d, igt_plane_t *plane,
 	mode = igt_output_get_mode(output);
 
 	/* create buffer in the range of  min and max source side limit.*/
-	width = height = 8;
-	if (is_i915_device(d->drm_fd) && is_planar_yuv_format(pixel_format))
-		width = height = 16;
+	width = height = 20;
 	igt_create_color_fb(display->drm_fd, width, height,
 		       pixel_format, tiling, 0.0, 1.0, 0.0, &d->fb[0]);
 	igt_plane_set_fb(plane, &d->fb[0]);
@@ -245,6 +230,18 @@ static bool test_format(data_t *data,
 	return true;
 }
 
+static bool test_pipe_iteration(data_t *data, enum pipe pipe, int iteration)
+{
+	if (!is_i915_device(data->drm_fd) ||
+	    data->extended)
+		return true;
+
+	if ((pipe > PIPE_B) && (iteration >= 2))
+		return false;
+
+	return true;
+}
+
 static void test_scaler_with_rotation_pipe(data_t *d, enum pipe pipe,
 					   igt_output_t *output)
 {
@@ -253,6 +250,8 @@ static void test_scaler_with_rotation_pipe(data_t *d, enum pipe pipe,
 	uint64_t tiling = is_i915_device(d->drm_fd) ?
 		LOCAL_I915_FORMAT_MOD_Y_TILED : LOCAL_DRM_FORMAT_MOD_NONE;
 
+	igt_require(get_num_scalers(d, pipe) > 0);
+
 	igt_output_set_pipe(output, pipe);
 	for_each_plane_on_pipe(display, pipe, plane) {
 		if (plane->type == DRM_PLANE_TYPE_CURSOR)
@@ -293,6 +292,8 @@ static void test_scaler_with_pixel_format_pipe(data_t *d, enum pipe pipe, igt_ou
 	igt_display_t *display = &d->display;
 	igt_plane_t *plane;
 
+	igt_require(get_num_scalers(d, pipe) > 0);
+
 	igt_output_set_pipe(output, pipe);
 
 	for_each_plane_on_pipe(display, pipe, plane) {
@@ -366,6 +367,8 @@ test_plane_scaling_on_pipe(data_t *d, enum pipe pipe, igt_output_t *output)
 	uint64_t tiling = is_i915_device(display->drm_fd) ?
 		LOCAL_I915_FORMAT_MOD_X_TILED : LOCAL_DRM_FORMAT_MOD_NONE;
 
+	igt_require(get_num_scalers(d, pipe) > 0);
+
 	igt_skip_on(!igt_display_has_format_mod(display, DRM_FORMAT_XRGB8888,
 						tiling));
 
@@ -492,19 +495,8 @@ test_plane_scaling_on_pipe(data_t *d, enum pipe pipe, igt_output_t *output)
 }
 
 static void
-__test_scaler_with_clipping_clamping_scenario(data_t *d, drmModeModeInfo *mode,
-					      uint32_t f1, uint32_t f2)
+__test_scaler_with_clipping_clamping_scenario(data_t *d, drmModeModeInfo *mode)
 {
-	cleanup_fbs(d);
-
-	igt_create_pattern_fb(d->drm_fd,
-			      mode->hdisplay, mode->vdisplay, f1,
-			      LOCAL_I915_FORMAT_MOD_X_TILED, &d->fb[1]);
-
-	igt_create_pattern_fb(d->drm_fd,
-			      mode->hdisplay, mode->vdisplay, f2,
-			      LOCAL_I915_FORMAT_MOD_Y_TILED, &d->fb[2]);
-
 	igt_plane_set_fb(d->plane1, &d->fb[1]);
 	igt_plane_set_fb(d->plane2, &d->fb[2]);
 
@@ -552,12 +544,19 @@ test_scaler_with_clipping_clamping_scenario(data_t *d, enum pipe pipe, igt_outpu
 		unsigned f1 = d->plane1->drm_plane->formats[i];
 		struct igt_vec tested_formats2;
 
+		if (!test_pipe_iteration(d, pipe, i))
+			continue;
+
 		if (!test_format(d, &tested_formats1, f1) ||
 		    !can_scale(d, f1))
 			continue;
 
 		igt_vec_init(&tested_formats2, sizeof(uint32_t));
 
+		igt_create_pattern_fb(d->drm_fd,
+				      mode->hdisplay, mode->vdisplay, f1,
+				      LOCAL_I915_FORMAT_MOD_X_TILED, &d->fb[1]);
+
 		for (int j = 0; j < d->plane2->drm_plane->count_formats; j++) {
 			unsigned f2 = d->plane2->drm_plane->formats[j];
 
@@ -565,8 +564,15 @@ test_scaler_with_clipping_clamping_scenario(data_t *d, enum pipe pipe, igt_outpu
 			    !can_scale(d, f2))
 				continue;
 
-			__test_scaler_with_clipping_clamping_scenario(d, mode, f1, f2);
+			igt_create_pattern_fb(d->drm_fd,
+					      mode->hdisplay, mode->vdisplay, f2,
+					      LOCAL_I915_FORMAT_MOD_Y_TILED,
+					      &d->fb[2]);
+
+			__test_scaler_with_clipping_clamping_scenario(d, mode);
+			igt_remove_fb(d->drm_fd, &d->fb[2]);
 		}
+		igt_remove_fb(d->drm_fd, &d->fb[1]);
 
 		igt_vec_fini(&tested_formats2);
 	}
@@ -700,32 +706,39 @@ igt_main_args("", long_opts, help_str, opt_handler, &data)
 		igt_require(data.display.is_atomic);
 	}
 
-	for_each_pipe_static(pipe) igt_subtest_group {
+	igt_subtest_group {
 		igt_output_t *output;
 
-		igt_fixture {
-			igt_display_require_output_on_pipe(&data.display, pipe);
-
-			igt_require(get_num_scalers(&data, pipe) > 0);
+		igt_describe("Tests plane scaling per pipe.");
+		igt_subtest_with_dynamic("plane-scaling") {
+			for_each_pipe_with_single_output(&data.display, pipe, output)
+				igt_dynamic_f("pipe-%s-plane-scaling", kmstest_pipe_name(pipe))
+					test_plane_scaling_on_pipe(&data, pipe, output);
 		}
 
-		igt_subtest_f("pipe-%s-plane-scaling", kmstest_pipe_name(pipe))
-			for_each_valid_output_on_pipe(&data.display, pipe, output)
-				test_plane_scaling_on_pipe(&data, pipe, output);
+		igt_describe("Tests scaling with pixel formats.");
+		igt_subtest_with_dynamic("scaler-with-pixel-format") {
+			for_each_pipe_with_single_output(&data.display, pipe, output)
+				igt_dynamic_f("pipe-%s-scaler-with-pixel-format", kmstest_pipe_name(pipe))
+					test_scaler_with_pixel_format_pipe(&data, pipe, output);
+		}
 
-		igt_subtest_f("pipe-%s-scaler-with-pixel-format", kmstest_pipe_name(pipe))
-			for_each_valid_output_on_pipe(&data.display, pipe, output)
-				test_scaler_with_pixel_format_pipe(&data, pipe, output);
+		igt_describe("Tests scaling with tiling rotation.");
+		igt_subtest_with_dynamic("scaler-with-rotation") {
+			for_each_pipe_with_single_output(&data.display, pipe, output)
+				igt_dynamic_f("pipe-%s-scaler-with-rotation", kmstest_pipe_name(pipe))
+					test_scaler_with_rotation_pipe(&data, pipe, output);
+		}
 
-		igt_subtest_f("pipe-%s-scaler-with-rotation", kmstest_pipe_name(pipe))
-			for_each_valid_output_on_pipe(&data.display, pipe, output)
-				test_scaler_with_rotation_pipe(&data, pipe, output);
-
-		igt_subtest_f("pipe-%s-scaler-with-clipping-clamping", kmstest_pipe_name(pipe))
-			for_each_valid_output_on_pipe(&data.display, pipe, output)
-				test_scaler_with_clipping_clamping_scenario(&data, pipe, output);
+		igt_describe("Tests scaling with clipping and clamping.");
+		igt_subtest_with_dynamic("scaler-with-clipping-clamping") {
+			for_each_pipe_with_single_output(&data.display, pipe, output)
+				igt_dynamic_f("pipe-%s-scaler-with-clipping-clamping", kmstest_pipe_name(pipe))
+					test_scaler_with_clipping_clamping_scenario(&data, pipe, output);
+		}
 	}
 
+	igt_describe("Tests scaling with multi-pipe scenario.");
 	igt_subtest_f("2x-scaler-multi-pipe")
 		test_scaler_with_multi_pipe_plane(&data);
 
